Package: topics
Type: Package
Title: Creating and Significance Testing Language Features for
        Visualisation
Version: 1.0
Authors@R: c(
    person("Leon", "Ackermann", 
    role = c("aut"), 
      comment = c(ORCID = "0009-0008-8583-8748"),
    ),
        person("Zhuojun", "Gu", 
        role = c("aut"), 
      comment = c(ORCID = "0009-0000-1610-4830"),
    ),
    person("Oscar", "Kjell", , "oscar.kjell@psy.lu.se", 
    role = c("aut", "cre"), 
      comment = c(ORCID = "0000-0002-2728-6278"))
    )
Description: Implements differential language analysis with statistical tests and offers various language visualization techniques for n-grams and topics. It also supports the 'text' package. For more information, visit <https://r-topics.org/> and <https://www.r-text.org/>.
License: GPL-3
URL: https://r-topics.org/
BugReports: https://github.com/theharmonylab/topics/issues
Encoding: UTF-8
Archs: x64
SystemRequirements: Python (>= 3.6.0)
LazyData: true
BuildVignettes: true
Imports: textmineR, quanteda, ggplot2, dplyr, ggwordcloud, tibble,
        methods, readr, stopwords, Matrix, stringr, stringi, rlang,
        tidyr, purrr, data.table, ggforce, patchwork, utils
RoxygenNote: 7.3.3
Suggests: text, rJava, mallet, glmnet, knitr, rmarkdown, testthat,
        covr, vdiffr, httr, arrow
VignetteBuilder: knitr
Depends: R (>= 4.00)
NeedsCompilation: no
Packaged: 2026-06-13 05:21:05 UTC; oscarkjell
Author: Leon Ackermann [aut] (ORCID: <https://orcid.org/0009-0008-8583-8748>),
  Zhuojun Gu [aut] (ORCID: <https://orcid.org/0009-0000-1610-4830>),
  Oscar Kjell [aut, cre] (ORCID: <https://orcid.org/0000-0002-2728-6278>)
Maintainer: Oscar Kjell <oscar.kjell@psy.lu.se>
Repository: CRAN
Date/Publication: 2026-06-13 16:30:02 UTC
Built: R 4.4.3; ; 2026-06-13 18:47:20 UTC; unix

# Generated by roxygen2: do not edit by hand

export(topicsAssess)
export(topicsClassify)
export(topicsDtm)
export(topicsDtmEval)
export(topicsGrams)
export(topicsGridLegend)
export(topicsModel)
export(topicsPlot)
export(topicsPlotOverview)
export(topicsPredict)
export(topicsPreds)
export(topicsScatterLegend)
export(topicsTest)
export(topicsTutorialData)
importFrom(Matrix,colSums)
importFrom(Matrix,t)
importFrom(data.table,":=")
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,contains)
importFrom(dplyr,desc)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_modify)
importFrom(dplyr,if_else)
importFrom(dplyr,join_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,slice_head)
importFrom(dplyr,summarise)
importFrom(dplyr,top_n)
importFrom(dplyr,ungroup)
importFrom(ggforce,geom_circle)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggsave)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_gradient)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_identity)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_size_area)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(ggwordcloud,geom_text_wordcloud)
importFrom(patchwork,plot_layout)
importFrom(patchwork,plot_spacer)
importFrom(patchwork,wrap_plots)
importFrom(purrr,map_dbl)
importFrom(quanteda,dfm)
importFrom(quanteda,dfm_select)
importFrom(quanteda,featnames)
importFrom(quanteda,quanteda_options)
importFrom(quanteda,tokens)
importFrom(quanteda,tokens_ngrams)
importFrom(quanteda,tokens_remove)
importFrom(readr,write_csv)
importFrom(rlang,"!!")
importFrom(rlang,sym)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,reorder)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stopwords,stopwords)
importFrom(stringr,str_count)
importFrom(stringr,str_detect)
importFrom(stringr,str_split)
importFrom(textmineR,CalcGamma)
importFrom(textmineR,CreateDtm)
importFrom(textmineR,Dtm2Docs)
importFrom(tibble,as_tibble)
importFrom(tibble,is_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,gather)
importFrom(tidyr,separate)
importFrom(utils,combn)
importFrom(utils,packageVersion)
importFrom(utils,tail)

<!-- README.md is generated from README.Rmd. Please edit that file -->

<!-- badges: start -->

[![CRAN
Status](https://www.r-pkg.org/badges/version/topics)](https://CRAN.R-project.org/package=topics)
[![DOI](https://zenodo.org/badge/785738351.svg)](https://zenodo.org/doi/10.5281/zenodo.11165377)
[![Github build
status](https://github.com/theharmonylab/topics/workflows/R-CMD-check/badge.svg)](https://github.com/theharmonylab/topics/actions)
[![Project Status:
Active](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![Lifecycle:
maturing](https://img.shields.io/badge/lifecycle-maturing-blue.svg)](https://lifecycle.r-lib.org/articles/stages.html#maturing-1)
[![codecov](https://codecov.io/gh/theharmonylab/topics/graph/badge.svg?token=7ZTWBNIVCX)](https://app.codecov.io/gh/theharmonylab/topics/)
[![CRAN
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/topics)](https://CRAN.R-project.org/package=topics)
<!-- badges: end -->

# topics <a href="https://r-topics.org"><img src="man/figures/logo.png" align="right" height="138" alt="topics website" /></a>

## Overview

An R-package for analyzing natural language implementing Differential
Language Analysis using words, phrases and topics. <br> <br> Check out
our tutorial paper: [Multiple Methods for Visualizing Human Language: A
Tutorial for Social and Behavioural
Scientists](https://osf.io/preprints/psyarxiv/nxfvr_v1/). If you use the
topics package, please cite this tutorial in your work. <br> <br> The
`topics` package is part of the *R Language Analysis Suite*, including
`talk`, `text` and `topics`.

- [`talk`](https://www.r-talk.org/) transforms voice recordings into
  text, audio features, or embeddings.<br> <br>
- [`text`](https://www.r-text.org/) provides many language tasks such as
  converting digital text into word embeddings.<br> <br> `talk` and
  `text` offer access to Large Language Models from Hugging Face.<br>
  <br>
- [`topics`](https://www.r-topics.org/) visualizes language patterns
  into words, phrases or topics to generate psychological insights. <br>
  The `topics` package supports the `text` package in analysing and
  visualizing topics from BERTtopics.<br> <br>

<img src="man/figures/talk_text_topics.svg" style="width:50.0%" />

<br> When using the `topics` package, please cite:

Ackermann L., Zhuojun G. & Kjell O.N.E. (2024). An R-package for
visualizing text in topics. <https://github.com/theharmonylab/topics>.
`DOI:zenodo.org/records/11165378`.

> ### 📣 Online Workshop: *Analysing Human Language using R*
>
> **August 11–13, 2026** — three half-day sessions (2:00–5:00 pm CEST /
> 8:00–11:00 am ET)
>
> Learn the full *R Language Analysis Suite* —
> [`talk`](https://www.r-talk.org/), [`text`](https://www.r-text.org/),
> and [`topics`](https://www.r-topics.org/) — with Oscar Kjell.
>
> **[Read more & register →](https://smart-workshops.com/lang-r-info)**

## Differential Language Analysis with Topics and N-grams

The `topics` pipeline is designed for a seamless transition from raw
text to statistically grounded visualizations. It is composed of the
following steps:

**1. Data Preprocessing** Transform raw text into a Document-Term Matrix
(DTM) or extract n-grams. This step handles cleaning, including the
removal of stopwords and punctuation, to prepare data for modeling or
frequency analysis.

**2. Model Training** For topic modeling, an LDA (Latent Dirichlet
Allocation) model is trained on the DTM. Users can specify the number of
topics and iterations to optimize the thematic representation of the
corpus.

**3. Model Inference** The model inference step uses the trained LDA
model to infer the topic-term distribution across documents, converting
qualitative text into quantitative topic loadings.

**4. Statistical Analysis** Perform Differential Language Analysis (DLA)
using `topicsTest()`. The analysis now supports: \* **Automatic
Detection:** Intelligent per-variable method detection (e.g.,
automatically applying logistic regression for binary factors and linear
regression for continuous data). \* **Multi-Element Analysis:**
Statistically test both LDA topics and n-grams. \* **Rigorous
Controls:** Support for control variables and various p-value adjustment
methods for multiple comparisons (e.g., FDR, Bonferroni, Holm).

**5. Visualization** Generate publication-ready visualizations of your
results: \* **Wordclouds:** Create clouds of significant topics where
word size reflects the contribution to the theme. \* **N-gram Plots:**
Directly visualize the relationship between specific phrases and your
variables of interest.

<img src="man/figures/one_dim.png" style="width:75.0%"
alt="One-dimensional plots based on words and phrases (top) and LDA topics (bottom)." />
<br> <br> <br>

<img src="man/figures/two_dim.png" style="width:75.0%"
alt="A two-dimensional plot showing topics related to depression versus worry responses (x-axis) and low versus high depression severity (y-axis)." />
<br> <br>

#### func topic testing ####
# p.adjust https://www.rdocumentation.org/packages/stats/versions/3.6.2/topics/p.adjust

#' The function for topic testing
#' @param topic_loadings (tibble) The predicted loadings of topics including the grouping variable.
#' @param grouping_variable (tibble) The variable for grouping
#' @param topic_terms (R_obj) The object from model$summary in textmineR package vignette topic_modeling
#' @param split (string) How to split the CONTINUOUS test_values for testing
#' @param n_min_max (integer) If split = "min_max", the number of records to test per group.
#' @param multiple_comparison (string) The p-correction method
#' @return Results
#' @importFrom dplyr contains select everything bind_cols bind_rows right_join join_by rename_with mutate across
#' @importFrom tibble is_tibble as_tibble
#' @importFrom stats  complete.cases sd lm glm as.formula
#' @noRd
topic_test <- function(
    topic_terms,
    topics_loadings,
    x_y_axis1,
    controls,
    test_method,
    split,
    n_min_max = 20,
    multiple_comparison
) {
  
  # Format Checker
  if (!tibble::is_tibble(topics_loadings)) {
    stop("Parameter `topics_loadings` must be a tibble.")
  }

  if (test_method %in% c("linear_regression", "logistic_regression")) {
    
    # Get number of topics automatically
    num_topics <- ncol(topics_loadings)
    lda_topics <- paste0("t_", 1:num_topics)
    z_lda_topics <- paste0("z_", lda_topics)
    preds <- topics_loadings
    
    # Standardize topic loadings; harmonize how scaling are made. 
    for (topic in lda_topics) {
      mean_value <- mean(preds[[topic]], na.rm = TRUE)
      std_dev <- stats::sd(preds[[topic]], na.rm = TRUE)
      preds[[paste0("z_", topic)]] <- (preds[[topic]] - mean_value) / std_dev
    }
    
    # Standardize control variables
    if(ncol(controls) != 0){
       # Rename columns to start with "z_"
       controls <- controls %>%
         dplyr::rename_with(~ paste0("z_", .), dplyr::everything())
       
       # Scale each column and convert the result to a numeric vector
       controls <- controls %>%
         dplyr::mutate(dplyr::across(dplyr::everything(), ~ as.numeric(base::scale(.))))
       
     }
    
    column_name <- colnames(x_y_axis1)
# #   # z_outcome <- tibble(!!paste0("z_", column_name) := base::scale(x_y_axis1)[1:nrow(x_y_axis1)])
# #    z_outcome <- tibble::tibble(x_y_axis1) %>%
# #      dplyr::mutate(dplyr::across(dplyr::everything(), ~ base::scale(.))) %>%
# #      dplyr::rename_with(~ paste0("z_", column_name))
# #    
# #    colnames(z_outcome) <- (paste0("z_", column_name))
    
    if (test_method == "linear_regression") {
      # Linear: We scale, so the "z_" prefix makes sense
      out_prefix <- "z_"
      outcome_data <- tibble::tibble(x_y_axis1) %>%
        dplyr::mutate(dplyr::across(dplyr::everything(), ~ as.numeric(base::scale(.)))) %>%
        dplyr::rename_with(~ paste0(out_prefix, column_name))
    } 
    if (test_method == "logistic_regression") {
      # Logistic: No scaling, so we use a different prefix or none at all
      # Let's use "y_" to denote the outcome/target
      out_prefix <- "y_"
      outcome_data <- x_y_axis1 %>%
        dplyr::rename_with(~ paste0(out_prefix, column_name))
    }
    
    # Now your target_name matches the actual column name created above
    target_name <- colnames(outcome_data)
    
    # Replace NA values with 0 #### Should be a flag
    #preds[is.na(preds)] <- 0
    if (any(is.na(preds))) {
      message("There are NA values in the predictions (preds).")
      
      #preds[is.na(preds)]
      
      #which(is.na(preds), arr.ind = TRUE)
      #rows_with_na <- which(apply(preds, 1, anyNA))
      
      #message("Note that all NAs are set to 0.")
      #preds[is.na(preds)] <- 0
    } 
    
    
    # For logistic regression, ensure the outcome variable is binary
# #    if (test_method == "logistic_regression") {
# #      
# #      if (!all(x_y_axis1[[1]] %in% c(0, 1))) {
# #        stop(paste("The outcome variable", colnames(x_y_axis1), "must be binary (0 or 1) for logistic regression."))
# #      }
# #    }
    # For logistic regression, ensure the outcome variable is binary (0/1) OR a 2-level factor
    if (test_method == "logistic_regression") {
      actual_values <- x_y_axis1[[1]]
      
      # 1. Check if it's a Factor or Character (Categorical)
      if (is.factor(actual_values) || is.character(actual_values)) {
        num_unique <- length(unique(stats::na.omit(actual_values)))
        if (num_unique != 2) {
          stop(paste("The outcome variable '", colnames(x_y_axis1), 
                     "' must have exactly 2 levels/groups, but found", num_unique))
        }
      } 
      # 2. Check if it's Numeric (must be exactly 0 and 1)
      else if (is.numeric(actual_values)) {
        unique_vals <- unique(stats::na.omit(actual_values))
        if (!all(unique_vals %in% c(0, 1))) {
          stop(paste("Numeric outcome variable '", colnames(x_y_axis1), 
                     "' must be 0 or 1 for logistic regression. Found:", 
                     paste(unique_vals, collapse = ", ")))
        }
      } 
      else {
        stop("Outcome variable must be numeric (0/1) or a 2-level factor/character.")
      }
    }
    
    # Initialize an empty list to store models
    multi_models <- list()
    
    # Making the regression formulas
    if(!ncol(controls) == 0){
      formula_tail <- paste0("+", paste(paste0(colnames(controls)), collapse = " + "))
    } else {
      formula_tail <- NULL
    }
    
    regression_data <-  dplyr::bind_cols(
      preds, x_y_axis1, outcome_data, controls) #z_outcome is now outcome_data
    
    if (test_method == "linear_regression" | test_method == "logistic_regression"){
      
      if (test_method == "linear_regression") {
        target_name <- colnames(outcome_data)
      }
      if (test_method == "logistic_regression") {
        target_name <- colnames(x_y_axis1)
      }
      
      for (i in 1:length(lda_topics)) {
        topic <- paste0("z_", lda_topics[i])
          
        #formula <- stats::as.formula(paste0(topic, formula_tail))
        formula <- stats::as.formula(paste0(target_name, "~", topic, formula_tail))
        
        message(colourise(
          paste0(i, ": fitting model formula: ", 
                 paste(deparse(formula), collapse = " "), " \n"), 
          "green"))
        
        # Extract variables used in the formula
        vars_needed <- all.vars(formula)
        
        # Check if there are complete cases
        n_complete <- sum(complete.cases(regression_data[vars_needed]))
        
        if (n_complete == 0) {
          warning(paste0("Skipping model for ", topic, ": no complete cases."))
          multi_models[[i]] <- NA  # or list(formula = formula, model = NA)
          next
        }
        
        # Fit the model
        if (test_method == "linear_regression") {
          multi_models[[i]] <- stats::lm(formula, data = regression_data)
        }
        
        if (test_method == "logistic_regression") {
          multi_models[[i]] <- stats::glm(formula, family = binomial, data = regression_data)
        }
      }
    }
    #### Extract statistics from models ####
    summary_statistics <- list()
    
 #   for (i in seq_along(multi_models)) {
 #     
 #     model_summary <- summary(multi_models[[i]])$coefficients
 #     
 #     if (test_method == "linear_regression") {
 #       
 #       estimate_values <- model_summary[, "Estimate"][z_lda_topics[i]][[1]]
 #       t_values <- model_summary[, "t value"][z_lda_topics[i]][[1]]
 #       p_values <- model_summary[, "Pr(>|t|)"][z_lda_topics[i]][[1]]
 #       
 #     } else if (test_method == "logistic_regression") {
 #       
 #       estimate_values <- model_summary[, "Estimate"][z_lda_topics[i]][[1]]
 #       t_values <- model_summary[, "z value"][z_lda_topics[i]][[1]]
 #       p_values <- model_summary[, "Pr(>|z|)"][z_lda_topics[i]][[1]]
 #       
 #     }
 #     res <- tibble::tibble(
 #       estimate_values = estimate_values,
 #       t_values = t_values,
 #       p_values = p_values 
 #     )
 #     
 #     colnames(res) <- c(
 #       paste0(target_name, ".estimate_beta"),
 #       if(test_method == "linear_regression") paste0(target_name, ".t"),
 #       if(test_method == "logistic_regression") paste0(target_name, ".z"), 
 #       paste0(target_name, ".p")
 #     )
 #     summary_statistics[[i]] <- res
 #   }
    
    for (i in seq_along(multi_models)) {
      
      model_i <- multi_models[[i]]
      
      # Skip if model is NA
      if (is.na(model_i)[1]) {
        summary_statistics[[i]] <- tibble::tibble(
          estimate = NA_real_,
          stat = NA_real_,
          p_value = NA_real_
        )
        colnames(summary_statistics[[i]]) <- c(
          paste0(target_name, ".estimate_beta"),
          if (test_method == "linear_regression") paste0(target_name, ".t"),
          if (test_method == "logistic_regression") paste0(target_name, ".z"),
          paste0(target_name, ".p")
        )
        next
      }
      
      # Otherwise extract summary
      model_summary <- summary(model_i)$coefficients
      
      if (test_method == "linear_regression") {
        estimate_values <- model_summary[, "Estimate"][z_lda_topics[i]][[1]]
        stat_values <- model_summary[, "t value"][z_lda_topics[i]][[1]]
        p_values <- model_summary[, "Pr(>|t|)"][z_lda_topics[i]][[1]]
      } else if (test_method == "logistic_regression") {
        estimate_values <- model_summary[, "Estimate"][z_lda_topics[i]][[1]]
        stat_values <- model_summary[, "z value"][z_lda_topics[i]][[1]]
        p_values <- model_summary[, "Pr(>|z|)"][z_lda_topics[i]][[1]]
      }
      
      res <- tibble::tibble(
        estimate_values = estimate_values,
        stat_values = stat_values,
        p_values = p_values
      )
      
      colnames(res) <- c(
        paste0(target_name, ".estimate_beta"),
        if (test_method == "linear_regression") paste0(target_name, ".t"),
        if (test_method == "logistic_regression") paste0(target_name, ".z"),
        paste0(target_name, ".p")
      )
      
      summary_statistics[[i]] <- res
    }
    
    
    summary_statistics <- dplyr::bind_rows(summary_statistics)
  
    ### Adjust p-value here
    p_variable <- summary_statistics[, grepl("\\.p", colnames(summary_statistics))]
    summary_statistics[paste0(target_name, ".p_adjusted")] <- stats::p.adjust(
      p_variable[[1]], multiple_comparison)
    
    
    levels_comment <- NULL
    if(test_method == "logistic_regression"){
      if (is.factor(actual_values)) {
        levels <- levels(actual_values)
        levels_comment <- paste0("Logistic Regression: Baseline (0) = ", levels[1], 
                                 ", Target (1) = ", levels[2], "  \n")
        message(colourise(
          levels_comment, 
          "blue"))
      }
    }
   
    
    # Merge with topic_terms for additional metadata
    output <- dplyr::bind_cols(
      topic_terms[c("topic", "top_terms", "prevalence", "coherence")], 
      summary_statistics)
    
    comment(output) <- levels_comment
    
    return(output)
  }
}


#' Test topics or n-grams
#' 
#' Statistically test topics or n-grams in relation to one or two other variables using 
#' regression or t-test.  
#' @param model (list) A trained model LDA-model from the topicsModel() function.
#' @param data (tibble) The tibble containing the variables to be tested.
#' @param preds (tibble) The predictions from the topicsPred() function.
#' @param ngrams (list) Output of the n-gram function.
#' @param x_variable (string) The x variable name to be predicted, and to be plotted (only needed for regression or correlation).
#' @param y_variable (string) The y variable name to be predicted, and to be plotted (only needed for regression or correlation).
# @param group_var (string) The variable to group by (only needed for t-test)
#' @param controls (vector) The control variables (not supported yet).
#' @param test_method (string) The test method to use. "default" checks if x_variable and y_variable
#' only contain 0s and 1s, for which it applies logistic regression; otherwise it applies linear regression. 
#' Alternatively, if only specifying x_variable, the user may manually specify either "linear_regression" or "logistic_regression".  
# @param p_alpha (numeric) Threshold of p value set by the user for visualising significant topics 
#' @param p_adjust_method (character) Method to adjust/correct p-values for multiple comparisons
#' (default = "fdr"; see also "holm", "hochberg", "hommel", "bonferroni", "BH", "BY",  "none").
#' @param complete_cases If TRUE, it will only use complete cases for x_variable, y_variable, controls, and preds.
#' @param seed (integer) The seed to set for reproducibility
#' @return A list of the test results, test method, and prediction variable.
#' @examples
#' \donttest{
#' # Test the topic document distribution in respect to a variable
#' 
#' dtm <- topicsDtm(
#'   data = dep_wor_data$Depphrase)
#' 
#' model <- topicsModel(
#'   dtm = dtm, # output of topicsDtm()
#'   num_topics = 20,
#'   num_top_words = 10,
#'   num_iterations = 1000,
#'   seed = 42)
#'                      
#' preds <- topicsPreds(
#'  model = model, # output of topicsModel()
#'  data = dep_wor_data$Depphrase)
#'                      
#' test <- topicsTest(
#'   model = model, # output of topicsModel()
#'   data=dep_wor_data,
#'   preds = preds, # output of topicsPreds()
#'   test_method = "linear_regression",
#'   x_variable = "Age")
#' }                 
#' @importFrom dplyr bind_cols all_of
#' @importFrom readr write_csv
#' @export
topicsTest <- function(
    data,
    model = NULL,
    preds = NULL,
    ngrams = NULL,
    x_variable = NULL,
    y_variable = NULL,
    controls = c(),
    test_method = "default",
    p_adjust_method = "fdr",
    complete_cases = FALSE,
    seed = 42){
  
  ##### Getting complete.cases #####
  if(complete_cases){
    # Select relevant columns from data
    relevant_columns <- c(x_variable, y_variable, controls)
    
    # Ensure relevant columns exist in data
    if (!all(relevant_columns %in% colnames(data))) {
      stop("One or more relevant columns are missing in the data.")
    }
    
    # Create a subset with relevant columns
    data_subset <- data %>% dplyr::select(dplyr::all_of(relevant_columns))
    
    # Merge preds with the data subset
    merged_data <- dplyr::bind_cols(preds, data_subset)
    
    # Check initial number of rows
    initial_rows <- nrow(merged_data)
    
    # Remove rows with any NA values
    cleaned_data <- merged_data[complete.cases(merged_data), ]
    
    # Check number of rows after removing NAs
    final_rows <- nrow(cleaned_data)
    
    # Compare and report the number of rows
    message(sprintf("Rows before NA removal: %d", initial_rows))
    message(sprintf("Rows after NA removal: %d", final_rows))
    
    # Optionally stop if too many rows are removed
    if (final_rows == 0) {
      stop("All rows have been removed due to missing values. Check your data for NAs.")
    }
    
    # Update preds and data for subsequent analysis
    preds <- cleaned_data %>% dplyr::select(names(preds))
    data <- cleaned_data %>% dplyr::select(dplyr::all_of(relevant_columns))
  }

 
  ###### End of handling NAs ####
  
  if (is.null(x_variable) & is.null(y_variable)){
    msg <- 'Please input the x_variable, and/or y_variable.'
    message(colourise(msg, "brown"))
    # return (NULL)
  }
  
  #### Warnings and instructions ####
  if(!is.null(y_variable)){
    if(grepl("__", y_variable)){
      stop("The x_variable, y_variable or controls cannot have names containing 2 underscores in a row ('__'). 
           Please rename the variable in the dataset.")
    }
  }
  
  if(!is.null(x_variable)){
    if(grepl("__", x_variable)){
      stop("The x_variable, y_variable or controls cannot have names containing 2 underscores in a row ('__'). 
           Please rename the variable in the dataset.")
    }
  }
  
  if (length(controls) > 0){
    for (control_var in controls){
      if (!is.numeric(data[[control_var]])){
        
        msg <- paste0("The controls variable '", 
                      control_var, 
                      "' should be numeric! \n")
        
        message(
          colourise(msg, "brown"))
        
        return (NULL)
      }}
  }
  
  if (is.null(x_variable) & is.null(y_variable)) {
    msg <- "Prediction variable is missing. Please input a prediction variable."
    message(colourise(msg, "brown"))
    return(NULL)
  }
  
  if (!is.list(model) & !is.list(ngrams)){
    msg <- "Input a model from the topicsModel() function or an ngram object from the topicsGrams() function."
    
    message(colourise(msg, "brown"))
    
    return(NULL)
  }
  
  if (length(data) == 0){
    msg <- "The data provided is empty. Please provide a list of text data."
    message(colourise(msg, "brown"))
    
    return(NULL)
  }
  
  if(!is.null(preds)){
    if (nrow(preds) == 0){
      msg <- "The predictions provided are empty. Please provide a list of predictions."
      message(colourise(msg, "brown"))
      return(NULL)
    }
    
    if (nrow(data) != nrow(preds)){
      msg <- "The number of data points and predictions do not match. Please provide predictions that were created from the same data."
      message(colourise(msg, "brown"))
      return(NULL)
    }
  }
  
  #### N-grams testing ####
  # (rearranging the data so that it fits the topics pipeline)
  if (!is.null(ngrams)){
    
    #freq_per_user <- tibble(ngrams$freq_per_user[,2:ncol(ngrams$freq_per_user)])
    freq_per_user <- ngrams$freq_per_user
    ngrams <- ngrams$ngrams
    total_n <- nrow(ngrams)
    colnames(freq_per_user) <- paste0("t_", 1:total_n)
    preds <- freq_per_user
    
    
    # 
    
    model$summary <- list(topic = paste0("t_", 1:total_n),
                          top_terms = ngrams$ngrams, 
                          prevalence = ngrams$prevalence, 
                          coherence = ngrams$coherence, 
                          pmi = ngrams$pmi)
    
    model$summary <- data.frame(model$summary)
  }
  
  
  #### Testing the elements (i.e., ngrams or topics) ####
  x_y_axis <- c(x_variable, y_variable)
  
  if(!is.null(x_variable) & !is.null(y_variable) & length(test_method) == 1 &
     test_method[1] != "default"){
    
    stop(
      paste("When setting both x_variable and y_variable, ",
            "you have to provide two methods to test_method;", 
            "e.g., test_method = c('logistic_regression', 'linear_regreission')  .")
    )
    
  }
  # # # # # #
  
  if(test_method[1] == "default"){
    # Updated Function to check if a column is binary (0/1 or a 2-level factor)
    is_binary_or_factor <- function(column) {
      vals <- column[!is.na(column)]
      if (is.factor(vals) || is.character(vals)) {
        return(length(unique(vals)) == 2)
      }
      return(all(unique(vals) %in% c(0, 1)))
    }
    
    # Create a vector of test_method for each column in x_y_axis
    test_method <- sapply(data[x_y_axis], function(col) {
      if (is_binary_or_factor(col)) {
        "logistic_regression"
      } else {
        "linear_regression"
      }
    })
  }
  
  # # # # # #
  #  if(test_method == "default"){
  #    # Function to check if a column is binary (contains only 0 and 1)
  #    is_binary <- function(column) {
  #      all(unique(column[!is.na(column)]) %in% c(0, 1))
  #    }
  #    
  #    # Create a vector of test_method for each column in x_y_axis
  #    test_method <- sapply(data[x_y_axis], function(col) {
  #      if (is_binary(col)) {
  #        "logistic_regression"
  #      } else {
  #        "linear_regression"
  #      }
  #    })
  #  } else if (test_method == "linear_regression") {
  #    
  #    test_method <- rep("linear_regression", 2)
  #    
  #  } else if (test_method == "logistic_regression") {
  #    
  #    test_method <- rep("logistic_regression", 2)
  #    
  #  }
  
  topic_loadings_all <- list()
  pre <- c('x','y')
  # i = 1
  for (i in 1:length(x_y_axis)){
    
    test <- topic_test(
      topic_terms = model$summary,
      topics_loadings = preds,
      x_y_axis1 = data[x_y_axis[i]],
      controls = data[controls],
      test_method = test_method[i],
      multiple_comparison = p_adjust_method)
    
    topic_loading <- list(
      test = test,
      test_method = test_method[i],
      x_y_axis1 = x_y_axis[i], 
      comment = comment(test))
    
    
    # Sorting output when not using ridge regression
      
    colnames(topic_loading$test) <- c("topic", "top_terms", "prevalence", "coherence",
                                        paste(pre[i], 
                                              colnames(topic_loading$test)[5:8], 
                                              sep = "."))
      
    topic_loadings_all[[i]] <- topic_loading
    
  }
  
  if (!is.null(y_variable)){
    # create the x.y.word.category
    topic_loadings_all[[3]] <- list()
    topic_loadings_all[[3]]$test <- dplyr::left_join(topic_loadings_all[[1]][[1]],#[,1:8], 
                                                     topic_loadings_all[[2]][[1]],#[,1:8],
                                                     by = c("topic", "top_terms", 
                                                            "prevalence", "coherence"))
    
    topic_loadings_all[[3]]$test_method <- topic_loadings_all[[1]]$test_method
    topic_loadings_all[[3]]$x_y_axis <- paste0(topic_loadings_all[[1]]$x_y_axis, '__',
                                               topic_loadings_all[[2]]$x_y_axis) 
    
    topic_loadings_all[[3]]$comment <- paste0("x_axis: ", topic_loadings_all[[1]]$comment, 
                                              "; y_axis: ",
                                              topic_loadings_all[[2]]$comment)
    
  } else {
    
      msg <- "The parameter y_variable is not set! Output 1 dimensional results."
      message(colourise(msg, "blue"))
      
      topic_loadings_all[[2]] <- list()
      topic_loadings_all[[3]] <- list()
      topic_loadings_all[[3]]$test <- topic_loadings_all[[1]][[1]]#[,1:8]
      topic_loadings_all[[3]]$test_method <- topic_loadings_all[[1]]$test_method
      topic_loadings_all[[3]]$x_y_axis <- topic_loadings_all[[1]]$x_y_axis
      topic_loadings_all[[3]]$comment <- topic_loadings_all[[1]]$comment
      
  }
  
  topic_loadings_all <- topic_loadings_all[[length(topic_loadings_all)]]
  return(topic_loadings_all)
}

library(testthat)
library(topics)
library(text)
library(glmnet)
test_that("Handling NAs",{
  
  testthat::skip_on_cran()
  
  NA_data <- dep_wor_data %>% 
    dplyr::select(c(Deptext, Gender, Age, PHQ9tot))
  
  colnames(NA_data)
  
  set.seed(123) # Set seed for reproducibility
  
  # Introduce NAs in the 'Deptext' column
  NA_data$Deptext[sample(1:nrow(NA_data), size = 5)] <- NA
  table(is.na(NA_data$Deptext))
  # Introduce NAs in the 'Gender' column
  NA_data$Gender[sample(1:nrow(NA_data), size = 5)] <- NA
  table(is.na(NA_data$Gender))
  # Introduce NAs in the 'Age' column
  NA_data$Age[sample(1:nrow(NA_data), size = 5)] <- NA
  
  # Introduce NAs in the 'PHQ9tot' column
  NA_data$PHQ9tot[sample(1:nrow(NA_data), size = 5)] <- NA
  
  # Introduce an entire row of NAs
  row_to_na <- sample(1:nrow(NA_data), size = 1) # Randomly select one row
  NA_data[row_to_na, ] <- NA
  
  
  # Introducing rows with few words
  NA_data$Deptext[[2]] <- "hello"
  NA_data$Deptext[[3]] <- "hello you"
  NA_data$Deptext[[4]] <- "hello how are"
  ##### Testing dataset on our functions ####
  
  # Testing with _ 
  data_test <- NA_data

  ## 1-Dimension
  dtm <- topics::topicsDtm(
    data = data_test$Deptext)
  
  model <- topics::topicsModel(
    dtm = dtm)
  
  preds <- topics::topicsPreds(
    model = model, 
    data = data_test$Deptext)
  
  test1 <- topics::topicsTest(
    model= model,
    preds = preds,
    data = data_test,
    x_variable = "PHQ9tot",
    y_variable = "Age")
  
  save_dir_temp <- tempfile()
  
  topics::topicsPlot(
    model = model, 
    test = test1, 
    p_alpha = 1,
    figure_format = "png",
    seed = 11, 
    save_dir = save_dir_temp)
  
  # Check if the wordcloud directory exists
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_11/wordclouds/dot_legend_corvar_PHQ9tot__Age.png")))
  
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_11/wordclouds/grid_legend_corvar_PHQ9tot__Age.png")))
  
 
})

library(testthat)
library(topics)
library(dplyr)


#remotes::install_github("theharmonylab/topics@12cd5b9ad33ef30d012b4aa65790a9be81afb5dd")
# .rs.restartR()

test_that("topicsTest performs linear regression correctly", {
  
  testthat::skip_on_cran()
  
  dtm <- topics::topicsDtm(
    data = dep_wor_data$Deptext)
  
  model <- topics::topicsModel(
    dtm = dtm)
  
  #help(topicsPreds)
  preds <- topics::topicsPreds(
    model = model, 
    data = dep_wor_data$Deptext)
  
  ############################
  #### linear_regression #####
  ############################
  result <- topics::topicsTest(
    model = model,
    preds = preds,
    data = dep_wor_data,
    x_variable = "Age",
    y_variable = "PHQ9tot",
    controls = NULL,
    test_method = c("linear_regression", "linear_regression")
    )
  result

  testthat::expect_true(is.list(result))
  testthat::expect_equal(result$test_method, "linear_regression")
  testthat::expect_true(any(grepl("estimate", names(result$test))))
  testthat::expect_true(any(grepl("t", names(result$test))))
  testthat::expect_true(any(grepl("p", names(result$test))))
  testthat::expect_true(any(grepl("p_adjusted", names(result$test))))
  
  testthat::expect_equal(result$test_method, "linear_regression")
  testthat::expect_equal(result$test$topic[1], 
                         "t_1")
  
  testthat::expect_equal(result$test$prevalence[1:4], 
                         c(3.389,  2.601,  3.733,  2.864), tolerance = 0.001)
  testthat::expect_equal(result$test$coherence[1:4], 
                         c(1.453, 0.803, 1.236, 0.361), tolerance = 0.001)
  
  
  testthat::expect_equal(result$test$x.z_Age.estimate_beta[1:5], 
           c(-0.02968493, -0.02739334,  0.03571515, -0.01120996, -0.08525859), tolerance = 0.0001)
  testthat::expect_equal(result$test$x.z_Age.t[1:5], 
                         c(-0.6627383, -0.6115369,  0.7975251, -0.2501762, -1.9095765), tolerance = 0.0001)
  testthat::expect_equal(result$test$x.z_Age.p[1:5], 
                         c(0.50780478, 0.54112328, 0.42552623, 0.80255430, 0.05676206), tolerance = 0.0001)
  testthat::expect_equal(result$test$x.z_Age.p_adjusted[1:5], 
                         c(0.7459746, 0.7459746, 0.7459746, 0.8917270, 0.3784137), tolerance = 0.0001)

  testthat::expect_equal(result$test$y.z_PHQ9tot.estimate[1:5], 
                         c(-0.028913708,  0.063841350, -0.012663082, -0.048725894,  0.004654584), tolerance = 0.0001)
  
  
  ####### Controlling for variables ####
  
  result_ctrl <- topics::topicsTest(
    model = model,
    preds = preds,
    data = dep_wor_data,
    x_variable = "Age",
    y_variable = "PHQ9tot",
    controls = c("Gender",  "GAD7tot"), #,
    test_method = c("linear_regression", "linear_regression")
  )
  result_ctrl
  
  # these are with the previous code; where controls where scaled in manua calcuation
  # -0.03121410, -0.02527967,  0.03445701, -0.02278217) the (average diff: 0.000265)
  testthat::expect_equal(result_ctrl$test$x.z_Age.estimate_beta[1:4], 
                         c(-0.03089316, -0.02503072,  0.03423101, -0.02287568), tolerance = 0.0001)
  
  testthat::expect_equal(result_ctrl$test$x.z_Age.t[1:4], 
                         c(-0.6919216, -0.5604032,  0.7653247, -0.5085558), tolerance = 0.0001)
  testthat::expect_equal(result_ctrl$test$x.z_Age.p[1:4], 
                         c(0.48931035, 0.57545745, 0.44444219, 0.61128977), tolerance = 0.0001)
  testthat::expect_equal(result_ctrl$test$x.z_Age.p_adjusted[1:4], 
                         c(0.7527852, 0.7858261, 0.7527852, 0.7858261), tolerance = 0.0001)
  
  # When not z-scoring control in scale:-0.02416600, 0.10753802, -0.07512578, 0.17245684; the average diff: 0.0653)
  testthat::expect_equal(result_ctrl$test$y.z_PHQ9tot.estimate_beta[1:4], 
                         c(-0.007475498,  0.033280355, -0.023326785,  0.054123169), tolerance = 0.0001)
  
  
  ##############################
  #### logistic_regression #####
  ##############################

  result_log <- topics::topicsTest(
    model = model,
    preds = preds,
    data = dep_wor_data,
    x_variable = "Gender",
   # y_variable = "PHQ9tot",
    test_method = "logistic_regression"
  )
  result_log
  
  
  testthat::expect_equal(result_log$test$x.Gender.estimate_beta[1:4], 
                         c(-0.07507502,  0.05734802,  0.15556099, -0.03501527), tolerance = 0.0001)
  testthat::expect_equal(result_log$test$x.Gender.z[1:4], 
                         c(-0.7553748,  0.6360858,  1.4940836, -0.3581462), tolerance = 0.0001)
  testthat::expect_equal(result_log$test$x.Gender.p[1:4], 
                         c(0.4500241, 0.5247205, 0.1351538, 0.7202339), tolerance = 0.0001)
  testthat::expect_equal(result_log$test$x.Gender.p_adjusted[1:4], 
                         c(0.6173182, 0.6173182, 0.6173182, 0.7581409), tolerance = 0.0001)
  
  # Should be able to have different test methods for x and y. 
  # Or could make it so that is is using logistic when it is a dichotomouse factor, 
  # otherwise it uses linear regression. 
  
})

test_that("topicsTest adjusts p-values for multiple comparisons", {
  
  testthat::skip_on_cran()
  
  dtm <- topics::topicsDtm(
    data = dep_wor_data$Deptext)
  
  model <- topics::topicsModel(
    dtm = dtm)
  
  preds <- topics::topicsPreds(
    model = model, 
    data = dep_wor_data$Deptext)
  
  result <- topics::topicsTest(
    model = model, 
    preds = preds, 
    data = dep_wor_data, 
    x_variable = "Age", 
    p_adjust_method = "bonferroni")
  
  testthat::expect_true(is.list(result))
  testthat::expect_equal(result$test_method[[1]], "linear_regression")
  testthat::expect_true(any(grepl("Age.estimate", names(result$test))))
  testthat::expect_true(any(grepl("Age.t", names(result$test))))
  testthat::expect_true(any(grepl("Age.p", names(result$test))))
  
})

library(testthat)
library(vdiffr)
library(topics)
library(ggforce)

test_that("N-Grams: topicsPlot with topicsGrams (without and with test",{
  
  testthat::skip_on_os(c("linux", "windows"))  # Skip on Ubuntu (Linux) and Windows
  testthat::skip_on_cran()
  
  # No test (i.e., no dimension) help(topicsGrams)
  ngrams <- topics::topicsGrams(
    data = dep_wor_data$Worphrase[1:100], 
    ngram_window = c(1:2), 
    stopwords = NULL,
    pmi_threshold = NULL)
  
  save_dir_temp <- tempfile()
  #save_dir_temp = "./results3"
  
  # Create a plot function
  ngram <- function() {
    topics::topicsPlot(
      ngrams = ngrams, 
      ngrams_max = 40,
      ngram_select = "prevalence",
      figure_format = "png", 
      save_dir = save_dir_temp)
  }
  
  # If it fails you should review changes <-  if they are intentionally you accept them: 
  # vdiffr::manage_cases()
  expect_doppelganger("1_ngram", ngram())
  
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_42/wordclouds/ngrams.png")))
  
  
  # With test (i.e., 1 dimension = two plots)
  ngrams <- topics::topicsGrams(
    data = dep_wor_data$Worphrase,
    pmi_threshold = 3)
  
  # help(topicsTest)
  test <- topics::topicsTest(
    data = dep_wor_data,
    ngrams = ngrams, 
    x_variable = "Age")
  
  
  pl <- topics::topicsPlot(
    ngrams = ngrams, 
    test = test,
    ngrams_max = 10,
    ngram_select = "prevalence",
    figure_format = "png", 
    p_alpha = 1,
    save_dir = save_dir_temp
    )
  
  if (!isTRUE(as.logical(Sys.getenv("CI")))) {
  
    ngram_negative <- function() {
      pl$negative
    }
    
    expect_doppelganger("2_ngram_negative", ngram_negative())
  
    ngram_positive <- function() {
      pl$positive
    }
    
    expect_doppelganger("3_ngram_positive", ngram_positive())
  
    testthat::expect_true(file.exists(paste0(
      save_dir_temp, "/seed_42/wordclouds/ngrams_negative.png")))
  
    testthat::expect_true(file.exists(paste0(
      save_dir_temp, "/seed_42/wordclouds/ngrams_positive.png")))
  }
  
})


test_that("topicsPlot WITHOUT test and preds", {
  testthat::skip_on_os(c("linux", "windows"))  # Skip on Ubuntu (Linux) and Windows
  testthat::skip_on_cran()
  
  dtm <- topics::topicsDtm(
    data = dep_wor_data$Deptext)
  
  #help(topicsModel)
  model <- topics::topicsModel(
    dtm = dtm, 
    num_topics = 50,
    num_top_words = 20, 
    num_iterations = 100)
  
  save_dir_temp <- tempfile()
  
  topics_1 <- topics::topicsPlot(
    model = model,
    plot_topics_idx = c(1,3),
    figure_format = "png", 
    save_dir = save_dir_temp)

  
  topics_t1 <- function() {
    topics_1$t1
  }
  expect_doppelganger("topics_t1", topics_t1())
  
  topics_t3 <- function() {
    topics_1$t3
  }
  expect_doppelganger("topics_t3", topics_t1())
  
  topics_overview_plot <- function() {
    topics_1$overview_plot
  }
  expect_doppelganger("topics_overview_plot", topics_overview_plot())
  
  # Check if the word cloud directory exists
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_42/wordclouds/t_1.png")))
  
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_42/wordclouds/t_3.png")))
  
  #### Plot most prevalent topics in model ####  
  
  topics2 <- topics::topicsPlot(
    model = model,
    plot_n_most_prevalent_topics = 5,
    figure_format = "png", 
    save_dir = save_dir_temp)
  
  testthat::expect_equal(
    names(topics2), 
    c("t_t_2",  "t_t_29", "t_t_46", "t_t_36", "t_t_35", "overview_plot")
  )
  
  topics2_t_t_2 <- function() {
    topics2$t1
  }
  expect_doppelganger("topics2_t_t_2", topics2_t_t_2())
  
  topics2_overview_plot <- function() {
    topics2$overview_plot
  }
  expect_doppelganger("topics2_overview_plot", topics2_overview_plot())
  
  topics3 <- topics::topicsPlot(
    model = model,
    plot_n_most_prevalent_topics = 5,
    allowed_word_overlap = 2,
    figure_format = "png", 
    save_dir = save_dir_temp)
  
  testthat::expect_equal(
    names(topics3), 
    c("t_t_2", "t_t_46", "t_t_31", "t_t_18", "t_t_19", "overview_plot")
  )
  
  topics3_t3 <- function() {
    topics3$t3
  }
  expect_doppelganger("topics3_t3", topics3_t3())
  
  
  testthat::expect_error(topics::topicsPlot(
    model = model,
    plot_topics_idx = c("t1"),
    plot_n_most_prevalent_topics = 5,
    figure_format = "png", 
    save_dir = save_dir_temp))
  
})


test_that("topicsPlot WITH test", {
  
  testthat::skip_on_os(c("linux", "windows"))  # Skip on Ubuntu (Linux) and Windows
  testthat::skip_on_cran()
  
  ## 1-Dimension
  dtm <- topics::topicsDtm(
    data = dep_wor_data$Deptext)
  
  model <- topics::topicsModel(
    dtm = dtm, 
    num_topics = 50,
    num_top_words = 20, 
    num_iterations = 100)
  
  #### Plots one-dimensional regression type plot ####
  preds <- topics::topicsPreds(
    model = model, 
    data = dep_wor_data$Deptext)
  
  test1 <- topics::topicsTest(
    model = model,
    preds = preds,
    data = dep_wor_data,
    x_variable = "Age")
  

  save_dir_temp <- tempdir()
  
  topics4 <- topics::topicsPlot(
    model = model,
    test = test1,
    p_alpha = .1,
    scatter_legend_dot_size = c(1, 10),
    scatter_legend_bg_dot_size = c(10, 20),
    scatter_legend_dots_alpha = 0.3,
    scatter_legend_bg_dots_alpha = 0.1,
    figure_format = "png",
    seed = 11,
    allowed_word_overlap = 3,
    save_dir = save_dir_temp,
    grid_legend_number_color = "white")
  
  topics4$distribution
  topics4$legend

  

  topics4_legend <- function() {
    topics4$legend
  }
  expect_doppelganger("topics4_legend", topics4_legend())
  
  topics4_distribution <- function() {
    topics4$distribution
  }
  expect_doppelganger("topics4_distribution", topics4_distribution())
  
  topics4square1t_48 <- function() {
    topics4$square1$t_48
  }
  expect_doppelganger("topics4square1t_48", topics4square1t_48())

  
  topics4overview_plot <- function() {
    topics4$overview_plot
  }
  expect_doppelganger("topics4overview_plot", topics4overview_plot())
  
  
    
#  topics4square2t_1 <- function() {
#    topics4$square2$t_1
#  }
#  expect_doppelganger("topics4square2t_1", topics4square2t_1())
  
  topics4square3t_34 <- function() {
    topics4$square3$t_34
  }
  expect_doppelganger("topics4square3t_34", topics4square3t_34())
  
  # Check if the wordcloud directory exists
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_11/wordclouds/dot_legend_corvar_Age.png")))
  
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_11/wordclouds/grid_legend_corvar_Age.png")))
  
  
  
  #### Plots one-dimensional LOGISTIC regression type plot ####
  preds <- topics::topicsPreds(
    model = model, 
    data = dep_wor_data$Deptext)
  
  test1 <- topics::topicsTest(
    model = model,
    preds = preds,
    data = dep_wor_data,
    x_variable = "Gender")
  
  save_dir_temp <- tempdir()
  
  topics4_Logistic <- topics::topicsPlot(
    model = model, 
    test = test1, 
    p_alpha = .1,
    figure_format = "png",
    seed = 11)
  
  topics4_Logistic$legend
  topics4_Logistic$distribution
  
  topics4_legend_Logistic <- function() {
    topics4_Logistic$legend
  }
  expect_doppelganger("topics4_legend_Logistic", topics4_legend_Logistic())
  
  topics4_distribution_Logistic <- function() {
    topics4_Logistic$distribution
  }
  expect_doppelganger("topics4_distribution_Logistic", topics4_distribution_Logistic())
  
#  topics4square2t_1 <- function() {
#    topics4_Logistic$square2$t_1
#  }
#  expect_doppelganger("topics4square2t_1", topics4square2t_1())
  
  
  ###########################
  #### 2-Dimension  #########
  ###########################
  
  test2 <- topics::topicsTest(
    model = model, 
    preds = preds, 
    data = dep_wor_data, 
    x_variable = "PHQ9tot",
    y_variable = "Age"
    )
  
  topics5 <- topics::topicsPlot(
    model = model, 
    test = test2, 
    p_alpha = 0.5, 
    color_scheme =  c(
      "yellow", "#398CF9",  # quadrant 1 (upper left corner)
      "yellow", "#60A1F7",  # quadrant 2 
      "yellow", "#5dc688",  # quadrant 3 (upper right corner)
      "yellow", "#e07f6a",  # quadrant 4
      "yellow", "darkgray", # quadrant 5 (middle square)
      "yellow", "#40DD52",  # quadrant 6 
      "yellow", "#FF0000",  # quadrant 7 (bottom left corner)
      "yellow", "#EA7467",  # quadrant 8 
      "yellow", "#85DB8E"),
    figure_format = "png",
    seed = 12, 
    save_dir = save_dir_temp
    )
  

  # Legend
  topics5legend <- function() {
    topics5$legend
  }
  expect_doppelganger("topics5legend", topics5legend())
  
  # Distribution
  topics5distribution <- function() {
    topics5$distribution
  }
  expect_doppelganger("topics5distribution", topics5distribution())
  
  # Square 1
  topics5square1 <- function() {
    topics5$square1$t_6
  }
  expect_doppelganger("topics5square1", topics5square1())
  
  # Square 2
  topics5square2 <- function() {
    topics5$square2$t_34
  }
  expect_doppelganger("topics5square2", topics5square2())
  
#  # Square 3
#  topics5square3 <- function() {
#    topics5$square3$t_1
#  }
#  expect_doppelganger("topics5square3", topics5square3())
  
  # Square 4
  topics5square4 <- function() {
    topics5$square4$t_2
  }
  expect_doppelganger("topics5square4", topics5square4())
  
#  # Square 5
#  topics5square5 <- function() {
#    topics5$square5
#  }
#  expect_doppelganger("topics5square5", topics5square5())
  
  # Square 6
  topics5square6 <- function() {
    topics5$square6$t_7
  }
  expect_doppelganger("topics5square6", topics5square6())
  
#  # Square 7
#  topics5square7 <- function() {
#    topics5$square7
#  }
#  expect_doppelganger("topics5square7", topics5square7())
#  
#  # Square 8
#  topics5square8 <- function() {
#    topics5$square8
#  }
#  expect_doppelganger("topics5square8", topics5square8())
  
  # Square 9
  topics5square9 <- function() {
    topics5$square9$t_23
  }
  expect_doppelganger("topics5square9", topics5square9())
  
  
  # Square 9
  topics5overview_plot <- function() {
    topics5$overview_plot
  }
  expect_doppelganger("topics5overview_plot", topics5overview_plot())
  
  
  # Check if the wordcloud directory exists
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_12/wordclouds/dot_legend_corvar_PHQ9tot__Age.png")))
  
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_12/wordclouds/grid_legend_corvar_PHQ9tot__Age.png")))
  
  
  #### Setting specific topics to plot ######
  topics_specific <- topics::topicsPlot(
    model = model,
    test = test2,
    p_alpha = 0.99,
    scatter_legend_specified_topics = c('t_1', 't_2'),
    seed = 8, 
    save_dir = save_dir_temp, 
    figure_format = "png")
  
  # Checking legend
  topics_specific_t1 <- function() {
    topics_specific$distribution
  }
  expect_doppelganger("topics_specific_t1", topics_specific_t1())
  
  
  ##### Using Control Variables 
  
  tests2D <- topics::topicsTest(
    model = model,
    preds = preds,
    data =  dep_wor_data,
    x_variable = 'PHQ9tot',
    y_variable = 'GAD7tot',
    controls = c('Age','Gender')
  )
  
  #random_sequence <- sample(1:9, size = nrow(tests2D[[3]]$test), replace = TRUE)
  #tests2D[[3]]$test$color_categories <- random_sequence
  
  # Why do we get a warning here
  plots_controlled <- topics::topicsPlot(
    model = model,
    test = tests2D,
    p_alpha = 0.99,
    seed = 1,
    figure_format = "png",
    save_dir = save_dir_temp
  )
  
  # Checking legend
  plots_controlled_distribution <- function() {
    plots_controlled$distribution
  }
  expect_doppelganger("plots_controlled_distribution", plots_controlled_distribution())
  
  # Checking legend
  plots_controlled_legend <- function() {
    plots_controlled$legend
  }
  expect_doppelganger("plots_controlled_legend", plots_controlled_legend())
  
  # Checking legend
  plots_controlled_square1t_3 <- function() {
    plots_controlled$square1$t_3
  }
  expect_doppelganger("plots_controlled_square1t_3", plots_controlled_square1t_3())
  
  
})


test_that("topicsPlot WITH underscores in names", {
  
  testthat::skip_on_os(c("linux", "windows"))  # Skip on Ubuntu (Linux) and Windows
  testthat::skip_on_cran()
  save_dir_temp <- tempfile()
  #save_dir_temp <- "./res_under"
  
  # Testing with _ 
  data_test <- dep_wor_data
  data_test$Dep_text <- data_test$Deptext
  data_test$Age_test <- data_test$Age
  
  ## 1-Dimension
  dtm <- topics::topicsDtm(
    data = data_test$Dep_text)
  
  model <- topics::topicsModel(
    dtm = dtm)
  
  preds <- topics::topicsPreds(
    model = model, 
    data = data_test$Dep_text)
  
  test1 <- topics::topicsTest(
    model= model,
    preds = preds,
    data = data_test,
    x_variable = "Age_test",
    y_variable = "Age")
  
  plot5 <- topics::topicsPlot(
    model = model, 
    test = test1, 
    p_alpha = 1,
    figure_format = "png",
    seed = 11, 
    save_dir = save_dir_temp)
  plot5
  # Check if the wordcloud directory exists
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_11/wordclouds/dot_legend_corvar_Age_test__Age.png")))
  
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_11/wordclouds/grid_legend_corvar_Age_test__Age.png")))
  
  
  ## 2-Dimension  
  
  test2 <- topics::topicsTest(
    model = model, 
    preds = preds, 
    data = data_test, 
    x_variable = "PHQ9tot",
    y_variable = "Age_test"
  )
  
  plot6 <- topics::topicsPlot(
    model = model, 
    test = test2, 
    p_alpha = 1, 
    color_scheme =  c(
      "yellow", "#398CF9",  # quadrant 1 (upper left corner)
      "yellow", "#60A1F7",  # quadrant 2 
      "yellow", "#5dc688",  # quadrant 3 (upper right corner)
      "yellow", "#e07f6a",  # quadrant 4
      "yellow", "darkgray", # quadrant 5 (middle square)
      "yellow", "#40DD52",  # quadrant 6 
      "yellow", "#FF0000",  # quadrant 7 (bottom left corner)
      "yellow", "#EA7467",  # quadrant 8 
      "yellow", "#85DB8E"),
    figure_format = "png",
    seed = 12, 
    save_dir = save_dir_temp
  )
  
  
  # Check if the wordcloud directory exists
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_12/wordclouds/dot_legend_corvar_PHQ9tot__Age_test.png")))
  
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_12/wordclouds/grid_legend_corvar_PHQ9tot__Age_test.png")))
  
})


test_that("topicsPlot WITH PMI", {
  
  testthat::skip_on_os(c("linux", "windows"))  # Skip on Ubuntu (Linux) and Windows
  testthat::skip_on_cran()
  save_dir_temp <- tempfile()

  
  ## 1-Dimension
  dtm <- topics::topicsDtm(
    data = dep_wor_data$Deptext,
    pmi_threshold = 1
    )
  
  model <- topics::topicsModel(
    dtm = dtm)
  
  preds <- topics::topicsPreds(
    model = model, 
    data = dep_wor_data$Deptext)
  
  test1 <- topics::topicsTest(
    model= model,
    preds = preds,
    data = dep_wor_data,
    x_variable = "Age")
  
  testthat::expect_equal(test1$test$x.z_Age.estimate[1:4], 
                         c(0.012425389,  0.034527241,  0.039768994,  0.007723522), 
                         tolerance = 0.00001)
  
  
  
})

test_that("topicsPlot WITH concentric circles", {
  
  testthat::skip_on_os(c("linux", "windows"))  # Skip on Ubuntu (Linux) and Windows
  testthat::skip_on_cran()
  save_dir_temp <- tempfile()
  #save_dir_temp <- "./res_under"
  # Testing with _ 
  data_test <- dep_wor_data
  data_test$Dep_text <- data_test$Deptext
  data_test$Age_test <- data_test$Age


  has_geom_circle <- function(plot) {
    any(sapply(plot$layers, function(layer) inherits(layer$geom, "GeomCircle")))
  }


  ## 1-Dimension
  dtm <- topics::topicsDtm(
    data = data_test$Dep_text)
  
  model <- topics::topicsModel(
    dtm = dtm)
  
  preds <- topics::topicsPreds(
    model = model, 
    data = data_test$Dep_text)
  
  test1 <- topics::topicsTest(
    model= model,
    preds = preds,
    data = data_test,
    x_variable = "Age_test")
  
  plot5 <- topics::topicsPlot(
    model = model, 
    test = test1, 
    p_alpha = 1,
    scatter_legend_circles = TRUE,
    figure_format = "png",
    seed = 14, 
    save_dir = save_dir_temp)
  plot5
  # Check if the wordcloud directory exists
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_14/wordclouds/dot_legend_corvar_Age_test.png")), info = 'One Dimensional scatter legend file exists')
 
  testthat::expect_false(has_geom_circle(plot5$distribution), info = 'One Dimensional Scatter Legend should not plot circles even with a TRUE parameter')
   
  ## 2-Dimensions  
  test2 <- topics::topicsTest(
    model = model, 
    preds = preds, 
    data = data_test, 
    x_variable = "PHQ9tot",
    y_variable = "Age_test"
  )
  
  plot6 <- topics::topicsPlot(
    model = model, 
    test = test2, 
    p_alpha = 1, 
    figure_format = "png",
    scatter_legend_circles = TRUE,
    seed = 15, 
    save_dir = save_dir_temp
  )
  
  
  # Check if the wordcloud directory exists
  testthat::expect_true(file.exists(paste0(
    save_dir_temp, "/seed_15/wordclouds/dot_legend_corvar_PHQ9tot__Age_test.png")),  info = '2d Scatter Legend file exists')

  testthat::expect_true(has_geom_circle(plot6$distribution),  info = 'Two Dimensional Scatter Legend should plot circles with a TRUE parameter')

  plot7 <- topics::topicsPlot(
    model = model, 
    test = test2, 
    p_alpha = 1, 
    figure_format = "png",
    save_dir = save_dir_temp
  )

  testthat::expect_false(has_geom_circle(plot7$distribution),  info = 'No Circles plot by default for Scatter Legend')

  plot8 <- topics::topicsPlot(
    model = model, 
    test = test2, 
    p_alpha = 1, 
    figure_format = "png",
    scatter_legend_circles = TRUE,
    scatter_legend_circles_num = 3,
    scatter_legend_circles_radius = 0.02,
    save_dir = save_dir_temp
  )

  testthat::expect_true(has_geom_circle(plot8$distribution),  info = 'Scatter Legend circles plot can use custom values')


})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
evaluate = FALSE

## ----dtm, eval = FALSE, warning=TRUE, message=TRUE----------------------------
# 
# library(topics)
# 
# dtm <- topicsDtm(
#   data = dep_wor_data$Depword)
# 
# # Check the results from the dtm and refine stopwords and removal rates if necessary
# dtm_evaluation <- topicsDtmEval(
#   dtm)
# dtm_evaluation$frequency_plot

## ----model, eval = FALSE, warning=FALSE, message=FALSE------------------------
# 
# model <- topicsModel(
#   dtm = dtm,
#   num_topics = 20,
#   num_iterations = 1000)
# 

## ----preds, eval = FALSE, warning=FALSE, message=FALSE------------------------
# 
# preds <- topicsPreds(
#   model = model,
#   data = dep_wor_data$Depword)
# 

## ----test, eval = FALSE, warning=FALSE, message=FALSE-------------------------
# 
# test <- topicsTest(
#   data = dep_wor_data,
#   model = model,
#   preds = preds,
#   x_variable = "PHQ9tot",
#   controls = c("Age"),
#   test_method = "linear_regression")
# 

## ----plot_list, eval = FALSE, warning=FALSE, message=FALSE--------------------
# 
# plot_list <- topicsPlot(
#   model = model,
#   test = test,
#   figure_format = "png")
# 
# # showing some of the plots
# plot_list$square1

#' Rename the columns of a model with the vocabulary
#' @param model (list) the model to be modified
#' @param col (string) the column to be modified
#' @param vocabulary (list) the vocabulary to be used
#' @return (list) the modified model
#' @noRd
name_cols_with_vocab <- function(
    model, 
    col, 
    vocabulary){
  
  colnames(model[[col]]) <- as.character(unlist(vocabulary))
  return(model)
}

#' This is a private function
#' @param df_list (list) a list of data frames with each topic
#' @param phi (data.frame) data frame with topic word scores
#' @param model_type (string) "mallet"
#' @return list of data.frames with assigned phi
#' @noRd
assign_phi_to_words <- function(
    df_list, 
    phi, 
    model_type){
  
  
  for (i in 1:length(df_list)){
    df <- data.frame(df_list[[i]]) 
    colnames(df)[1] <- "Word"
    phi_vector <- c()
    for (j in 1:nrow(df)){
      word <- df[j,]
      if (model_type=="mallet"){
        phi_vector <- c(phi_vector,phi[i,][word])
      } else {
        phi_vector <- c(phi_vector,phi[paste0("t_",i),][word])
      }
    }
    df$phi <- phi_vector
    df_list[[i]] <- df
  }
  return(df_list)
}

#
#' Create list of data.frames with topics most frequent words and topic term scores
#' @param df (string) 
#' @return list of data.frames
#' @noRd
create_df_list_bert_topics <- function(
    df
    ) {
  
  n <- nrow(df$summary)
  df_list <- vector("list", n)

  for (i in 1:n) {
    top_terms_bert <- df$summary$top_terms[i]
    df_list[[i]] <- top_terms_bert
  }
  
  # Convert each topic's word list to a dataframe with Word and phi
  df_list <- lapply(df_list, function(words_string) {
    # Split into individual words
    words <- unlist(strsplit(words_string, ",\\s*"))
    # If removing/changing below line then update message in text-package
    phi <- rev(seq_along(words)) / sum(seq_along(words))
    
    # Return as data.frame
    tibble::tibble(Word = words, phi = phi)
  })
  
  # Assign pseudo-phi values as descending importance scores (normalized from 1 to N)
  # This is moved to the text-package message(colourise("Note: BERTopic does not provide true 'phi' values (i.e., P(word | topic)) as in probabilistic models like LDA. Instead, we assign descending normalized values to approximate word importance.", "blue"))
  
  return(df_list)
}



#' This is a private function 
#' @param summary (data.frame) the models summary
#' @return a list of dataframes for each topic filled with top terms
#' @importFrom stats complete.cases
#' @noRd
create_topic_words_dfs <- function(
    summary){
  
  n <- nrow(summary)
  df_list <- vector("list", n)
  
  # Create and name the dataframes in a loop
  for (i in 1:n) {
    word_vector <- unlist(strsplit(summary[paste0("t_",i),]$top_terms, ", "))
    df <- data.frame(Word = word_vector) # Create an empty dataframe
    df <- df_cleaned <- df[stats::complete.cases(df), ]
    df_list[[i]] <- df  # Add the dataframe to the list
    
    name <- paste("t", i, sep = "_")  # Create the name for the dataframe
    assign(name, df_list[[i]])  # Assign the dataframe to a variable with the specified name
  }
  return(df_list)
}



#' Function to select non-overlapping texts and count excluded topics
#'
#' @param df A tibble or data frame containing the text data.
#' @param text_column A character string specifying the name of the text column.
#' @param n_texts A numeric value indicating the number of texts to select (default is 3).
#' @param allowed_word_overlap A numeric value indicating the maximum number of words allowed to overlap (default is 5).
#' @return A list containing:
#'   - `selected`: A tibble or data frame containing the selected non-overlapping texts.
#'   - `excluded_count`: The number of topics that were not selected due to overlap.
#' @importFrom stringr str_split
#' @importFrom dplyr filter
#' @noRd
select_non_overlapping_texts <- function(
    df, 
    text_column, 
    n_texts = 3, 
    allowed_word_overlap = 5) {
  
  # Function to count word overlap between two texts
  count_word_overlap <- function(text1, text2) {
    words1 <- unlist(stringr::str_split(text1, "\\s+"))
    words2 <- unlist(stringr::str_split(text2, "\\s+"))
    length(intersect(words1, words2))
  }
  
  # Extract the text data
  texts <- df[[text_column]]
  
  # Initialize with the first text
  selected_texts <- texts[1]
  excluded_count <- 0  # Counter for excluded topics
  
  # Loop through each subsequent text
  for (i in 2:length(texts)) {
    # Check overlap with all currently selected texts
    overlap_counts <- sapply(selected_texts, function(selected) {
      count_word_overlap(selected, texts[i])
    })
    
    # Include the text only if the overlap with all selected texts is less than allowed_word_overlap
    if (all(overlap_counts < allowed_word_overlap)) {
      selected_texts <- c(selected_texts, texts[i])
    } else {
      # Increment the excluded count if the text is not selected
      excluded_count <- excluded_count + 1
    }
    
    # Stop if we have reached the desired number of texts
    if (length(selected_texts) >= n_texts) {
      break
    }
  }
  
  # Filter the data frame to keep only the selected texts
  #filtered_df <- filter(df, !!rlang::sym(text_column) %in% selected_texts)
  filtered_df <- df[df[[text_column]] %in% selected_texts, ]
  
  if(!is.null(filtered_df$color_categories1[[1]])){
    
    colour_category_number <- unique(filtered_df$color_categories1)
    # Return a list with the filtered data frame and excluded count
    message(colourise(paste0(
      excluded_count," topic(s) were excluded in color_category ", 
      colour_category_number, " due to the `allowed_word_overlap` filter. "
    ), "blue"))
    
  } else {
    
    message(colourise(paste0(
      excluded_count," topics were excluded due to the `allowed_word_overlap` filter. ",
      "blue")))
    
  }
  
  
  return(filtered_df)
  #return(list(
  #  selected = filtered_df,
  #  excluded_count = excluded_count
  #))
}

#' Assgning numeric categories for further topic visualization colors.
#' @param topic_loadings_all (tibble) The tibble from topicsTest1
#' @param p_alpha (numeric) Threshold of p value set by the user for visualising significant topics 
#' @param dimNo (numeric) 1 dimension or 2 dimensions
#' @return A new tibble with the assigned numbers
#' @importFrom dplyr mutate
#' @noRd
topicsNumAssign_dim2 <- function(
    topic_loadings_all,
    p_alpha = 0.05, 
    dimNo = 2){
  
  if (dimNo == 1){
    num1 <- 1:3
    topic_loadings_all <- topic_loadings_all %>%
      dplyr::mutate(color_categories = dplyr::case_when(
        x_plotted < 0 & adjusted_p_values.x < p_alpha ~ num1[1],
        adjusted_p_values.x > p_alpha ~ num1[2],
        x_plotted > 0 & adjusted_p_values.x < p_alpha ~ num1[3]
      ))
  }else{
    num1 <- 1:9
    topic_loadings_all <- topic_loadings_all %>%
      dplyr::mutate(color_categories = dplyr::case_when(
        x_plotted < 0 & adjusted_p_values.x < p_alpha & y_plotted > 0 & adjusted_p_values.y < p_alpha ~ num1[1],
        adjusted_p_values.x > p_alpha & y_plotted > 0 & adjusted_p_values.y < p_alpha ~ num1[2],
        x_plotted > 0 & adjusted_p_values.x < p_alpha & y_plotted > 0 & adjusted_p_values.y < p_alpha ~ num1[3],
        x_plotted < 0 & adjusted_p_values.x < p_alpha & adjusted_p_values.y > p_alpha ~ num1[4],
        adjusted_p_values.x > p_alpha & adjusted_p_values.y > p_alpha ~ num1[5],
        x_plotted > 0 & adjusted_p_values.x < p_alpha & adjusted_p_values.y > p_alpha ~ num1[6],
        x_plotted < 0 & adjusted_p_values.x < p_alpha & y_plotted < 0 & adjusted_p_values.y < p_alpha ~ num1[7],
        adjusted_p_values.x > p_alpha & y_plotted < 0 & adjusted_p_values.y < p_alpha ~ num1[8],
        x_plotted > 0 & adjusted_p_values.x < p_alpha & y_plotted < 0 & adjusted_p_values.y < p_alpha ~ num1[9]
      ))
  }
  
  return (topic_loadings_all)
}

#' A Private function to check the validity of color hex codes for parameter highlight_topic_words.
#' highlight_topic_words (named vector) the named vector with negative words as elements and color hex codes as names for each word
#' @noRd
all_hex <- function(highlight_topic_words) {
  all(grepl("^#[0-9A-Fa-f]{6}$", highlight_topic_words))
}

#' Separate words for negative words as a fixed-colour tibble and other words as a gradient-color tibble.
#' @noRd
separate_neg_words <- function(
    df_list_element, 
    highlight_topic_words) {
  
  pattern <- paste0("(^|[ _])(", paste(highlight_topic_words, collapse = "|"), ")(?=[ _])")
  
  # Create df_list_ele_fixed with words that match or contain the dictionary keys
  df_list_ele_fixed <- df_list_element %>%
    dplyr::filter(stringr::str_detect(Word, pattern)) %>%
    dplyr::mutate(FixedColor = highlight_topic_words[stringr::str_extract(
      Word, 
      pattern)])
  
  # Create df_list_ele_gradient with the remaining words
  df_list_ele_gradient <- df_list_element %>%
    dplyr::filter(!stringr::str_detect(
      Word,
      pattern))
  
  return (list(
    df_list_ele_gradient, 
    df_list_ele_fixed))
}


# Helper functions for create_plots 
#' @noRd
ensure_dir <- function(path){ if(!dir.exists(path)) dir.create(path, recursive = TRUE) }

#' @noRd
save_plot <- function(plot, stub, header='', save_dir, figure_format, width, height, seed){
  if(is.null(save_dir)) return(invisible(NULL))
  root <- file.path(save_dir, sprintf("seed_%s", seed), "wordclouds")
  ensure_dir(root)
  ggplot2::ggsave(
    file.path(root, sprintf("%s%s.%s", header, stub, figure_format)),
    plot   = plot,
    width  = width,
    height = height,
    units  = "in"
  )
}

#choose_scheme <- function(est){ if(est[1] < 0) color_negative_cor else color_positive_cor }

#' @noRd
highlight_neg <- function(topic_df){
  if(is.null(highlight_topic_words) || !is.character(highlight_topic_words))
    return(topic_df)
  separated <- separate_neg_words(topic_df, highlight_topic_words)
  separated[[2]] <- dplyr::mutate(separated[[2]], source = "negDic",
                                  label_content = sprintf("<b><i>%s</i></b>", Word))
  separated[[1]] <- dplyr::mutate(separated[[1]], source = "notNegDic",
                                  label_content = Word)
  dplyr::bind_rows(separated[[1]], separated[[2]]) %>%
    dplyr::mutate(phi = phi^1.1 / sum(phi^1.1))
}

## #' @noRd
## build_cloud <- function(data, scheme, max_size_local){
##   lbl <- if("label_content" %in% colnames(data)) "label_content" else "Word"
##   # Here we set what the size and colour of the words in topics represents
##   ggplot2::ggplot(data, ggplot2::aes_string(label = lbl, size = "phi", color = "phi")) +
##     ggwordcloud::geom_text_wordcloud() +
##     ggplot2::scale_size_area(max_size = max_size_local) +
##     ggplot2::theme_minimal() +
##     scheme
## }
#' @noRd
build_cloud <- function(data, scheme, max_size_local, word_font = "sans", title_font = "sans"){
  lbl <- if ("label_content" %in% colnames(data)) "label_content" else "Word"
  
  ggplot2::ggplot(
    data,
    ggplot2::aes(
      label = .data[[lbl]],
      size  = phi,
      color = phi
    )
  ) +
    ggwordcloud::geom_text_wordcloud(family = word_font) +
    ggplot2::scale_size_area(max_size = max_size_local) +
    ggplot2::theme_minimal(base_family = title_font) +
    scheme
}


#' @noRd
min_p <- function(p_vec) suppressWarnings(min(p_vec, na.rm = TRUE))

#' @noRd
format_p <- function(p_vec) format(min_p(p_vec), scientific = TRUE, digits = 2)

#' Generate topic and n-gram wordcloud plots
#'
#' @param df_list list of data.frames with topic word distributions
#' @param summary data.frame with topic prevalences (row‑names t_1, t_2, …)
#' @param ngrams data.frame with two columns: ngrams, prevalence
#' @param test data.frame returned by textTopicTest()
#' @param test_type "linear_regression" | "binary_regression"
#' @param cor_var variable name used in the regression columns (e.g. "age")
#' @param popout tibble of topic indices to always plot regardless of p‑value
#' @param color_negative_cor, color_positive_cor ggplot2 scale functions
#' @param grid_pos numeric grid position used when emphasising topics
#' @param scale_size logical, scale cloud size by topic prevalence?
#' @param plot_topics_idx vector of topic indices to plot (NULL => all)
#' @param p_alpha max adjusted p‑value for a topic/term to be plotted
#' @param highlight_topic_words character vector of words to emphasise
#' @param save_dir directory where plots are saved (mandatory)
#' @param figure_format image file extension (svg | png | …)
#' @param width,height,max_size basic ggsave options
#' @param seed integer used in the file‑path so different runs do not collide
#' @importFrom ggwordcloud geom_text_wordcloud
#' @importFrom ggplot2 ggsave labs scale_size_area theme_minimal ggplot aes
#' @importFrom ggplot2 scale_colour_identity scale_color_gradient
#' @importFrom dplyr rename left_join filter bind_rows mutate
#' @noRd
create_plots <- function(
    df_list = NULL,
    summary = NULL,
    ngrams = NULL,
    test = NULL,
    test_type = NULL,
    cor_var = NULL,
    popout = NULL,
    color_negative_cor = NULL,
    color_positive_cor = NULL,
    grid_pos = "",
    scale_size = FALSE,
    plot_topics_idx = NULL,
    p_alpha = NULL,
    highlight_topic_words = c("not", "never"),
    save_dir,
    figure_format = "svg",
    width = 10,
    height = 8,
    max_size = 10,
    seed = 42,
    word_font  = "sans",
    title_font = "sans"
){
  
  # Dependencies
  for(pkg in c("ggplot2", "ggwordcloud", "dplyr", "tibble", "rlang")){
    if(!requireNamespace(pkg, quietly = TRUE))
      stop(sprintf("Package '%s' is required but not installed.", pkg), call. = FALSE)
  }
  
  # Pre-processing
  if(is.null(plot_topics_idx)){
    plot_topics_idx <- seq(1, length(df_list))
  }else{plot_topics_idx <- plot_topics_idx}
  
  plots <- list()
  
  # CASE 1: Topic models + test
  if(!is.null(df_list) && !is.null(summary) && !is.null(test)){
    # Column extractors for test-results 
    split_vars <- strsplit(cor_var, "__", fixed = TRUE)[[1]]
    get_stats <- function(topic_id){
      if(is.null(test)) return(list(est = NA, p = NA))
      row <- dplyr::filter(tibble::as_tibble(test, .name_repair = "minimal"), 
                           topic == paste0("t_", topic_id))
      if(length(split_vars) == 1){
        list(
          est = row[[grep(paste0(cor_var, ".estimate"),names(row))]],
          p   = row[[grep(paste0(cor_var, ".p_adjusted"),names(row))]]
        )
      } else {
        est_x <- row[[grep(paste0(split_vars[1],".estimate"),names(row))]]
        est_y <- row[[grep(paste0(split_vars[2],".estimate"),names(row))]]
        p_x   <- row[[grep(paste0(split_vars[1],".p_adjusted"),names(row))]]
        p_y   <- row[[grep(paste0(split_vars[2],".p_adjusted"),names(row))]]
        list(est = c(x = est_x, y = est_y), p = c(x = p_x, y = p_y), na.rm = TRUE)
      }
    }
    
    for(topic in plot_topics_idx){
      stats <- get_stats(topic)
      est   <- stats$est
      p_vec <- stats$p
      p_val_min <- min_p(p_vec)
      cur_alpha <- if(is.null(p_alpha)) p_val_min + 1 else p_alpha
      is_pop <- !is.null(popout) && paste0("t_",topic) %in% popout$topic
      if(!is.nan(p_val_min) && (p_val_min < cur_alpha || is_pop)){
        idx   <- as.numeric(sub(".*_", "", topic))
        data  <- df_list[[idx]]
        if(!is.null(highlight_topic_words))
          data <- highlight_neg(data)
        # size scaling
        max_size_topic <- if(scale_size){
          prev <- summary[topic, "prevalence"][[1]]
          max_size * log(prev)
        } else max_size
        scheme <- if (est[1] < 0) color_negative_cor else color_positive_cor
        plot   <- build_cloud(
          data, scheme, max_size_topic,
          word_font = word_font,
          title_font = title_font
        )
        # axis annotations
        x_lab  <- if(length(est) == 1){
          sprintf("r = %.4f", est)
        } else {
          sprintf("r_x = %.4f\nr_y = %.4f", est["x"], est["y"])
        }
        plot   <- plot + ggplot2::labs(x = x_lab)
        # file name stub
        stub <- if(length(est) == 1){
          sprintf("%scorvar_%s_%s_r_%.4f_p_%s",
                  if(grid_pos != "") sprintf("grid_pos_%s_", grid_pos) else "",
                  cor_var, topic, est, format_p(p_vec))
        } else {
          sprintf("%scorvar_%s_%s_rx_%.4f_ry_%.4f_px_%s_py_%s",
                  if(grid_pos != "") sprintf("grid_pos_%s_", grid_pos) else "",
                  cor_var, topic, est["x"], est["y"],
                  format(p_vec["x"], scientific = TRUE, digits = 2),
                  format(p_vec["y"], scientific = TRUE, digits = 2))
        }
        save_plot(plot, stub, 't_', save_dir, figure_format, width, height, seed)
        plots[[paste0("t_",topic)]] <- plot
      }
    }
  }
  
  # CASE 2: Topic models alone
  if(!is.null(df_list) && !is.null(summary) && is.null(test)){
    for(topic in plot_topics_idx){
      idx      <- as.numeric(sub(".*_", "", topic))
      data     <- df_list[[idx]]
      max_size_topic <- if(scale_size){
        prev <- summary[topic, "prevalence"][[1]]
        max_size * log(prev)
      } else max_size
      plot <- build_cloud(
        data, color_negative_cor, max_size_topic,
        word_font = word_font,
        title_font = title_font
      )
      save_plot(plot, topic, 't_', save_dir, figure_format, width, height, seed)
      plots[[paste0('t_',as.character(topic))]] <- plot
    }
  }
  
  # CASE 3: N-grams (no topic model) 
  if(is.null(df_list) && !is.null(ngrams)){
    if(is.null(test)){
      plot <- ggplot2::ggplot(ngrams, ggplot2::aes(label = ngrams, size = prevalence, color = prevalence)) +
        ggwordcloud::geom_text_wordcloud(show.legend = TRUE, family = word_font) +
        ggplot2::theme_minimal(base_family = title_font) +
        color_positive_cor
      save_plot(plot, "ngrams", '', save_dir, figure_format, width, height, seed)
      plots <- plot
    } else {
      ng  <- ngrams %>% dplyr::rename(top_terms = ngrams)
      tst <- test   %>% dplyr::rename(estimate   = dplyr::contains("estimate"),
                                      p_adjusted = dplyr::contains("p_adjusted")) %>%
        dplyr::left_join(ng, by = "top_terms")
      if(!is.null(p_alpha))
        tst <- dplyr::filter(tst, p_adjusted < p_alpha)
      if(nrow(tst) == 0){
        message("No significant terms. No wordclouds generated.")
      } else {
        make_ngram_plot <- function(df, scheme){
          prevalence_col <- grep("^prevalence", names(df), value = TRUE)[1]
          
          size_legend_title <- "prevalence"
          color_legend_title <- "estimate"
          
          ggplot2::ggplot(df, 
                          ggplot2::aes(label = top_terms, 
                                       size = .data[[prevalence_col]], 
                                       color = abs(estimate))) +
            ggwordcloud::geom_text_wordcloud(show.legend = TRUE, family = word_font) +
            ggplot2::scale_size_area(max_size = max_size, name = size_legend_title)+
            ggplot2::labs(color = color_legend_title) +
            ggplot2::theme_minimal(base_family = title_font) +
            scheme
        }
        pos  <- make_ngram_plot(df = dplyr::filter(tst, estimate > 0), scheme = color_positive_cor)
        neg  <- make_ngram_plot(dplyr::filter(tst, estimate < 0), color_negative_cor)
        save_plot(pos, "ngrams_positive", '', save_dir, figure_format, width, height, seed)
        save_plot(neg, "ngrams_negative", '', save_dir, figure_format, width, height, seed)
        plots <- list(positive_association = pos, negative_association = neg)
      }
    }
  }
  return(plots)
}

#' This is a private function
#' @param df_list (list) list of data.frames with topics most frequent words and assigned topic term scores
#' @param test (data.frame) the test returned from textTopicTest()
#' @param test_type (string) "linear_regression", or "binary_regression"
#' @param cor_var (string) Variable for t-test, linear, or binary/logistic regression
#' @param popout (tibble) The tibble containing topic idx to popout
#' @param color_negative_cor (function) color of topic cloud with negative correlation
#' @param color_positive_cor (function) color of topic cloud with positive correlation
#' @param grid_pos (numeric) position of grid plots
#' @param scale_size (bool) if True, then the size of the topic cloud is scaled by the prevalence of the topic
#' @param plot_topics_idx (list) if specified, then only the specified topics are plotted
#' @param p_alpha (float) set threshold which determines which topics are plotted
#' @param highlight_topic_words (str vector) The dictionary to popout negative words to an individual plot for easier reading. 
#'  Default words are "not", "never". 
#'  The values of the vector determine the color code to popout. The color values can be different for different words.
#' @param save_dir (string) save plots in specified directory, if left blank, plots is not saved,
#' thus save_dir is necessary.
#' @param figure_format (string) Set the figure format, e.g., .svg, or .png.
#' @param seed (int) seed is needed for saving the plots in the correct directory
#' @importFrom ggwordcloud geom_text_wordcloud
#' @importFrom ggplot2 ggsave labs scale_size_area theme_minimal ggplot aes scale_color_gradient scale_colour_identity
#' @importFrom dplyr rename
#' @noRd
create_plots_old <- function(
    df_list = NULL,
    summary = NULL,
    ngrams = NULL,
    test = NULL,
    test_type = NULL,
    cor_var = NULL,
    popout = NULL,
    color_negative_cor = NULL,
    color_positive_cor = NULL,
    grid_pos = "",
    scale_size = FALSE,
    plot_topics_idx = NULL,
    p_alpha = NULL,
    highlight_topic_words = c("not", "never"),
    save_dir,
    figure_format = "svg",
    width = 10,
    height = 8,
    max_size = 10,
    seed = 42){
  
  plot_list <- list()
  # this code plots the wordclouds in respect to the statistical test
  # For test, topics list and summary
  if(!is.null(test) & !is.null(df_list) & !is.null(summary)){
    
    if (is.null(plot_topics_idx)){
      grid1 <- ""
      plot_topics_idx <- seq(1, length(df_list))
    } else {
      grid1 <- paste0("grid_pos_", grid_pos, "_")
      pred_var_x <- strsplit(cor_var, "__")[[1]][1]
      if (length(strsplit(cor_var, "__")[[1]]) > 1){
        pred_var_y <- strsplit(cor_var, "__")[[1]][2]
      }
    }
    
    for (i in paste0('t_', plot_topics_idx)){ 
      #for (i in 1:length(df_list)){
      #view(df_list[[i]])
      if (test_type == "linear_regression" | test_type == "logistic_regression"){
        if (grid1 == ""){
          colNo.estimate <- grep(paste0(cor_var, '.estimate'),colnames(test))
          colNo.p_adjusted <- grep(paste0(cor_var, '.p_adjusted'),colnames(test))
          estimate_col <- colnames(test)[colNo.estimate] 
          p_adjusted_col <- colnames(test)[colNo.p_adjusted]
          #estimate_col <- paste0(cor_var,".estimate") # grep(partial_name, data_frame_names, value = TRUE)
          #p_adjusted_col <- paste0(cor_var,".p_adjusted")
        } else {
          colNo.estimate.x <- grep(paste0(pred_var_x, '.estimate'),colnames(test))
          colNo.p_adjusted.x <- grep(paste0(pred_var_x, '.p_adjusted'),colnames(test))
          estimate_col_x <- colnames(test)[colNo.estimate.x] 
          p_adjusted_col_x <- colnames(test)[colNo.p_adjusted.x]
          
          # Why is it doing this? Its not reliable to split it up based on "_"
          if (length(strsplit(cor_var, "__")[[1]]) > 1){
            colNo.estimate.y <- grep(paste0(pred_var_y, '.estimate'),colnames(test))
            colNo.p_adjusted.y <- grep(paste0(pred_var_y, '.p_adjusted'),colnames(test))
            estimate_col_y <- colnames(test)[colNo.estimate.y] 
            p_adjusted_col_y <- colnames(test)[colNo.p_adjusted.y]
          }}
      } else if (test_type == "t-test"){
        estimate_col <- "cohens d" # probably doesn't work yet
        
      } #else if (test_type == "logistic_regression"){
      #estimate_col <- "estimate"
      #p_adjusted_col <- "p_adjusted"
      #}
      if (grid1 == ""){
        estimate <- dplyr::filter(tibble::as_tibble(test,.name_repair='minimal'), topic==i)[[estimate_col]] # $PHQtot.estimate
        p_adjusted <- dplyr::filter(tibble::as_tibble(test,.name_repair='minimal'), topic==i)[[p_adjusted_col]] # $PHQtot.p_adjustedfdr
      }else{
        estimate_x <- dplyr::filter(tibble::as_tibble(test,.name_repair='minimal'), topic==i)[[estimate_col_x]]
        p_adjusted_x <- dplyr::filter(tibble::as_tibble(test,.name_repair='minimal'), topic==i)[[p_adjusted_col_x]]
        estimate <- estimate_x
        if (length(strsplit(cor_var, "__")[[1]]) > 1){
          estimate_y <- dplyr::filter(tibble::as_tibble(test,.name_repair='minimal'), topic==i)[[estimate_col_y]]
          p_adjusted_y <- dplyr::filter(tibble::as_tibble(test,.name_repair='minimal'), topic==i)[[p_adjusted_col_y]]
          p_adjusted <- min(p_adjusted_x, p_adjusted_y)
        }else{
          estimate <- estimate_x
          p_adjusted <- p_adjusted_x
        }
      }
      
      
      if (scale_size==TRUE){
        prevalence <- summary[paste0("t_",i),]$prevalence
      }
      
      
      # this will ensure that all topics are plotted
      if (is.null(p_alpha) ){
        if (grid1 == ""){
          p_alpha <- p_adjusted +1 
        }
      }
      
      
      if ((!is.nan(p_adjusted) & p_adjusted < p_alpha) || i %in% popout$topic){
        
        #estimate <- test[i,][[grep(estimate_col, colnames(test), value=TRUE)]]# $PHQtot.estimate
        #p_adjusted <- test[i,][[grep("p_adjusted", colnames(test), value=TRUE)]] # $PHQtot.p_adjustedfdr
        if (estimate < 0){
          color_scheme <- color_negative_cor # scale_color_gradient(low = "darkgreen", high = "green")
        } else {
          color_scheme <- color_positive_cor # scale_color_gradient(low = "darkred", high = "red")
        }
        if (scale_size == TRUE){
          max_size <- max_size*log(prevalence)
          y <- paste0("P = ", prevalence)
        } else {
          max_size <- max_size
          y <- ""
        }
        
        # For constant bold and itatlic format of negative words. 
        if (!is.null(highlight_topic_words) && is.vector(highlight_topic_words)){
          if(is.character(highlight_topic_words) && is.vector(highlight_topic_words)){
            # Assign color to neg_words in the dictionary object "highlight_topic_words"
            df_list_separated <- separate_neg_words(df_list[[as.numeric(sub(".*_", "", i))]], highlight_topic_words)
            df_list_separated[[2]] <- dplyr::mutate(df_list_separated[[2]], source = "negDic") 
            #df_list_separated[[1]]$color <- df_list_separated[[1]]$phi^3 / sum(df_list_separated[[1]]$phi^3) 
            #df_list_separated[[1]]$color <- color_scheme$palette(df_list_separated[[1]]$color)
            colnames(df_list_separated[[2]])[3] <- c('color') 
            df_list_separated[[1]] <- dplyr::mutate(df_list_separated[[1]], source = "notNegDic")
            target_topic <- dplyr::bind_rows(df_list_separated[[1]],df_list_separated[[2]]) 
            target_topic <- target_topic %>%
              dplyr::mutate(label_content = if_else(source == "negDic",
                                                    sprintf("<b><i>%s</i></b>", Word),
                                                    Word))
            target_topic$phi_ori <- target_topic$phi
            target_topic$phi <- target_topic$phi^1.1 / sum(target_topic$phi^1.1) # Make the differences of size and color more obvious.
          }else{stop('Invalid settings for the parameter "highlight_topic_words".\nConsider use the default option!\n')}
        }else{target_topic <- df_list[[as.numeric(sub(".*_", "", i))]]}
        
        if (grid1 == ""){ .
          plot <- ggplot2::ggplot(target_topic, 
                                  ggplot2::aes(label = Word, 
                                               size = phi, 
                                               color = phi)) + #,x=estimate)) +
            ggwordcloud::geom_text_wordcloud() +
            ggplot2::scale_size_area(max_size = max_size) +
            ggplot2::theme_minimal() +
            #theme(plot.margin = margin(0,0,0,0, "cm")) +
            color_scheme + 
            ggplot2::labs(x = paste0("r = ", estimate),
                          y= y)
        } else {
          
          if (length(strsplit(cor_var, "__")[[1]]) > 1){
            x_message = paste0("r_x = ", round(estimate_x,4), "\n",
                               "r_y = ", round(estimate_y,4))
          } else {
            
            x_message = paste0("r_x = ", round(estimate_x,4))                    
          }
          if (!is.null(highlight_topic_words) && is.vector(highlight_topic_words)){
            plot <- ggplot2::ggplot(target_topic, 
                                    ggplot2::aes(label = Word, 
                                                 size = phi, 
                                                 color = color_scheme$palette(phi),
                                                 label_content = label_content
                                    ))  # +,x=estimate)) +
          }else{
            plot <- ggplot2::ggplot(target_topic, 
                                    ggplot2::aes(label = Word, 
                                                 size = phi, 
                                                 color = phi))
          }
          
          plot <- plot + ggwordcloud::geom_text_wordcloud() + 
            ggplot2::scale_size_area(max_size = max_size)
          
          if (!is.null(highlight_topic_words) && is.vector(highlight_topic_words)){
            plot <- plot + ggplot2::scale_colour_identity() + 
              ggplot2::theme_minimal() 
          }else{
            plot <- plot + ggplot2::theme_minimal() +
              #theme(plot.margin = margin(0,0,0,0, "cm")) +
              color_scheme
          }
          plot <- plot + ggplot2::labs(x = x_message, y = y)
        }
        
        if (!is.null(save_dir)){
          if (!dir.exists(save_dir)) {
            
            # Create the directory
            dir.create(save_dir)
            msg <- "Directory created successfully.\n"
            message(colourise(msg, "green"))
          } 
          if(!dir.exists(paste0(save_dir, "/seed_", seed, "/wordclouds"))){
            dir.create(paste0(save_dir, "/seed_", seed, "/wordclouds"))
          }
          
          p_adjusted <- sprintf("%.2e", p_adjusted)
          
          if (grid1 == ""){
            ggplot2::ggsave(paste0(save_dir,"/seed_", seed, 
                                   "/wordclouds/",
                                   grid1,
                                   "corvar_", cor_var,"_",
                                   i, "_r_", 
                                   estimate, "_p_", 
                                   p_adjusted,
                                   ".",
                                   figure_format),
                            plot = plot, 
                            width = width, 
                            height = height, 
                            units = "in", 
                            create.dir = TRUE)
          } else {
            
            if (length(strsplit(cor_var, "__")[[1]]) > 1){
              p_adjusted_x <- sprintf("%.2e", p_adjusted_x)
              p_adjusted_y <- sprintf("%.2e", p_adjusted_y)
              fileMsg <- paste0(
                "_rx_", estimate_x,
                "_ry_", estimate_y, 
                "_px_", p_adjusted_x,
                "_py_", p_adjusted_y, ".",
                figure_format
              )
            } else {
              
              p_adjusted_x <- sprintf("%.2e", p_adjusted_x)
              fileMsg <- paste0(
                "_rx_", estimate_x, 
                "_px_", p_adjusted_x, ".",
                figure_format
              )
            }
            
            if (is.null(popout)){fileHead <- ''} else {
              if (i %in% popout$topic){fileHead <- '0_scatter_emphasised_'}else{fileHead <- ''}
            }
            
            ggplot2::ggsave(paste0(save_dir,"/seed_", seed, 
                                   "/wordclouds/", 
                                   fileHead,
                                   grid1,
                                   "corvar_", cor_var,"_",
                                   i, fileMsg),
                            plot = plot, 
                            width = width, 
                            height = height, 
                            units = "in", 
                            create.dir = TRUE)
          }
        }
      }
      plot_list[[i]] <- plot
    }
  }
  
  
  # For topic df_list, summary but no test
  if (!is.null(df_list) & !is.null(summary) & is.null(test)){
    if (is.null(plot_topics_idx)){
      plot_topics_idx <- seq(1, length(df_list))
    }
    if (scale_size==TRUE){
      prevalence <- summary[paste0("t_",i),]$prevalence
      max_size <- max_size*log(prevalence)
    } else {
      max_size <- max_size
    }
    
    
    if (!is.null(save_dir)){
      if (!dir.exists(save_dir)) {
        # Create the directory
        dir.create(save_dir)
        msg <- "Directory created successfully.\n"
        message(colourise(msg, "green"))
      } 
      if(!dir.exists(paste0(save_dir, "/seed_", seed, "/wordclouds"))){
        dir.create(paste0(save_dir, "/seed_", seed, "/wordclouds"))
      }
    }
    
    for (i in paste0('t_', plot_topics_idx)){
      
      plot <- ggplot2::ggplot(
        df_list[[as.numeric(sub(".*_", "", i))]],
        ggplot2::aes(label = Word,
                     size = phi,
                     #label_content = label_content,
                     color = phi)) +
        ggwordcloud::geom_text_wordcloud() +
        ggplot2::scale_size_area(max_size = max_size) +
        ggplot2::theme_minimal() +
        color_negative_cor
      
      if (!is.null(save_dir)){
        ggplot2::ggsave(paste0(save_dir,"/seed_", seed, 
                               "/wordclouds/",
                               i, 
                               ".",
                               figure_format),
                        plot = plot, 
                        width = width, 
                        height = height, 
                        units = "in", 
                        create.dir = TRUE)
      }
      
      plot_list[[i]] <- plot
    }
  }
  
  # For N-grams with NO test
  if (is.null(df_list) & is.null(test) & !is.null(ngrams)){
    
    if (!is.null(save_dir)){
      if (!dir.exists(save_dir)) {
        # Create the directory
        dir.create(save_dir)
        msg <- "Directory created successfully.\n"
        message(colourise(msg, "green"))
      } 
      if(!dir.exists(paste0(save_dir, "/seed_", seed, "/wordclouds"))){
        dir.create(paste0(save_dir, "/seed_", seed, "/wordclouds"))
      }
    }
    
    plot <- ggplot2::ggplot(ngrams, 
                            ggplot2::aes(label = ngrams, 
                                         size = prevalence, 
                                         color = prevalence)) +
      ggwordcloud::geom_text_wordcloud(show.legend = TRUE) +
      ggplot2::theme_minimal() +
      color_positive_cor # ggplot2::scale_color_gradient(low = "yellow", high = "red")
    
    if (!is.null(save_dir)){
      ggplot2::ggsave(paste0(save_dir,"/seed_", seed, 
                             "/wordclouds/ngrams", 
                             ".",
                             figure_format),
                      plot = plot, 
                      width = width, 
                      height = height, 
                      units = "in", 
                      create.dir = TRUE)
    }
    
    plot_list <- plot
  } 
  
  # For N-gram with test
  if (is.null(df_list) & !is.null(test) & !is.null(ngrams)){
    
    ngrams <- ngrams %>% dplyr::rename(top_terms = ngrams)
    test <- test %>% dplyr::rename(estimate = contains("estimate"))
    test <- test %>% dplyr::rename(p_adjusted = contains("p_adjusted"))
    test <- test %>% dplyr::left_join(ngrams, by = "top_terms")
    if (!is.null(p_alpha)){
      test <- test %>% dplyr::filter(p_adjusted < p_alpha)
    }
    # test for the fact that all words could be insignificant
    if (nrow(test) == 0){
      msg <- "No significant terms.\n No wordclouds generated."
      message(colourise(msg, "blue"))
    } else {
      test_positive <- test%>% dplyr::filter(estimate > 0)
      test_negative <- test%>% dplyr::filter(estimate < 0)
      
      if (!is.null(save_dir)){
        if (!dir.exists(save_dir)) {
          # Create the directory
          dir.create(save_dir)
          msg <- "Directory created successfully.\n"
          message(colourise(msg, "green"))
        } 
        if(!dir.exists(paste0(save_dir, "/seed_", seed, "/wordclouds"))){
          dir.create(paste0(save_dir, "/seed_", seed, "/wordclouds"))
        }
      }
      
      # Word cloud with correlation strength mapped to color gradient
      plot1 <- ggplot2::ggplot(test_positive, 
                               ggplot2::aes(label = top_terms,
                                            size = prevalence,
                                            color = estimate)) +
        ggwordcloud::geom_text_wordcloud(show.legend = TRUE) +
        ggplot2::scale_size_area(max_size = max_size) +  # Adjust max size
        #scale_color_gradient(low = "grey", high = "red") +  # Blue for low, red for high correlation strength
        ggplot2::theme_minimal() +
        color_positive_cor
      
      if (!is.null(save_dir)){
        ggplot2::ggsave(paste0(save_dir,"/seed_", seed, 
                               "/wordclouds/ngrams_positive", 
                               ".",
                               figure_format),
                        plot = plot1, 
                        width = width, 
                        height = height, 
                        units = "in", 
                        create.dir = TRUE) 
      }
      
      # Word cloud with correlation strength mapped to color gradient
      plot2 <- ggplot2::ggplot(test_negative, 
                               ggplot2::aes(label = top_terms,
                                            size = prevalence,
                                            color = abs(estimate))) +
        ggwordcloud::geom_text_wordcloud(show.legend = TRUE) +
        ggplot2::scale_size_area(max_size = max_size) +  # Adjust max size
        #scale_color_gradient(low = "grey", high = "red") +  # Blue for low, red for high correlation strength
        ggplot2::theme_minimal() +
        color_negative_cor
      
      if (!is.null(save_dir)){
        ggplot2::ggsave(paste0(save_dir,"/seed_", seed, 
                               "/wordclouds/ngrams_negative", 
                               ".",
                               figure_format),
                        plot = plot2, 
                        width = width, 
                        height = height, 
                        units = "in", 
                        create.dir = TRUE)
      }
      
      plot_list[[1]] <- plot1
      plot_list[[2]] <- plot2
      names(plot_list) <- c("positive", "negative")
    }
  }
  return(plot_list)
  
}

check_java_installed <- function(verbose = TRUE) {
  
  java_path <- Sys.which("java")[[1]]
  has_java <- java_path != ""
  
  summary_lines <- character()
  
  if (!has_java) {
    summary_lines <- c(
      "Java is NOT installed or not found in your system PATH.",
      "To install Java:",
      "  - On macOS: brew install openjdk",
      "  - On Ubuntu/Debian: sudo apt-get install default-jdk",
      "  - On Windows: Install Java from https://www.java.com/"
    )
  } else {
    # Try running `java -version` to confirm it's accessible
    java_version_output <- tryCatch(
      system2("java", args = "-version", stderr = TRUE, stdout = TRUE),
      error = function(e) NA
    )
    
    if (is.na(java_version_output)[[1]]) {
      summary_lines <- c(
        "Java is found, but 'java -version' failed to run.",
        "There may be permission or PATH issues."
      )
      has_java <- FALSE
    } else {
      summary_lines <- c("Java is installed.")
      if (length(java_version_output) > 0) {
        summary_lines <- c(summary_lines, paste("Version info:", java_version_output[1]))
      }
    }
  }
  
  if (verbose) message(paste(summary_lines, collapse = "\n"))
  
  list(
    java_found = has_java,
    java_path = if (has_java) java_path else NULL,
    version_info = if (has_java) java_version_output else NULL,
    summary_lines = summary_lines
  )
}


#' @importFrom utils packageVersion
#' @noRd
.onAttach <- function(libname, pkgname) {
  
  if (!grepl(x = R.Version()$arch, pattern = "64")) {
    warning("\n\nThe topic package requires running R on a 64-bit systems.")
  }

  log <- check_java_installed(verbose = FALSE)
  
  if(log$java_found){
    java_msg <- "" 
  } else {
    java_msg <- "\nPlease note that the topics package requires you to download and install java from www.java.com. \n" 
  }
  
  topics_version_nr <- tryCatch(
    {
      topics_version_nr1 <- paste(" (version ", 
                                packageVersion("topics"), ")", 
                                sep = "")
    },
    error = function(e) {
      topics_version_nr1 <- ""
    }
  )
  
  nowarranty <- c("The topics package is provided 'as is' without any warranty of any kind. \n")
  
  
  packageStartupMessage(
    colourise(
      paste("\nThis is topics: your text's new best friend",
            topics_version_nr,
        ".\n",
        sep = ""
      ),
      fg = "blue", bg = NULL
    ),
    colourise(java_msg,
      fg = "brown", bg = NULL
    ),
    colourise(nowarranty,
              fg = "purple", bg = NULL
    ),
    colourise("\nFor more information about the topics package see www.r-topics.org and www.r-text.org.",
      fg = "green", bg = NULL
    )
  )
}


# Below function is from testthat:
# https://github.com/r-lib/testthat/blob/717b02164def5c1f027d3a20b889dae35428b6d7/R/colour-text.r
#' Colourise text for display in the terminal.
#'
#' If R is not currently running in a system that supports terminal colours
#' the text will be returned unchanged.
#'
#' Allowed colours are: black, blue, brown, cyan, dark gray, green, light
#' blue, light cyan, light gray, light green, light purple, light red,
#' purple, red, white, yellow
#'
#' @param text character vector
#' @param fg foreground colour, defaults to white
#' @param bg background colour, defaults to transparent
# @examples
#' @noRd
colourise <- function(text, fg = "black", bg = NULL) {
  term <- Sys.getenv()["TERM"]
  colour_terms <- c("xterm-color", "xterm-256color", "screen", "screen-256color")

  if (rcmd_running() || !any(term %in% colour_terms, na.rm = TRUE)) {
    return(text)
  }

  col_escape <- function(col) {
    paste0("\033[", col, "m")
  }

  col <- .fg_colours[tolower(fg)]
  if (!is.null(bg)) {
    col <- paste0(col, .bg_colours[tolower(bg)], sep = ";")
  }

  init <- col_escape(col)
  reset <- col_escape("0")
  paste0(init, text, reset)
}

.fg_colours <- c(
  "black" = "0;30",
  "blue" = "0;34",
  "green" = "0;32",
  "cyan" = "0;36",
  "red" = "0;31",
  "purple" = "0;35",
  "brown" = "0;33"
  # "light gray" = "0;37",
  # "dark gray" = "1;30",
  # "light blue" = "1;34",
  # "light green" = "1;32",
  # "light cyan" = "1;36",
  # "light red" = "1;31",
  # "light purple" = "1;35",
  # "yellow" = "1;33",
  # "white" = "1;37"
)

.bg_colours <- c(
  "black" = "40",
  "red" = "41",
  "green" = "42",
  "brown" = "43",
  "blue" = "44",
  "purple" = "45",
  "cyan" = "46",
  "light gray" = "47"
)

rcmd_running <- function() {
  nchar(Sys.getenv("R_TESTS")) != 0
}

#' get_mallet_model
#' @param dtm (R_obj) A document-term matrix
#' @param num_topics (integer) The number of topics
#' @param num_top_words (string) The number of words
#' @param num_iterations (string) Number of iterations
#' @param seed Set see with L, like 42L.
#' @return Mallet model
#' @importFrom textmineR Dtm2Docs CalcGamma
#' @importFrom stats setNames
# @importFrom mallet mallet.top.words mallet.doc.topics mallet.word.freqs mallet.topic.labels MalletLDA  mallet.import  mallet.topic.words
#' @importFrom utils combn
#' @noRd
get_mallet_model <- function(
    dtm,
    num_topics = 20,
    num_top_words = 10, 
    num_iterations = 1000, 
    seed){
  
  
  if (!requireNamespace("mallet", quietly = TRUE)) {
    stop(paste0("JAVA and the R-packages 'rJava' and 'mallet' are required but one or more are not installed. \n", 
                "Please install JAVA from https://www.java.com/. \n", 
                "And then run install.packages(c('mallet', 'rJava')."))
  }
  
  docs <- textmineR::Dtm2Docs(dtm)
  
  model <- mallet::MalletLDA(
    num.topics = num_topics,
    alpha.sum = 5,
    beta = 0.01)
  
  instances <- mallet::mallet.import(
    as.character(seq_along(docs)),
    docs,
    preserve.case = FALSE,
    token.regexp = "[\\p{L}\\p{N}_]+|[\\p{P}]+\ ")
  
  # set seed (it does not make the entire ldaModel() output 
  # https://stackoverflow.com/questions/37356001/using-a-random-seed-in-rmallet
  # object setqual TRUE, but many more objectis within the object becomes equal)
  model$setRandomSeed(as.integer(seed))
  model$loadDocuments(instances)
  
  model$train(num_iterations)
  
  topic.words <- mallet::mallet.topic.words(
    model,
    smoothed = TRUE,
    normalized = TRUE)
  
  df_top_terms <- data.frame(matrix(NA, 
                                    nrow = num_top_words, 
                                    ncol = num_topics))
  
  colnames(df_top_terms) <- paste0("t_", 
                                   1:num_topics)
  
  for(i_topic in 1:num_topics){
    top_terms <- mallet::mallet.top.words(
      model,
      word.weights = topic.words[i_topic,],
      num.top.words = num_top_words)
    
    #top_terms <- paste(top_terms$term, collapse=" ")
    #topic <- paste("t_", i_topic, sep="")
    #df <- tibble(topic, top_terms)
    #list_top_terms[[i_topic]] <- df
    top_terms <- top_terms$term
    df_top_terms[paste0("t_", i_topic)] <- top_terms
  }
  
  return_model <- list()
  #return_model$top_terms_mallet <- bind_rows(list_top_terms)
  return_model$instances <- instances #model$instances()
  return_model$inferencer <- model$getInferencer()
  #view(return_model$instances)
  return_model$top_terms_mallet <- df_top_terms
  return_model$top_terms <- return_model$top_terms_mallet
  #return_model$phi <- mallet.topic.w
  return_model$phi <- mallet::mallet.topic.words(model, 
                                                 smoothed=TRUE, 
                                                 normalized=TRUE)
  
  return_model$topic_docs <- mallet::mallet.doc.topics(model, 
                                                       smoothed=TRUE,
                                                       normalized=TRUE)
  #return_model$top_terms <- GetTopTerms(phi = return_model$phi, M = num_top_words)
  return_model$frequencies <- mallet::mallet.word.freqs(model)
  return_model$vocabulary <- model$getVocabulary()
  
  model$prevalence_mallet <- colSums(mallet::mallet.doc.topics(
    model,
    smoothed=TRUE,
    normalized=TRUE)) /
    sum(mallet::mallet.doc.topics(
      model,
      smoothed=TRUE,
      normalized=TRUE)) * 100
  
  #sum(list_top_terms$prevalence)
  return_model$labels <- mallet::mallet.topic.labels(model)
  return_model$theta <- mallet::mallet.doc.topics(
    model,
    smoothed = TRUE,
    normalized = TRUE)
  
  return_model$prevalence <- colSums(return_model$theta) / 
    sum(return_model$theta) * 100
  
  keys <- paste0("t_", 1:num_topics)
  
  return_model$prevalence <- stats::setNames(
    return_model$prevalence, 
    keys)
  
  #return_model$labels <- LabelTopics(assignments = return_model$theta > 0.05, dtm = dtm, M = 1)
  
  #### Compute a Coherence Score ####
  
  # Step 1: Calculate Term Co-occurrence Matrix from the DTM
  # Convert the document-term matrix (DTM) to a binary form indicating term presence/absence
  # Multiplying by 1 converts the TRUE values to 1 and FALSE values to 0.
  binary_dtm <- (dtm > 0) * 1
  
  # Compute the co-occurrence matrix by multiplying the transposed binary DTM with itself
  # This results in a matrix where each entry [i, j] represents the number of documents where both terms i and j co-occur
  co_occurrence <- Matrix::t(binary_dtm) %*% binary_dtm
  
  # Step 2: Compute Total Number of Documents
  N <- nrow(dtm)  # The total number of documents in the corpus
  
  # Step 3: Calculate the Probability of Each Individual Term
  # p_wi is a vector where each entry represents the proportion of documents containing a specific term
  p_wi <- Matrix::diag(co_occurrence) / N
  
  # Step 4: Define a Function to Compute Coherence for a Topic
  compute_topic_coherence <- function(terms, co_occurrence, p_wi, N) {
    
    # Get the indices of the terms in the DTM's vocabulary
    term_indices <- match(terms, colnames(dtm))
    
    # Create all possible pairs of the term indices
    term_pairs <- utils::combn(term_indices, 2)
    
    # Calculate PMI (Pointwise Mutual Information) for each pair of terms
    pmi_values <- apply(term_pairs, 2, function(pair) {
      w1 <- pair[1]
      w2 <- pair[2]
      
      # Ensure both term indices are valid and co-occur in at least one document
      if (!is.na(w1) && !is.na(w2) && co_occurrence[w1, w2] > 0) {
        
        # Calculate the joint probability of terms w1 and w2
        p_wi_wj <- co_occurrence[w1, w2] / N
        
        # Calculate the PMI score for the pair
        pmi <- log(p_wi_wj / (p_wi[w1] * p_wi[w2]))
        
        return(pmi)
      } else {
        return(0)  # Return 0 if terms do not co-occur or indices are invalid
      }
    })
    
    # Return the average PMI for all term pairs in the topic
    mean(pmi_values)
  }
  
  # Step 5: Compute Coherence Scores for Each Topic
  coherence_scores <- sapply(1:num_topics, function(i) {
    
    # Get the top terms for the current topic from the `df_top_terms` data frame
    top_terms <- df_top_terms[, i]
    
    # Compute the coherence score for the current topic using the defined function
    compute_topic_coherence(top_terms, co_occurrence, p_wi, N)
  })
  
  # Step 6: Add the Computed Coherence Scores to the `return_model` List
  return_model$coherence <- coherence_scores
  
  ##############
  
  # put theta into the right format
  df_theta <- data.frame(return_model$theta)
  df_theta <- stats::setNames(
    df_theta,
    keys)
  
  return_model$theta <- df_theta
  
  # take the first word as dummy label, no other solution worked
  first_row <- return_model$top_terms_mallet[1,]
  return_model$labels <- matrix(first_row, 
                                nrow = num_topics, 
                                ncol = 1)
  
  rownames(return_model$labels) <- paste0("t_", 
                                          1:num_topics)
  
  colnames(return_model$labels) <- "label_1"
  #return(c(result1 = result1, result2 = result2))
  
  pred_model <- list()
  pred_model$phi <- mallet::mallet.topic.words(
    model,
    smoothed = TRUE,
    normalized = TRUE)
  
  colnames(pred_model$phi) <- as.character(unlist(
    return_model$vocabulary))
  
  pred_model$theta <- mallet::mallet.doc.topics(
    model, 
    smoothed = TRUE, 
    normalized = TRUE)
  
  k <- ncol(pred_model$theta) # Specify the value of k  
  new_col_names <- paste("t", 
                         1:k, 
                         sep = "_") # Generate new column names
  
  colnames(pred_model$theta) <- new_col_names # Assign new column names to the dataframe
  pred_model$alpha <- model$alpha
  
  pred_model$gamma <- textmineR::CalcGamma(
    phi = pred_model$phi, 
    theta = pred_model$theta)
  
  pred_model$data <- dtm
  #names(pred_model)
  class(pred_model) <- "lda_topic_model"
  
  return_model$pred_model <- pred_model
  
  #return(list(pred_model = pred_model, return_model=return_model))
  return(return_model)
}



#' Topic modelling
#' 
#' The function to create and train and an LDA model.
#' @param dtm (R_obj) The document term matrix -> output of topicsDtm function
#' @param num_topics (integer) The number of topics to be created
#' @param num_top_words (integer) The number of top words to be displayed
#' @param num_iterations (integer) The number of iterations to run the model
#' @param seed (integer) A seed to set for reproducibility
#' @param matrix_size_check (boolean) If TRUE, checks whether the vocabulary 
#'   size and Java heap memory are sufficient before fitting the model. If the 
#'   vocabulary is too large or the heap too small, the function stops early with 
#'   advice on how to fix the issue - preventing R from crashing. Set to FALSE to 
#'   skip this check and attempt to fit the model regardless, though this may crash 
#'   your R session if memory is insufficient.
# @param save_dir (string) The directory to save the model, if NULL, the model will not be saved
# @param load_dir (string) The directory to load the model from, if NULL, the model will not be loaded
#' @return A named list containing the following elements:
#' \describe{
#'   \item{name}{Description}
#'   \item{instances}{Java object reference: A list of all documents used for topic modeling, 
#'   in which each document is preprocessed (e.g., tokenized and vectorized). This object is part of 
#'   the Mallet package's internal structure.}
#'   \item{inferencer}{Java object reference: This is the topic inferencer, which allows the inference 
#'   of topic distributions for new, unseen documents based on the trained model.}
#'   \item{top_terms_mallet}{A data frame containing the top terms of each topic, showing which concepts
#'    each topic likely represents. The number of top terms shown here can be adjusted with the argument
#'     num_top_words.}
#'   \item{top_terms}{A data frame containing the top terms of each topic, showing which concepts each 
#'   topic likely represents. The number of top terms shown here can be adjusted with the argument 
#'   num_top_words.}
#'   \item{phi}{A matrix of the topic-word distribution: Each row represents a topic, and each column 
#'   represents a word from the document term matrix. The values show the probability of a word given a 
#'   topic P(word|topic).}
#'   \item{topic_docs}{A matrix of document-topic distribution: Each row represents a document, and each 
#'   column represents a topic. The values show the probability of a topic given a document, P(topic|document).}
#'   \item{frequencies}{A data frame of term frequencies. word = every word in the document term matrix, 
#'   word.freq = the frequency of each word across all documents, doc.freq = the number of documents in which each word appears.}
#'   \item{vocabulary}{A character vector of all unique terms in the document term matrix.}
#'   \item{labels}{A list of topic labels. These short labels are the most representative term for each topic, 
#'   making it easy to identify and understand them.}
#'   \item{theta}{A data frame of document-topic probabilities: each row represents a document, and each column
#'    represents a topic. Similar to topic_docs, this shows the contribution of each topic to each document. 
#'    Each row sums to 1, representing the document's composition of topics.}
#'   \item{prevalence}{A numeric vector showing the overall prevalence (prominence) of each topic in the corpus.
#'    The prevalences are expressed as percentages relative to the other topics  and add up to 100%. 
#'    Higher values indicate topics that are present in more documents.}
#'   \item{coherence}{A numeric vector showing the coherence of each topic. Coherence scores indicate how
#'    semantically consistent and interpretable the topics are. Higher coherence generally indicates 
#'    better-quality topics.}
#'   \item{pred_model}{A list containing components of the predictive model, including phi
#'    (word-topic probability matrix), theta (document-topic probabilities matrix), alpha (Dirichlet prior of topics), 
#'    gamma (hyperparameters of word-topic assignments), and data (sparse matrix representing the document term matrix.)}
#'   \item{dtm_settings}{A list of settings used for preprocessing and building the document term matrix (dtm), 
#'   including n-gram ranges, stopword removal, frequency thresholds, and random seed settings.}
#'   \item{summary}{A summary data frame comprising of the topic numbers, labels, coherence scores, prevalence scores, and top terms.}
#'  }
#' @examples
#' \donttest{
#' # Create LDA Topic Model 
#' save_dir_temp <- tempfile()
#' dtm <- topicsDtm(data = dep_wor_data$Depphrase)
#' 
#' model <- topicsModel(
#' dtm = dtm, # output of topicsDtm()
#' num_topics = 20,
#' num_top_words = 10,
#' num_iterations = 1000,
#' seed = 42)
#' }
#' @export
topicsModel <- function(
    dtm,
    num_topics = 20,
    num_top_words = 10,
    num_iterations = 1000,
    seed = 42, 
    matrix_size_check = TRUE){
  
  
  if (!requireNamespace("mallet", quietly = TRUE)) {
    stop(paste0("JAVA and the R-packages 'rJava' and 'mallet' are required but one or more are not installed. \n", 
                "Please install JAVA from https://www.java.com/. \n", 
                "And then run install.packages(c('mallet', 'rJava')."))
  }
  
  
  set.seed(seed)
  
  dtm_settings <- dtm$settings
  dtm <- dtm$train_dtm
  
  if (length(Matrix::colSums(dtm)) == 0) {
    msg <- "The document term matrix is empty. Please provide a valid document term matrix."
    message(colourise(msg, "brown"))
    return(NULL)
  }
  
  # Get current Java heap size for diagnostic message
  # --- Memory check ---
  n_docs  <- nrow(dtm)
  n_terms <- ncol(dtm)
  n_tokens <- sum(dtm)
  
  phi_gb         <- (as.numeric(num_topics) * as.numeric(n_terms) * 8) / 1024^3
  theta_gb       <- (as.numeric(n_docs)     * as.numeric(num_topics) * 8) / 1024^3
  tokens_gb      <- (as.numeric(n_tokens)   * 4) / 1024^3
  # co-occurrence removed from heap estimate - it is sparse in practice and 
  # wildly overestimates memory for small datasets when treated as dense
  estimated_gb   <- phi_gb + theta_gb + tokens_gb
  
  current_heap_gb <- tryCatch({
    runtime <- rJava::.jcall("java/lang/Runtime", "Ljava/lang/Runtime;", "getRuntime")
    max_mem <- rJava::.jcall(runtime, "J", "maxMemory")
    round(max_mem / 1024^3, 1)
  }, error = function(e) NULL)
  
  message(paste0(
    "\n--- topicsModel() memory diagnostics ---\n",
    "  Documents:                     ", n_docs, "\n",
    "  Vocabulary (n-gram terms):     ", n_terms, "\n",
    "  Total tokens:                  ", n_tokens, "\n",
    "  Topic-word matrix (phi):       ", round(phi_gb, 2), " GB\n",
    "  Document-topic matrix (theta): ", round(theta_gb, 2), " GB\n",
    "  Token assignments:             ", round(tokens_gb, 2), " GB\n",
    "  Estimated total:               ", round(estimated_gb, 2), " GB\n",
    "  Current Java heap:             ", 
    ifelse(is.null(current_heap_gb), "unknown", paste0(current_heap_gb, " GB")), "\n",
    "----------------------------------------"
  ))
  
  # Hard stop only if co-occurrence matrix alone is physically unworkable
  if (matrix_size_check && n_terms > 100000) {
    stop(paste0(
      "\nVocabulary size (", n_terms, " terms) is too large to build the co-occurrence matrix.\n",
      "The co-occurrence matrix alone would require ~", round(cooccurrence_gb, 0), " GB - this will crash R.\n\n",
      "Please rebuild your DTM with a smaller vocabulary:\n\n",
      "  Option 1 - Remove trigrams:\n",
      "    dtm <- topicsDtm(data = text, ngram_window = c(1, 2), ...)\n\n",
      "  Option 2 - Remove rare n-grams (recommended):\n",
      "    dtm <- topicsDtm(data = text, ngram_window = c(1, 3), min_freq = 10, ...)\n\n",
      "  A healthy vocabulary for LDA is typically 20,000-80,000 terms.\n",
      "  Your current vocabulary has ", n_terms, " terms.\n\n",
      "  To skip this check and try anyway (not recommended):\n",
      "    topicsModel(dtm = dtm, ..., matrix_size_check = FALSE)\n"
    ), call. = FALSE)
  }
  
  # Heap check: only trigger if raw estimate (no multiplier) exceeds current heap
  if (matrix_size_check && !is.null(current_heap_gb) && estimated_gb > current_heap_gb) {
    stop(paste0(
      "\nInsufficient Java heap space for this model.\n",
      "  Current heap:  ", current_heap_gb, " GB\n",
      "  Estimated need: ", round(estimated_gb, 1), " GB (plus JVM overhead)\n\n",
      "Please restart R and set the heap BEFORE loading any packages:\n\n",
      '    options(java.parameters = "-Xmx', ceiling(estimated_gb * 2), 'g")  # 2x safety buffer\n',
      "    library(topics)\n\n",
      "To make this permanent, add that line to your .Rprofile:\n\n",
      "    usethis::edit_r_profile()\n\n",
      "  To skip this check and try anyway (not recommended):\n",
      "    topicsModel(dtm = dtm, ..., matrix_size_check = FALSE)\n"
    ), call. = FALSE)
  }

  model <-  get_mallet_model(
      dtm = dtm,
      num_topics = num_topics,
      num_top_words = num_top_words,
      num_iterations = num_iterations,
      seed = seed)
  
  model$dtm_settings <- dtm_settings
  
  model$summary <- data.frame(
    topic = rownames(model$labels),
    label = model$labels,
    coherence = round(model$coherence, 3),
    prevalence = round(model$prevalence,3),
    top_terms = apply(model$top_terms,
                      2, 
                      function(x){paste(x, collapse = ", ")}),
    stringsAsFactors = FALSE)
  model$summary[order(model$summary$prevalence, decreasing = TRUE) , ][ 1:10 , ]
  
  model$model_type <- "mallet"
  
  return(model)
}

#' topicsPredict, topicsPreds, topicsAssess and topicsClassify
#' 
#' The function to predict the topics of a new document with the trained model.
#' @param model (list) The trained model.
#' @param data (tibble) The text variable for which you want to infer the topic distribution. This can be the same data as used to create the dtm or new data.
#' @param num_iterations (integer) The number of iterations to run the model.
#' @param sampling_interval The number of iterations between consecutive samples collected.
#' during the Gibbs Sampling process. This technique, known as thinning, helps reduce the 
#' correlation between consecutive samples and improves the quality of the final estimates 
#' by ensuring they are more independent.
#' Purpose: By specifying a sampling_interval, you avoid collecting highly correlated samples, which can 
#' lead to more robust and accurate topic distributions.
#' Example: If sampling_interval = 10, the algorithm collects a 
#' sample every 10 iterations (e.g., at iteration 10, 20, 30, etc.).
#' Typical Values: Default: 10; Range: 5 to 50 (depending on the complexity and size of the data).
#' @param burn_in The number of initial iterations discarded during the Gibbs Sampling process. 
#' These early iterations may not be representative of the final sampling distribution because the model is still stabilizing.
#' Purpose: The burn_in period allows the model to converge to a more stable state before collecting samples, 
#' improving the quality of the inferred topic distributions.
#' Example: If burn_in = 50, the first 50 iterations of the Gibbs Sampling process are discarded,
#' and sampling begins afterward. Typical Values: Default: 50 to 100 
#' Range: 10 to 1000 (larger datasets or more complex models may require a longer burn-in period).
#' @param seed (integer) A seed to set for reproducibility.
#' @param create_new_dtm (boolean) If applying the model on new data (not used in training), it can help to make a new dtm.
#' Currently this is experimental, and using the textmineR::CreateDtm() function rather than the topicsDtm() function, which has more functions.
#' @return A tibble of the predictions: The rows represent the documents, and the columns represent the topics. The values in the cells indicate the proportion of each topic within the corresponding document.
#' @examples
#' \donttest{
#' # Predict topics for new data with the trained model
#' 
#' dtm <- topicsDtm(
#' data = dep_wor_data$Depphrase)
#' 
#' model <- topicsModel(dtm = dtm, # output of topicsDtm()
#'                      num_topics = 20,
#'                      num_top_words = 10,
#'                      num_iterations = 1000,
#'                      seed = 42)
#'                      
#' preds <- topicsPredict(model = model, # output of topicsModel()
#'                      data = dep_wor_data$Depphrase)
#' }
#' @importFrom tibble as_tibble tibble
#' @importFrom dplyr %>%
#' @export
topicsPredict <- function(
    model, 
    data, 
    num_iterations = 200,
    sampling_interval = 10, # aka thinning
    burn_in = 10, 
    seed = 42,
    create_new_dtm = FALSE
    ){
  
  if (!requireNamespace("mallet", quietly = TRUE)) {
    stop(paste0("JAVA and the R-packages 'rJava' and 'mallet' are required but one or more are not installed. \n", 
                "Please install JAVA from https://www.java.com/. \n", 
                "And then run install.packages(c('mallet', 'rJava')."))
  }
  set.seed(seed)
    
    if (length(data) == 0){
      msg <- "The data provided is empty. Please provide a list of text data."
      message(colourise(msg, "brown"))
      
      stop()
    }
    # create an id column for the data
    
    if (is.data.frame(data)){
      pred_text <- data[, 1]
    } else {
      pred_text <- data
    }
    
    pred_ids <- as.character(1:length(data))
    
    ###### 
    if (create_new_dtm){
      # Preprocess new data to create DTM
      new_dtm <- list()
      new_dtm$train_dtm <- textmineR::CreateDtm(
        doc_vec = data,
        doc_names = as.character(1:length(data)),
        ngram_window = model$dtm_settings$ngram_window,
        stopword_vec = model$dtm_settingsstopwords,  # Use the same stopwords as training
        lower = TRUE,
        remove_punctuation = TRUE,
        remove_numbers = TRUE,
        verbose = FALSE
      )
      
      # Align new DTM with model vocabulary
      model_vocab <- model$vocabulary
      new_vocab <- colnames(new_dtm$train_dtm)
      
      # Identify missing terms
      missing_terms <- setdiff(model_vocab, colnames(new_dtm$train_dtm))
      
      # Add missing terms with zero counts
      if (length(missing_terms) > 0) {
        zero_matrix <- Matrix::Matrix(0, nrow = nrow(new_dtm$train_dtm), ncol = length(missing_terms),
                                      dimnames = list(rownames(new_dtm$train_dtm), missing_terms))
        new_dtm$train_dtm <- cbind(new_dtm$train_dtm, zero_matrix)
      }
      
      new_dtm$train_dtm <- new_dtm$train_dtm[, model_vocab, drop = FALSE]
      # Convert the new DTM to the Mallet instances format
      new_docs <- textmineR::Dtm2Docs(new_dtm$train_dtm)
      
      new_instances <- mallet::mallet.import(
        as.character(seq_along(new_docs)),
        new_docs,
        preserve.case = FALSE,
        token.regexp = "[\\p{L}\\p{N}_]+|[\\p{P}]+\ "
      )
    }
    
    
    ######
    if (create_new_dtm == FALSE){
      
      new_instances <- compatible_instances(
        ids = pred_ids,
        texts = pred_text,
        instances = model$instances)
      
    }
    
    
    # Use the inferencer to predict topics for new instances
    inf_model <- model$inferencer
    
    
    preds <- infer_topics(
      inferencer = inf_model,
      instances = new_instances,
      n_iterations = num_iterations,
      sampling_interval = sampling_interval, # aka "thinning"
      burn_in = burn_in,
      random_seed = seed
    )
    
    preds <- tibble::as_tibble(preds, .name_repair = "minimal")
    colnames(preds) <- paste("t_", 1:ncol(preds), sep="")
    preds <- preds %>% tibble::tibble()
  
  return(preds)
}


# Alias functions
#' @rdname topicsPredict
# @examples textAssess(model = model, data = dep_wor_data$Depphrase)
#' @export
topicsAssess <- topicsPredict

#' @rdname topicsPredict
# @examples textClassify(model = model, data = dep_wor_data$Depphrase)
#' @export
topicsClassify <- topicsPredict

#' @rdname topicsPredict
# @examples textClassify(model = model, data = dep_wor_data$Depphrase)
#' @export
topicsPreds <- topicsPredict

library(testthat)
library(topics)
library(tibble)


test_that("topicsPreds generates predictions with default parameters", {
  
  testthat::skip_on_cran()
  
  dtm <- topics::topicsDtm(
    data = dep_wor_data$Depphrase)
  
  model <- topics::topicsModel(
    dtm = dtm)
  
  result <- topics::topicsPreds(
    model = model, 
    data = dep_wor_data$Depphrase)
  
  testthat::expect_true(is_tibble(result))
  testthat::expect_equal(result$t_1[[1]], 0.006824058, tolerance = 0.00001)
  testthat::expect_equal(nrow(result), length(dep_wor_data$Depphrase))
  testthat::expect_equal(ncol(result), 20)  # Assuming 5 topics
  
  
  # topicsPreds handles different numbers of iterations
  result <- topics::topicsPreds(
    model, 
    data = dep_wor_data$Depphrase, 
    num_iterations = 200)
  
  testthat::expect_true(is_tibble(result))
  testthat::expect_equal(nrow(result), 
                         length(dep_wor_data$Depphrase))
  
  # topicsPreds sets seed for reproducibility
  result1 <- topics::topicsPreds(
    model, 
    data = dep_wor_data$Depphrase, 
    seed = 123)
  
  result2 <- topics::topicsPreds(
    model, 
    data = dep_wor_data$Depphrase, 
    seed = 123)
  
  testthat::expect_equal(result1, result2)
  
})

# tests/test-topicsModel.R
library(testthat)
library(topics)  # Replace with your package name
#library(text)


test_that("topicsModel creates an LDA model correctly with default parameters", {
  
  testthat::skip_on_cran()
  
  dtm <- topicsDtm(
    data = c("This is a test document.", "This is another test."))
  
  result <- topicsModel(
    dtm)
  
  testthat::expect_true(is.list(result))
  testthat::expect_true("summary" %in% names(result))
  testthat::expect_true("top_terms" %in% names(result))
  testthat::expect_true("labels" %in% names(result))
  testthat::expect_true("coherence" %in% names(result))
  testthat::expect_true("prevalence" %in% names(result))
  
  
  result <- topics::topicsModel(
    dtm, 
    num_topics = 10)
  
  testthat::expect_true(is.list(result))
  testthat::expect_equal(nrow(result$summary), 10)
  
  
  # topicsModel handles different numbers of top words"
  result <- topics::topicsModel(
    dtm, 
    num_top_words = 5)
  
  testthat::expect_true(is.list(result))
  testthat::expect_true(all(sapply(result$summary$top_terms, function(x) length(strsplit(x, ", ")[[1]]) == 5)))
  
  # "topicsModel handles different numbers of iterations"
  result <- topics::topicsModel(
    dtm, 
    num_iterations = 500)
  
  testthat::expect_true(is.list(result))
  
  
  ## topicsModel sets seed for reproducibility
  
  testthat::skip_on_cran()
  
  result1 <- topics::topicsModel(
    dtm, 
    seed = 123)
  
  # They are the same even when having different seed; should look over this test. 
  result2 <- topics::topicsModel(
    dtm, 
    seed = 222 
    )
  
  testthat::expect_equal(result1$summary, result2$summary)
  
})

#' Example data about mental health descirptions
#'.
#'
#' @format A data frame with 500 participants and 13 variables:
#' \describe{
#'   \item{Depselect}{Words that respondents have selected from a pre-defined list}
#'   \item{Worselect}{Words that respondents have selected from a pre-defined list}
#'   \item{Depword}{Wrods where respondents describe their experience with depression in life}
#'   \item{Worword}{Words where respondents describe their experience with depression in life}
#'   \item{Depphrase}{phrases where respondents describe their experience with depression in life}
#'   \item{Worphrase}{Phrases where respondents describe their experience with anxiety in life}
#'   \item{Deptext}{Text where respondents describe their experience with depression in life}
#'   \item{Wortext}{Text where respondents describe their experience with anxiety in life}
#'   \item{Gender}{Respondents gender 0=male, 1=female}
#'   \item{Age}{respondents age in years}
#'   \item{PHQ9tot}{total score of the respondents PHQ-9}
#'   \item{GAD7tot}{total score of the respondents GAD-7}
#'   \item{MentalSickDays}{self-reported sick days due to mental health issues}
#'   \item{MentalSickDaysYear}{self-reported sick days due to mental health issues in the last year}
#'   \item{HealthCareMental}{self-reported health care visits due to mental health}
#'   \item{SickDaysMonth}{self-reported sick days last month}
#'   \item{SickDaysYear}{self-reported sick days last year}
#'   \item{HealthCareVisits}{self-reported health care visits}
#' }
#' @source <https://osf.io/preprints/psyarxiv/p67db>
"dep_wor_data"

#' Get the most or least frequent terms in a document-term matrix
#' @param dtm (R_obj) A document-term matrix
#' @param n (integer) The number of terms to be removed
#' @param type (string) most or least frequent terms
#' @param mode (string) absolute or relative amount of terms to be removed
#' @return A list of terms to be removed
#' @noRd
get_removal_terms <- function(
    dtm, 
    n, 
    type, 
    mode="absolute"){
  
  term_frequencies <- colSums(as.matrix(dtm))
  df <- data.frame(as.matrix(dtm))
  
  # Create a data frame with term and frequency
  term_frequency_df <- data.frame(Term = colnames(dtm),
                                  Frequency = term_frequencies)
  
  # Sort the data frame in descending order of frequency
  term_frequency_df <- term_frequency_df[order(-term_frequency_df$Frequency), ]
  
  # Print the terms with the highest frequencies (e.g., top 10 terms)
  #top_terms <- head(term_frequency_df, n = 10)
  if (mode=="percentage"){
    removal_index <- nrow(term_frequency_df)*n 
    
  } else if (mode == "term"){
    removal_index <- n-1
    
  } 
  if (type=="most"){
    removal_index <- round(removal_index) 
    removal_words <- term_frequency_df[["Term"]][1:removal_index]
    
  } else {
    removal_index <- nrow(term_frequency_df) - round(removal_index) # calculates index of term that has highest freqency of all percent least frequent words
    removal_words <- term_frequency_df[["Term"]][nrow(term_frequency_df):removal_index]
  }
  
  return(removal_words)
}

#' Get the column indices of the most or least frequent terms in a document-term matrix
#' @param dtm (R_obj) A document-term matrix
#' @param n (integer) The number of terms to be removed
#' @param type (string) most or least frequent terms
#' @param mode (string) absolute or relative amount of terms to be removed
#' @return A list of column indices to be removed
#' @noRd
get_removal_columns <- function(
    dtm, 
    n, 
    type, 
    mode="absolute"){
  
  terms <- get_removal_terms(dtm, 
                             n, 
                             type, 
                             mode)
  
  filtered_terms_test <- c()
  
  for (term in terms){
    filtered_terms_test <- c(filtered_terms_test, 
                             term)
  }
  
  #filtered_terms_test <- c("sad", "tired", "happy", "low")
  column_indices_to_remove <- which(colnames(dtm) %in% filtered_terms_test)
  
  return(column_indices_to_remove)
}


#' @param train_dtm A document-term matrix (DTM) containing n-grams.
#' @param unigram_dtm A document-term matrix containing unigrams (single words).
#' @param pmi_threshold The PMI threshold for filtering n-grams (default is 0).
#' @noRd
filter_by_pmi <- function(
    train_dtm, 
    unigram_dtm, 
    pmi_threshold = 0) {
  
  # Compute total number of terms in the train_dtm
  total_terms <- sum(train_dtm)
  
  # Compute joint probabilities of the n-grams
  ngram_probs <- Matrix::colSums(train_dtm) / total_terms
  
  # Split n-grams into their components
  ngram_components <- strsplit(colnames(train_dtm), "_")
  
  # Compute marginal probabilities for each component in the n-grams
  component_probs <- sapply(ngram_components, function(component) {
    # Compute probabilities for all components of the n-gram
    probs <- sapply(component, function(word) {
      if (word %in% colnames(unigram_dtm)) {
        word_index <- which(colnames(unigram_dtm) == word)
        prob <- Matrix::colSums(unigram_dtm[, word_index, drop = FALSE]) / sum(unigram_dtm)
        return(prob)
      } else {
        return(1e-10)  # Small value for unmatched words
      }
    })
    # Multiply probabilities for all components to get the joint marginal probability
    return(prod(probs))
  })
  
  # Function to compute PMI for a single n-gram
  compute_pmi <- function(joint_prob, marg_prob) {
    # Avoid division by very small probabilities
    safe_marg_prob <- max(marg_prob, 1e-10)
    # Compute PMI
    pmi <- log2(joint_prob / safe_marg_prob)
    return(pmi)
  }
  
  # Apply the PMI computation across all n-grams
  values <- mapply(compute_pmi, ngram_probs, component_probs)
  
  # Assign names to the PMI values based on the n-grams
  names(values) <- colnames(train_dtm)
  
  # Convert values into a tibble
  pmi_tibble <- tibble(
    n_gram = names(values),  # Use the names of 'values' as the word column
    pmi_value = values       # Use the PMI values as the value column
  )
  
  # Identify unigrams (terms without underscores)
  unigram_indices <- grep("^[^_]+$", colnames(train_dtm))
  
  # Identify n-grams with PMI values above the threshold
  pmi_indices <- which(values >= pmi_threshold)
  
  # Combine unigrams and filtered n-grams
  selected_indices <- union(unigram_indices, pmi_indices)
  
  # Filter the DTM to keep only the selected terms
  filtered_dtm <- train_dtm[, selected_indices, drop = FALSE]
  
  return(list(
    filtered_dtm = filtered_dtm,
    pmi_tibble = pmi_tibble
  ))
}



#' Document Term Matrix
#' 
#' This function creates a document term matrix
#' @param data (list) A list containing the text data with each entry belonging to a unique id
#' @param ngram_window (list) The minimum and maximum n-gram length, e.g., c(1,3)
#' @param stopwords (stopwords) The stopwords to remove, e.g., stopwords::stopwords("en", source = "snowball")
#' @param removalword (string) Character vector of words to remove; e.g., "word1" or c("word1", "word2", ...)
#' @param removal_mode (string) Mode of removal -> one of c("none", "frequency", "term", "percentage"). frequency removes all words under a certain frequency or over a certain frequency, as indicated by removal_rate_least and removal_rate_most. term removes an absolute number of terms that are most frequent and least frequent. percentage removes the number of terms indicated by removal_rate_least and removal_rate_most relative to the number of terms in the matrix
#' @param removal_rate_most (integer) The rate of most frequent words to be removed, functionality depends on removal_mode
#' @param removal_rate_least (integer) The rate of least frequent words to be removed, functionality depends on removal_mode
#' @param pmi_threshold (integer; experimental) Pointwise Mutual Information (PMI) measures the association 
#' between terms by comparing their co-occurrence probability to their individual probabilities, 
#' highlighting term pairs that occur together more often than expected by chance; in this implementation,
#' terms with average PMI below the specified threshold (pmi_threshold) are removed from the document-term matrix. 
#' @param shuffle (boolean) Shuffle the data before analyses
#' @param lower (boolean) If TRUE, converts all text to lowercase before embedding.
#' @param remove_punctuation (boolean) If TRUE, replaces non-alphanumeric characters with spaces. 
#' @param remove_numbers (boolean) If TRUE, replaces all numerical digits with spaces. 
#' @param stem_lemma_function (function). A custom function for stemming or lemmatization.
#' @param verbose Logical. If TRUE, displays progress bars and status updates during 
#' text vectorization and the embedding process. Defaults to FALSE.
#' @param seed (integer) A seed to set for reproducibility
# @param save_dir (string) the directory to save the results, if NULL, no results are saved.
# @param load_dir (string) the directory to load from.
#' @param occurance_rate (numerical) The occurance rate (0-1) removes words that occur less then in (occurance_rate)*(number of documents). Example: If the training dataset has 1000 documents and the occurrence rate is set to 0.05, the code will remove terms that appear in less than 49 documents. 
#' @param threads (integer) The number of threads to use; also called cpu in (CreateDtm).
#' @return The document term matrix
#' @examples
#' \donttest{
#' 
#' # Create a Dtm and remove the terms that occur less than 4 times and more than 500 times.
#' 
#' dtm <- topicsDtm(data = dep_wor_data$Depphrase,
#'                  removal_mode = "frequency",
#'                  removal_rate_least = 4,
#'                  removal_rate_most = 500)
#' 
#' # Create Dtm and remove the 1 least and 1 most frequent terms.
#' dtm <- topicsDtm(data = dep_wor_data$Depphrase,
#'                  removal_mode = "term",
#'                  removal_rate_least = 1,
#'                  removal_rate_most = 1)
#' 
#' # Create Dtm and remove the 1% least frequent and 1% most frequent terms. 
#' # The percentage values are scaled to values between 0 and 1.
#' dtm <- topicsDtm(data = dep_wor_data$Depphrase,
#'                  removal_mode = "percentage",
#'                  removal_rate_least = 0.01,
#'                  removal_rate_most = 0.01)
#'
#' }
#' @importFrom textmineR CreateDtm 
#' @importFrom stats complete.cases
#' @importFrom stopwords stopwords
#' @importFrom Matrix colSums t
#' @importFrom tibble as_tibble
#' @export
topicsDtm <- function(
    data, 
    ngram_window = c(1,3),
    stopwords = stopwords::stopwords("en", source = "snowball"),
    removalword = "",
    pmi_threshold = NULL, 
    occurance_rate = 0,
    removal_mode = "percentage",
    removal_rate_most = 0,
    removal_rate_least = 0,
    shuffle = TRUE,
    lower = TRUE, 
    remove_punctuation = TRUE, 
    remove_numbers = TRUE, 
    stem_lemma_function = NULL,
    verbose = FALSE,
    seed = 42L,
    threads = 1){
  
  # Create a named list of settings
  settings <- list(
    ngram_window = ngram_window,
    stopwords = stopwords,
    removalword = removalword,
    pmi_threshold = pmi_threshold,
    occurance_rate = occurance_rate,
    removal_mode = removal_mode,
    removal_rate_most = removal_rate_most,
    removal_rate_least = removal_rate_least,
    shuffle = shuffle,
    lower = lower, 
    remove_punctuation = remove_punctuation, 
    remove_numbers = remove_numbers, 
    verbose = verbose,
    seed = seed,
    threads = threads
  )
  
  
  pmi_tibble = NULL
  
  if (length(data) == 0){
      msg <- "The data provided is empty. Please provide a list of text data."
      message(colourise(msg, "brown"))
      
      return(NULL)
  }
  
  set.seed(seed)
  id_col <- "id"
  data_col <- "text"
  
  if (is.data.frame(data)){
    data <- data[,1]
  }
  
  text_cols <- data.frame(text = data)
  text_cols[[id_col]] <- 1:nrow(text_cols) # create unique ID
  text_cols <- as_tibble(text_cols)
  text_cols <- text_cols[complete.cases(text_cols), ] # remove missing rows
  
  if(shuffle){
    text_cols = text_cols[sample(1:nrow(text_cols)), ] # shuffle
    split_index <- round(nrow(text_cols) * 1) 
    train <- text_cols[1:split_index, ]
  } else {
    train <- text_cols 
  }

    if (length(removalword) > 0 && any(removalword != "")) {
      pattern <- paste0("\\b(", paste(removalword, collapse = "|"), ")\\b")
      train[[data_col]] <- gsub(pattern, "", train[[data_col]])
    }
  
  if (length(ngram_window) == 1){
    ngram_window <- c(ngram_window, ngram_window)
  }
  
  # Create Trigram DTM
  train_dtm <- textmineR::CreateDtm(
    doc_vec = train[["text"]], 
    doc_names = train[["id"]], 
    ngram_window = ngram_window, 
    stopword_vec = stopwords, 
    lower = lower, 
    remove_punctuation = remove_punctuation, 
    remove_numbers = remove_numbers, 
    verbose = verbose,
    stem_lemma_function = stem_lemma_function,
    cpus = threads
  )
  
  # Create Unigram DTM if necessary for PMI
  if (!is.null(pmi_threshold)) {
    unigram_dtm <- textmineR::CreateDtm(
      doc_vec = train[["text"]], 
      doc_names = train[["id"]], 
      ngram_window = c(1, 1), # Unigrams only
      stopword_vec = stopwords, 
      lower = lower, 
      remove_punctuation = remove_punctuation, 
      remove_numbers = remove_numbers, 
      verbose = verbose,
      stem_lemma_function = stem_lemma_function, 
      cpus = threads
    )
  }
  
  # Apply Occurrence Filtering
  if (occurance_rate > 0){
    removal_frequency <- round(nrow(train) * occurance_rate) - 1
    train_dtm <- train_dtm[, Matrix::colSums(train_dtm) > removal_frequency]
  }
  
  # Removal by Frequency or Custom Modes
  if (removal_mode != "frequency"){
    if (removal_rate_least > 0){
      removal_columns <- get_removal_columns(train_dtm, removal_rate_least, "least", removal_mode)
      if (removal_rate_most > 0){
        removal_columns_most <- get_removal_columns(train_dtm, removal_rate_most, "most", removal_mode)
        removal_columns <- c(removal_columns, removal_columns_most)
      }
      train_dtm <- train_dtm[, -removal_columns, drop = FALSE]
    } else if (removal_rate_most > 0){
      removal_columns <- get_removal_columns(train_dtm, removal_rate_most, "most", removal_mode)
      train_dtm <- train_dtm[, -removal_columns, drop = FALSE]
    }
  } else if (removal_mode == "frequency"){
    if (!is.null(removal_rate_least)){
      train_dtm <- train_dtm[, Matrix::colSums(train_dtm) > removal_rate_least]
    }
    if (!is.null(removal_rate_most)){
      train_dtm <- train_dtm[, Matrix::colSums(train_dtm) < removal_rate_most]
    }
  }
  
  # PMI Filter
  train_dtm_null <- train_dtm
  
  if (!is.null(pmi_threshold)) {
    
    # Assuming train_dtm and unigram_dtm are already created
    result <- filter_by_pmi(
      train_dtm = train_dtm, 
      unigram_dtm = unigram_dtm, 
      pmi_threshold = pmi_threshold)
    
    # Access the filtered DTM
    filtered_dtm <- result$filtered_dtm
    
    # Access the PMI tibble
    pmi_tibble <- result$pmi_tibble
  } else {
    filtered_dtm <- train_dtm
  }

  dtms <- list(
    n_grams_pmi = if (!is.null(pmi_tibble)) pmi_tibble else "No pmi_threshold was used",
    settings = settings,
    train_dtm = filtered_dtm #train_dtm
  )
  
  return(dtms)
}



#' Summarize and Visualize your Document Term Matrix
#' 
#' This function creates a frequency table of your DTM and generates up to four plots for visualization
#' @param dtm (R_obj) The document term matrix -> output of topicsDtm function
#' @importFrom Matrix colSums
#' @importFrom dplyr %>% select 
#' @importFrom ggplot2 ggplot margin element_text
#' @importFrom stats reorder median
#' @return A named list containing:
#' \describe{
#'   \item{dtm_summary}{A dataframe of terms and their frequencies.}
#'   \item{frequency_plot}{A bar plot of all term frequencies with example terms.}
#'   \item{frequency_plot_30_least}{A bar plot of the 30 least frequent terms (if numer of terms > 30).}
#'   \item{frequency_plot_30_most}{A bar plot of the 30 most frequent terms (if numer of terms > 30).}
#'   \item{histogram_of_frequencies}{A histogram of term frequencies (this is the same information as
#'   in the frequency_plot but presented differently).}
#' }
#' @export
topicsDtmEval <- function(dtm) {
  
  # create a summary dataframe of the dtm
  dtm_summary <- as.data.frame(Matrix::colSums(dtm$train_dtm))
  names(dtm_summary) <- c("freq") # rename column
  dtm_summary$term <- rownames(dtm_summary)
  rownames(dtm_summary) <- NULL
  dtm_summary$nr <- c(1:nrow(dtm_summary))
  dtm_summary <- dtm_summary %>% dplyr::select(nr, term, freq) # reorder columns
  
  # If the summary df has <= 30 words, we can use just 2 plots 
  if(nrow(dtm_summary) <= 30){
    
    
    plot_all <- ggplot2::ggplot(data = dtm_summary, ggplot2::aes(x = reorder(term, freq), y = freq, fill = nr))+
      ggplot2::geom_bar(stat = "identity", show.legend = F)+
      ggplot2::labs(title = paste0("Frequencies of all terms in the DTM (N = ", nrow(dtm_summary), ")"),
           x = "Terms", 
           y = "Frequency",
           subtitle = paste0("Min = ", min(dtm_summary$freq), ", Max = ", max(dtm_summary$freq), ", Med = ",
                             med = stats::median(dtm_summary$freq), ", SD = ", round(sd(dtm_summary$freq),2)))+
      ggplot2::scale_y_continuous(expand = c(0,0))+
      ggplot2::scale_fill_gradient(low = "#9BD7E9", high ="#15637F")+
      ggplot2::theme_minimal()+
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 35, hjust = 1, size = 11),
        panel.grid.minor.x = ggplot2::element_blank(),
        panel.grid.major.x = ggplot2::element_blank())
    
    # histogram of term frequencies
    plot_hist <- ggplot2::ggplot(data = dtm_summary, ggplot2::aes(x = freq)) +
      ggplot2::geom_histogram(fill = "#15637F", color = "white") +
      ggplot2::labs(x = "Term frequency",
           y = "Number of terms with this frequency",
           title = paste0("Histogram of term frequencies (N = ", nrow(dtm_summary), ")"),
           subtitle = paste0("Min = ", min(dtm_summary$freq), ", Max = ", max(dtm_summary$freq), ", Med = ",
                             med = stats::median(dtm_summary$freq), ", SD = ", round(sd(dtm_summary$freq),2)))+
      ggplot2::theme_minimal()
    
    # return output with only 2 plots
    return(list(dtm_summary = dtm_summary[,2:3],
                frequency_plot = plot_all,
                histogram_of_frequencies = plot_hist))
  }
  
  # Otherwise, split into 3 plots (first 30, last 30, overview):
  
  # subset first 30 terms (lowest frequencies) and last 30 terms (highest frequencies)
  dtm_summary_30_min <- dtm_summary[1:30,]
  dtm_summary_30_max <- dtm_summary[(nrow(dtm_summary)-30):nrow(dtm_summary),]
  
  
  # plot 30 least frequent
  plot_30_least <- ggplot2::ggplot(data = dtm_summary_30_min, ggplot2::aes(x = reorder(term, freq), y = freq))+
    ggplot2::geom_bar(stat = "identity", color = "white", fill = "#9BD7E9")+
    ggplot2::labs(title = "30 least frequent terms in the DTM",
         x = "Term", y = "Frequency")+
    ggplot2::scale_y_continuous(limits = c(0,max(dtm_summary$freq)))+
    ggplot2::theme_minimal()+
    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 50, hjust = 1, size = 12),
          axis.title.y = ggplot2::element_text(size = 12, face = "bold"),
          plot.title = ggplot2::element_text(size = 15),
          axis.title.x = ggplot2::element_text(margin = margin(t = 5), size = 12, face = "bold"))
  
  # plot 30 msot frequent
  plot_30_most <- ggplot2::ggplot(data = dtm_summary_30_max, ggplot2::aes(x = reorder(term, freq), y = freq))+
    ggplot2::geom_bar(stat = "identity", color = "white", fill = "#15637F")+
    ggplot2::labs(title = "30 most frequent terms in the DTM",
         x = "Term", y = "Frequency")+
    ggplot2::theme_minimal()+
    ggplot2::theme(axis.text.x = ggplot2::element_text(angle = 50, hjust = 1, size = 12),
          axis.title.y = ggplot2::element_text(size = 12, face = "bold"),
          plot.title = ggplot2::element_text(size = 15),
          axis.title.x = ggplot2::element_text(margin = margin(t = 5), size = 12, face = "bold"))
  
  
  # plot all terms and only show some example terms
  keep <- seq(nrow(dtm_summary), 1, -(ceiling(nrow(dtm_summary)/20))) 
  
  dtm_summary <- dtm_summary %>%
    mutate(example_terms =  ifelse(1:nrow(dtm_summary) %in% keep, term, ""))
  
  plot_all <- ggplot2::ggplot(data = dtm_summary, ggplot2::aes(x = reorder(term, freq), y = freq, fill = nr))+
    ggplot2::geom_bar(stat = "identity", show.legend = F)+
    ggplot2::labs(title = paste0("Frequencies of all terms in the DTM (N = ", nrow(dtm_summary), ")"),
         x = "Terms (only a few terms shown for illustration)", 
         y = "Frequency",
         subtitle = paste0("Min = ", min(dtm_summary$freq), ", Max = ", max(dtm_summary$freq), ", Med = ",
                           med = stats::median(dtm_summary$freq), ", SD = ", round(sd(dtm_summary$freq),2)))+
    ggplot2::scale_x_discrete(labels = dtm_summary$example_terms) +
    ggplot2::scale_y_continuous(expand = c(0,0))+
    ggplot2::scale_fill_gradient(low = "#9BD7E9", high ="#15637F")+
    ggplot2::theme_minimal()+
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 35, hjust = 1, size = 11),
      panel.grid.minor.x = ggplot2::element_blank(),
      panel.grid.major.x = ggplot2::element_blank())
  
  
  # histogram of term frequencies
  plot_hist <- ggplot(data = dtm_summary, ggplot2::aes(x = freq)) +
    ggplot2::geom_histogram(fill = "#15637F", color = "white") +
    ggplot2::labs(x = "Term frequency",
         y = "Number of terms with this frequency",
         title = paste0("Histogram of term frequencies (N = ", nrow(dtm_summary), ")"),
         subtitle = paste0("Min = ", min(dtm_summary$freq), ", Max = ", max(dtm_summary$freq), ", Med = ",
                           med = stats::median(dtm_summary$freq), ", SD = ", round(sd(dtm_summary$freq),2)))+
    ggplot2::theme_minimal()
  
  df_final <- dtm_summary %>% dplyr::select(term, freq)
  
  # output
  return(list(dtm_summary = df_final,
              frequency_plot = plot_all,
              frequency_plot_30_least = plot_30_least,
              frequency_plot_30_most = plot_30_most,
              histogram_of_frequencies = plot_hist))
}

library(testthat)
library(topics)  # Replace with your package name
library(tibble)
library(dplyr)
options(mc.cores = 1)




test_that("topicsDtm creates a DTM correctly with default parameters", {

  testthat::skip_on_cran()

  dtm_result <- topics::topicsDtm(
    data =  dep_wor_data$Deptext, 
    ngram_window = c(1,1), 
    removal_mode = "frequency",
    removal_rate_most = 30,
    removal_rate_least = 25
    )
  
  testthat::expect_true(is.list(dtm_result))
  testthat::expect_true("train_dtm" %in% names(dtm_result))
  #testthat::expect_true("test_dtm" %in% names(dtm_result))
  #testthat::expect_true("train_data" %in% names(dtm_result))
  #testthat::expect_true("test_data" %in% names(dtm_result))
  testthat::expect_s4_class(dtm_result$train_dtm, "dgCMatrix")
  
  # Testing with less than 30 words
  dtmeval <- topicsDtmEval(dtm_result)
  dtmeval$frequency_plot

})

test_that("topicsDtm handles different ngram_window values", {
  
  testthat::skip_on_cran()
  
  result <- topics::topicsDtm(
    data = dep_wor_data$Deptext, 
    ngram_window = c(1, 2), 
    threads = 1)
  
  testthat::expect_true(is.list(result))
  testthat::expect_s4_class(result$train_dtm, "dgCMatrix")
  
})

test_that("topicsDtm removes a specified word", {
  
  testthat::skip_on_cran()
  
  result <- topics::topicsDtm(
    data = dep_wor_data$Deptext, 
    removalword = "test")
  
  testthat::expect_true(is.list(result))
  testthat::expect_s4_class(result$train_dtm, "dgCMatrix")
  testthat::expect_false("test" %in% colnames(result$train_dtm))
  
})

test_that("topicsDtm handles different occurrence rates", {
  
  testthat::skip_on_cran()
  
  result <-topics:: topicsDtm(
    data = dep_wor_data$Deptext,
    occurance_rate = 1)
  
  testthat::expect_true(is.list(result))
  testthat::expect_s4_class(result$train_dtm, "dgCMatrix")
  
})

test_that("topicsDtm handles different removal modes", {
  
  testthat::skip_on_cran()
 
  result <- topics::topicsDtm(
    data = dep_wor_data$Deptext, 
    removal_mode = "absolute")
  
  testthat::expect_true(is.list(result))
  testthat::expect_s4_class(result$train_dtm, "dgCMatrix")
  
})

test_that("topicsDtm handles different split proportions", {
  
  testthat::skip_on_cran()
  
  result <- topics::topicsDtm(
    data = dep_wor_data$Deptext)
  
  testthat::expect_true(is.list(result))
#  testthat::expect_true(nrow(result$train_data) <= nrow(data)*0.5)
#  testthat::expect_true(nrow(result$test_data) <= nrow(data)*0.5)
  testthat::expect_s4_class(result$train_dtm, "dgCMatrix")
  
})

#test_that("topicsDtm handles different seeds for reproducibility", {
#  
#  testthat::skip_on_cran()
#  save_dir_temp <- tempdir()
#  
#  
#  result1 <- topics::topicsDtm(
#    data =  dep_wor_data$Deptext, 
#    seed = 123, 
#    save_dir = save_dir_temp)
#  
#  result2 <- topics::topicsDtm(
#    data =  dep_wor_data$Deptext, 
#    seed = 124, 
#    save_dir = save_dir_temp)
#  
#  testthat::expect_equal(result1$train_dtm, result2$train_dtm)
#  
#})



test_that("topicsDtm removes least frequent words based on a threshold", {
  
  testthat::skip_on_cran()
  
  result <- topics::topicsDtm(
    data = dep_wor_data$Deptext, 
    removal_mode = "frequency", 
    removal_rate_least = 2)
  
  testthat::expect_true(is.list(result))
  testthat::expect_s4_class(result$train_dtm, "dgCMatrix")
  
})

test_that("topicsDtm removes most frequent words based on a threshold", {
  
  testthat::skip_on_cran()
  
  result <- topics::topicsDtm(
    data = dep_wor_data$Deptext, 
    removal_mode = "frequency", 
    removal_rate_most = 50)
  
  testthat::expect_true(is.list(result))
  testthat::expect_s4_class(result$train_dtm, "dgCMatrix")

})

#test_that("topicsDtm removes least frequent words in percent mode", {
#  testthat::skip_on_cran()
# data <- dep_wor_data$Wortext
#  result <- topicsDtm(data, removal_mode = "percent", removal_rate_least = 50)
  
#  testthat::expect_true(is.list(result))
#  testthat::expect_s4_class(result$train_dtm, "dgCMatrix")
#})

test_that("topicsDtm removes most frequent words in percent mode", {
  
  testthat::skip_on_cran()
  
  result <- topics::topicsDtm(
    data = dep_wor_data$Deptext[1:20], 
    removal_mode = "percentage", 
    removal_rate_most = 5)
  
  testthat::expect_true(is.list(result))
  testthat::expect_s4_class(result$train_dtm, "dgCMatrix")
  
})


test_that("topicsDtm PMI thresholhd ", {
  
  testthat::skip_on_cran()
  
  result_pmi0 <- topics::topicsDtm(
    data = dep_wor_data$Deptext,
    pmi_threshold = NULL)
  
  result_pmi1 <- topics::topicsDtm(
    data = dep_wor_data$Deptext,
    pmi_threshold = 1)
  
  # Check the n-grams arranged according to pmi
  result_pmi1$n_grams_pmi %>% 
    arrange(desc(pmi_value))
  
  result_pmi1$n_grams_pmi %>% 
    arrange(-desc(pmi_value))
  
  testthat::expect_true(ncol(result_pmi1$train_dtm) == 21692)
  
})

Sys.setenv(OMP_NUM_THREADS = "1") #Limit the number of threads to prevent conflicts.

Sys.setenv(OMP_MAX_ACTIVE_LEVELS = "1")

# If above does not work, you can also try this; although this solution might have some risks assocaited with it (for more information see https://github.com/dmlc/xgboost/issues/1715)
Sys.setenv(KMP_DUPLICATE_LIB_OK = "TRUE") #Temporarily allows execution despite duplicate OpenMP libraries.

library(testthat)
library(topics)
library(text)

test_that("Testing to training topics distributions using textTrainRegression",{
  
  testthat::skip_on_cran()
  if (Sys.getenv("SKIP_GITHUB_ACTIONS") == "true") {
    testthat::skip("Skipping this test in GitHub Actions R CMD check.")
  }
  
  dtm <- topics::topicsDtm(
    data = dep_wor_data$Deptext, 
    removal_mode = "frequency",
    removal_rate_most = 50,
    removal_rate_least = 5)
  
  dtmeval <- topicsDtmEval(dtm)
  dtmeval$frequency_plot
  dtmeval$frequency_plot_30_least
  dtmeval$frequency_plot_30_most
  
  model <- topics::topicsModel(
    dtm = dtm, 
    num_topics = 50,
    num_top_words = 10,
    num_iterations = 1500)
  
  #### Same data predictions  ####
  same_data_preds <- topicsPreds(
    model = model,
    data = dep_wor_data$Deptext,
    num_iterations = 50
  )
  
  colnames(same_data_preds) <- paste0("Dim", 1:20, "_", colnames(same_data_preds)) 
  model_same_data <- text::textTrainRegression(
    x = same_data_preds, 
    y = dep_wor_data["PHQ9tot"], 
    multi_cores = FALSE
  )
  #model_same_data
#  testthat::expect_equal(model_same_data$results$estimate[[1]], .3906754, tolerance = .00001)
  testthat::expect_equal(model_same_data$results$estimate[[1]], .4097224, tolerance = .00001)
  
  ##### Testing on new data #####
  new_data_preds <- topics::topicsPreds(
    model = model,
    data = dep_wor_data$Worphrase,
    num_iterations = 50,
    create_new_dtm = FALSE
  )
  colnames(new_data_preds) <- paste0("Dim", 1:20, "_", colnames(new_data_preds)) 
  model_new_data <- text::textTrainRegression(
    x = new_data_preds, 
    y = dep_wor_data["PHQ9tot"], 
    multi_cores = FALSE
  )
  #model_new_data
  testthat::expect_equal(model_new_data$results$estimate[[1]], 0.14506, tolerance = .0001)
  
  ##### Testing on new data with new dtm ####
  new_data_newdtm_preds <- topics::topicsPreds(
    model = model,
    data = dep_wor_data$Worphrase,
    num_iterations = 100,
    create_new_dtm = TRUE
  )
  colnames(new_data_newdtm_preds) <- paste0("Dim", 1:20, "_", colnames(new_data_newdtm_preds)) 
  model_new_data_pewnew <- text::textTrainRegression(
    x = new_data_newdtm_preds, 
    y = dep_wor_data["PHQ9tot"], 
    multi_cores = FALSE
  )
  model_new_data_pewnew$results
  
  testthat::expect_equal(model_new_data_pewnew$results$estimate[[1]], .2380506, tolerance = .00001)
  
})

# Global settings (these avoids the note (warning) "no visible binding for global variable ‘XXX’
# https://community.rstudio.com/t/how-to-solve-no-visible-binding-for-global-variable-note/28887
utils::globalVariables(c(
  "Word", "phi", "scale_color_gradient", 
  
  "p.value", "adjust.p_value", "adjusted_p.value", 
  
  "top_terms", "prevalence", "coherence", "topic_name", "label_1", 
  "mean_group_1", "mean_group_2", "seed", "cohen_d", "value",
  "phi_max_size", "test_path", 
  
  ".", ".data", "color_categories", "contains_only_five", "fill", "group", "left_join",
  
  "map_num", "mean_value", "prevalence", "rename", "topic", "topic_number", "x", "y",
  
  # generate_scatter_plot
  "geom_text", 
  
  # topicsTest
  "binomial", 
  
  # topicsScatterLegend
  "contains_only", "label_content",
  
  # General
  "msg2",
  
  # create_plots
  "color",
  
  "color_categories1",
  
  # topicsDtmEval
  "nr", "term", "freq",
  
  # topicsGrams
  "n", "component_prob", "joint_prob", "n_gram_type", "ngrams", "pmi_value", ":=",
  
  # topicsModel
  "cooccurrence_gb",
  
  # topicsPlot
  "pmi", "color_negative_cor", "color_positive_cor", "estimate", "highlight_topic_words",
  "p_adjusted", 
  
  # topicsTutorialData
  "WordCount", "text"
  
))

#'  Plot a distribution plot (available for the text-package)
#'  
#' @param bivariate_color_codes A vector of color codes specifying colors for 
#' different categories in the scatter plot. 
#' Default: c("#398CF9", "#60A1F7", "#5dc688", "#e07f6a", "#EAEAEA", "#40DD52", "#FF0000", "#EA7467", "#85DB8E").
#' @param filtered_test A data frame containing the input data for the scatter plot. 
#' Must include columns like `color_categories` and other variables used in the function.
#' @param num_popout The number of topics to "pop out" in each category. Default: 1.
#'  Can be a single integer (applies to all categories) or a vector for specific categories.
#' @param way_popout_topics The method for selecting pop-out topics. Options: "mean", "max_y", or "max_x". Default: "mean".
#' @param user_spec_topics A vector of user-specified topics to highlight in the scatter plot. Default: NULL.
#' @param allow_topic_num_legend Logical; if TRUE, displays topic numbers in the legend. Default: FALSE.
#' @param scatter_show_axis_values Show values on the axises. 
#' @param y_axes_1 Specifies axis alignment for the scatter legend. Options: 1 (x-axis) or 2 (y-axis). Default: 2.
#' @param cor_var A string used for naming the correlation variable in labels or file names. Default: "".
#' @param label_x_name Label for the x-axis in the scatter plot. Default: "x".
#' @param label_y_name Label for the y-axis in the scatter plot. Default: "y".
#' @param save_dir Directory where the scatter legend plot will be saved. Default: "./results".
#' @param figure_format File format for the saved scatter plot. Examples: "svg", "png", "pdf". Default: "svg".
#' @param scatter_popout_dot_size Size of the dots for pop-out topics in the scatter legend. Set to "prevalence" for dot size changing based on topic prevalence. Default: 15.
#' @param scatter_bg_dot_size Size of the dots for background topics in the scatter legend. Default: 9.
#' @param scatter_legend_dots_alpha The transparency of the dots
#' @param scatter_legend_bg_dots_alpha The transparency of the dots
#' @param scatter_legend_circles Plot concentric circles for the scatter legend
#' @param scatter_legend_circles_radius Radius of first concentric circle
#' @param scatter_legend_circles_num Number of Concentric circles
#' @param width Width of the saved scatter plot in inches. Default: 10.
#' @param height Height of the saved scatter plot in inches. Default: 8.
#' @param seed Seed for reproducibility, ensuring consistent plot generation. Default: 42.
#' @param title_font Font family used for all non-word text elements in the plots (e.g., titles, axis labels, tick labels,
#' legend text, annotations). Default: "sans". Examples: "serif", "mono", or a system-installed font family name.
#' @importFrom ggplot2 ggplot geom_point scale_color_manual labs theme_minimal theme element_blank geom_hline geom_vline
#' @importFrom rlang sym !!
#' @importFrom ggforce geom_circle
#' @importFrom dplyr pull select filter mutate anti_join summarise pull group_by group_modify ungroup
#' @export
topicsScatterLegend <- function(
    bivariate_color_codes,
    filtered_test, 
    num_popout = 1, 
    way_popout_topics = "mean", 
    user_spec_topics = NULL, 
    allow_topic_num_legend = FALSE,
    scatter_show_axis_values = TRUE,
    y_axes_1 = 2, 
    cor_var = "", 
    label_x_name = "x", 
    label_y_name = "y", 
    save_dir, 
    figure_format = "svg",
    scatter_popout_dot_size = c(1, 5), 
    scatter_bg_dot_size = c(1, 5),
    scatter_legend_dots_alpha = 0.8,
    scatter_legend_bg_dots_alpha = 0.2, 
    scatter_legend_circles = FALSE,
    scatter_legend_circles_radius = 0,
    scatter_legend_circles_num = 4,
    width = 10, 
    height = 8, 
    seed = 42,
    title_font = "sans"
) {
  
  # Determine x, y, and color columns
  x_column <- names(filtered_test)[5]
  y_column <- if (y_axes_1 == 2) names(filtered_test)[9] else NULL
  color_column <- names(filtered_test)[ncol(filtered_test)]
  
  # Check for only significant or non-significant topics
  contains_category <- function(cat) {
    filtered_test %>%
      dplyr::summarise(contains_only = all(color_categories %in% cat)) %>%
      dplyr::pull(contains_only)
  }
  
  only_two <- contains_category(2)  # Non-significant topics
  only_five <- contains_category(5) # Significant topics
  
  # Logic for handling topics to be emphasised in scatter plot
  # User-specified topics for popout.
  if (!is.null(user_spec_topics)) {
    popout <- filtered_test %>% filter(topic %in% user_spec_topics)
    backgr_dots <- filtered_test %>% dplyr::anti_join(popout, by = colnames(filtered_test))
    
    # Only non-significant topics. Generating scatter legend.
  } else if (only_two && y_axes_1 == 1) {
    popout <- filtered_test %>% dplyr::filter(color_categories %in% 1:3)
    backgr_dots <- tibble::tibble(data.frame(matrix(0,nrow=1,ncol=ncol(popout)))) # No background dots
    names(backgr_dots) <- names(popout)# No background dots
    
    # Only significant topics. Generating scatter plot.\n
  } else if (only_five && y_axes_1 == 2) {
    popout <- filtered_test
    backgr_dots <- tibble::tibble(data.frame(matrix(0,nrow=1,ncol=ncol(popout)))) # No background dots
    names(backgr_dots) <- names(popout) # No background dots
    
    # Generating scatter plot based on specified popout criteria.\n
  } else {
    popout <- determine_popout_topics(
      filtered_test, num_popout, way_popout_topics, y_col =  y_column, x_col = x_column)
    
    # Convert `color_categories` in `popout` back to integer
    popout <- popout %>%
      dplyr::mutate(color_categories = as.integer(color_categories))
    filtered_test <- filtered_test %>%
      dplyr::mutate(color_categories = as.integer(color_categories))
    
    # Perform anti_join
    backgr_dots <- filtered_test %>% dplyr::anti_join(popout, by = colnames(filtered_test))
    if (nrow(backgr_dots) == 0){
      backgr_dots <- tibble::tibble(data.frame(matrix(0,nrow=1,ncol=ncol(popout)))) # No background dots
      names(backgr_dots) <- names(popout) # No background dots
    }
  }
  
  if (max(popout$prevalence) == min(popout$prevalence)) {
    # If all prevalence values are the same, assign the midpoint size to all rows
    popout <- popout %>%
      dplyr::mutate(dot_size = mean(scatter_popout_dot_size))
  } else {
    # If prevalence varies, apply the normal scaling
    popout <- popout %>%
      dplyr::mutate(
        dot_size = scatter_popout_dot_size[[1]] + 
          (prevalence - min(prevalence)) / (max(prevalence) - min(prevalence)) * 
          (scatter_popout_dot_size[[2]] - scatter_popout_dot_size[[1]])
      )
  }
  
  # Update scatter_popout_dot_size after mutation
  scatter_popout_dot_size <- popout$dot_size      
  
  if (max(backgr_dots$prevalence) == min(backgr_dots$prevalence)) {
    # If all prevalence values are the same, assign the midpoint size to all rows
    backgr_dots <- backgr_dots %>%
      dplyr::mutate(bg_dot_size = mean(scatter_bg_dot_size))
  } else {
    # If prevalence varies, apply the normal scaling
    backgr_dots <- backgr_dots %>%
      dplyr::mutate(
        bg_dot_size = scatter_bg_dot_size[[1]] + 
          (prevalence - min(prevalence)) / (max(prevalence) - min(prevalence)) * 
          (scatter_bg_dot_size[[2]] - scatter_bg_dot_size[[1]])
      )
  }
  
  scatter_bg_dot_size <- backgr_dots$`bg_dot_size`
  
  # Generate scatter plot
  plot <- generate_scatter_plot(
    popout = popout,
    background = backgr_dots,
    bivariate_color_codes = bivariate_color_codes,
    x_col = x_column, 
    y_col = y_column, 
    label_x_name = label_x_name, 
    label_y_name = label_y_name, 
    color_col = color_column, 
    popout_size = scatter_popout_dot_size, 
    bg_size = scatter_bg_dot_size, 
    scatter_legend_dots_alpha = scatter_legend_dots_alpha,
    scatter_legend_bg_dots_alpha = scatter_legend_bg_dots_alpha,
    allow_topic_num_legend = allow_topic_num_legend, 
    scatter_show_axis_values = scatter_show_axis_values,
    scatter_legend_circles = scatter_legend_circles,
    scatter_legend_circles_radius = scatter_legend_circles_radius,
    scatter_legend_circles_num = scatter_legend_circles_num,
    title_font = title_font
  )
  
  # Save the plot
  if (!is.null(save_dir)){
    ggplot2::ggsave(paste0(save_dir, "/seed_", seed, 
                           "/wordclouds/",
                           "dot_legend_",
                           "corvar_", cor_var, ".", 
                           figure_format),
                    plot = plot, 
                    width = width, 
                    height = height, 
                    units = "in", 
                    device = figure_format, 
                    create.dir = TRUE)
  }
  
  output <- list()
  output[[1]] <- popout
  output[[2]] <- plot
  names(output) <- c("popout", "legend")
  
  return(output)
}

#' @param filtered_test A data frame containing the input data, which must include a `color_categories` column. 
#' This column specifies the categories used for determining pop-out topics.
#' @param num_popout A vector of exactly 9 integers, specifying the number of topics to "pop out" for each 
#' category in a 3x3 grid. Each value corresponds to a category in `color_categories`.
#' @param way_popout_topics A string specifying the criterion for selecting pop-out topics. Options:
#'.  - "max_y": Selects topics with the maximum absolute values in the `y_col` column.
#'.  - "max_x": Selects topics with the maximum absolute values in the `x_col` column.
#'.  - "mean": Selects topics based on the highest mean of the absolute values of `x_col` and `y_col`.
#' @param y_col A string specifying the name of the column to be used for `y` values in the selection process.
#' This column must exist in `filtered_test`.
#' @param x_col A string specifying the name of the column to be used for `x` values in the selection process.
#' This column must exist in `filtered_test`.
#' @noRd
determine_popout_topics <- function(
    filtered_test, 
    num_popout, 
    way_popout_topics, 
    y_col = NULL, 
    x_col
) {
  # Ensure `color_categories` exists
  if (!"color_categories" %in% colnames(filtered_test)) {
    stop("The `filtered_test` dataset must include a `color_categories` column.")
  }
  
  # Convert `color_categories` to character for consistent comparison
  filtered_test <- filtered_test %>%
    dplyr::mutate(color_categories = as.character(color_categories))
  
  # Check for NA or unexpected values
  if (any(is.na(filtered_test$color_categories))) {
    stop("The `color_categories` column contains missing (NA) values.")
  }
  
  # Ensure `num_popout` has the correct number of values (either 3 or 9)
  if (!(length(num_popout) %in% c(3, 9))) {
    stop("`num_popout` must have exactly 3 or 9 values.")
  }
  
  # Map `num_popout` to corresponding categories (names will be "1", "2", etc.)
  legend_map_num_pop <- if (length(num_popout) == 9) {
    setNames(as.integer(num_popout), as.character(1:9))
  } else {
    setNames(as.integer(num_popout), as.character(1:3))
  }
  
  # Filter for categories present in `filtered_test`
  existing_categories <- unique(filtered_test$color_categories)
  valid_map <- legend_map_num_pop[names(legend_map_num_pop) %in% existing_categories]
  
  if (length(valid_map) == 0) {
    stop("No valid `color_categories` in `filtered_test` match `num_popout` mapping.")
  }
  
  # Define helper function for max-based selection (existing behavior)
  select_rows <- function(data, n_pop) {
    if (way_popout_topics == "max_y" && !is.null(y_col)) {
      return(dplyr::slice_max(data, order_by = abs(!!ggplot2::sym(y_col)), n = n_pop, with_ties = FALSE))
    }
    if (way_popout_topics == "max_x") {
      return(dplyr::slice_max(data, order_by = abs(!!ggplot2::sym(x_col)), n = n_pop, with_ties = FALSE))
    }
    if (way_popout_topics == "mean") {
      if (!is.null(y_col)) {
        data <- data %>%
          dplyr::mutate(mean_value = rowMeans(cbind(abs(!!ggplot2::sym(x_col)), abs(!!ggplot2::sym(y_col)))))
      } else {
        data <- data %>%
          dplyr::mutate(mean_value = abs(!!ggplot2::sym(x_col)))
      }
      return(dplyr::slice_max(data, order_by = mean_value, n = n_pop, with_ties = FALSE))
    }
    stop("Invalid `way_popout_topics`. Supported values are 'max_y', 'max_x', or 'mean'.")
  }
  
  # Define helper function for min-based selection (rows closest to 0)
  select_rows_min <- function(data, n_pop) {
    if (way_popout_topics == "max_y" && !is.null(y_col)) {
      return(dplyr::slice_min(data, order_by = abs(!!ggplot2::sym(y_col)), n = n_pop, with_ties = FALSE))
    }
    if (way_popout_topics == "max_x") {
      return(dplyr::slice_min(data, order_by = abs(!!ggplot2::sym(x_col)), n = n_pop, with_ties = FALSE))
    }
    if (way_popout_topics == "mean") {
      if (!is.null(y_col)) {
        data <- data %>%
          dplyr::mutate(mean_value = rowMeans(cbind(abs(!!ggplot2::sym(x_col)), abs(!!ggplot2::sym(y_col)))))
      } else {
        data <- data %>%
          dplyr::mutate(mean_value = abs(!!ggplot2::sym(x_col)))
      }
      return(dplyr::slice_min(data, order_by = mean_value, n = n_pop, with_ties = FALSE))
    }
    stop("Invalid `way_popout_topics`. Supported values are 'max_y', 'max_x', or 'mean'.")
  }
  
  # Determine which color category should use the min-based selection:
  # - For a 9-element vector, the popout category is "5".
  # - For a 3-element vector, the popout category is "2".
  num1 <- max(as.numeric(filtered_test$color_categories))
  popout_category <- if (length(num_popout) == 9 && num1 == 9) "5" else "2"
  
  # Process each category using group_modify:
  filtered_test %>% 
    dplyr::filter(color_categories %in% names(valid_map)) %>% 
    dplyr::group_by(color_categories) %>% 
    dplyr::group_modify(~ {
      category <- .y$color_categories
      n_pop    <- valid_map[[category]]
      
      if (n_pop <= 0) return(.x[0, ])
      
      if (category == popout_category) {
        select_rows_min(.x, n_pop)
      } else {
        select_rows(.x, n_pop)
      }
    }) %>% 
    dplyr::ungroup() %>% 
   dplyr::select(-color_categories, color_categories)

}

#' @param popout A data frame containing the data points to be highlighted ("pop-out") in the scatter plot.
#' @param background A data frame containing the background data points for the scatter plot.
#'                   Can be empty if no background points are needed.
#' @param bivariate_color_codes A vector of color codes used to map `color_col` categories to colors in the scatter plot.
#' @param x_col A string specifying the name of the column to be used for the x-axis in the scatter plot.
#'              Must exist in both `popout` and `background` data frames.
#' @param y_col A string specifying the name of the column to be used for the y-axis in the scatter plot. Default: NULL.
#'              If NULL, a constant value of 1 is used for all points.
#' @param label_x_name Label for the x-axis in the scatter plot.
#' @param label_y_name Label for the y-axis in the scatter plot.
#' @param color_col A string specifying the name of the column in `popout` and `background` used to map categories to colors.
#' @param popout_size The size of the dots for pop-out points in the scatter plot.
#' @param bg_size The size of the dots for background points in the scatter plot.
#' @param scatter_legend_bg_dots_alpha The transparency of the dots
#' @param scatter_legend_dots_alpha The transparency of the dots
#' @param allow_topic_num_legend Logical; if TRUE, adds topic numbers as text labels to the pop-out points. Default: FALSE.
#' @param scatter_show_axis_values Show the values on the axises. 
#' @param scatter_legend_circles Plot concentric circles for the scatter legend
#' @param scatter_legend_circles_radius Radius of first concentric circle
#' @param scatter_legend_circles_num Number of Concentric circles
#' @param title_font Font family used for all non-word text elements in the plots (e.g., titles, axis labels, tick labels,
#' legend text, annotations). Default: "sans". Examples: "serif", "mono", or a system-installed font family name.

#' @noRd
generate_scatter_plot <- function(
    popout,
    background,
    bivariate_color_codes,
    x_col, 
    y_col = NULL,
    label_x_name, 
    label_y_name,
    color_col, 
    popout_size, 
    bg_size, 
    scatter_legend_dots_alpha = 0.8, 
    scatter_legend_bg_dots_alpha = 0.2,
    allow_topic_num_legend, 
    scatter_show_axis_values,
    scatter_legend_circles = FALSE,
    scatter_legend_circles_radius = 0,
    scatter_legend_circles_num = 4,
    title_font = "sans"
) {
  
  # Define aesthetics for popout and background points
  # Ensure y_col is valid and resolve y_aesthetic
  y_aesthetic <- if (!is.null(y_col) && y_col != "") ggplot2::sym(y_col) else 1
  
  # Create aes with defined y aesthetic
  popout_aes <- ggplot2::aes(
    x = !!ggplot2::sym(x_col),
    y = y_aesthetic,
    color = as.factor(.data[[color_col]])
  )
  
  # Resolve y aesthetic value
  y_value <- if (is.null(y_col)) 1 else ggplot2::sym(y_col)
  
  # Aesthetics for background points
  bg_aes <- if (is.null(y_col)) {
    ggplot2::aes(
      x = !!ggplot2::sym(x_col),
      y = 1,
      color = as.factor(.data[[color_col]])
    )
  } else {
    ggplot2::aes(
      x = !!ggplot2::sym(x_col),
      y = !!ggplot2::sym(y_col),
      color = as.factor(.data[[color_col]])
    )
  }
  
  popout_aes <- if (is.null(y_col)) {
    ggplot2::aes(
      x = !!ggplot2::sym(x_col),
      y = 1,
      color = as.factor(.data[[color_col]])
    )
  } else {
    ggplot2::aes(
      x = !!ggplot2::sym(x_col),
      y = !!ggplot2::sym(y_col),
      color = as.factor(.data[[color_col]])
    )
  }
  
  # Build the plot
  plot <- ggplot2::ggplot()
  
  # Add background points only if background is not empty
  if (nrow(background) > 0) {
    plot <- plot +
      ggplot2::geom_point(data = background, bg_aes, size = bg_size, alpha = scatter_legend_bg_dots_alpha)
  }
  
  # Add popout points
  plot <- plot +
    ggplot2::geom_point(data = popout, 
                        popout_aes, 
                        size = popout_size, 
                        alpha = scatter_legend_dots_alpha) +
    ggplot2::scale_color_manual(values = bivariate_color_codes) +
    ggplot2::labs(x = label_x_name, y = label_y_name, color = '') +
    ggplot2::theme_minimal(base_family = title_font) +
    ggplot2::theme(
      axis.text = if (scatter_show_axis_values) ggplot2::element_text(size = 12) else ggplot2::element_blank(),
      axis.ticks = if (scatter_show_axis_values) ggplot2::element_line() else ggplot2::element_blank(),
      legend.position = "none"
    )
  
  
  # Add topic numbers if enabled
  if (allow_topic_num_legend) {
    plot <- plot + ggplot2::geom_text(
      data = popout, 
      ggplot2::aes(x = !!ggplot2::sym(x_col), 
                   y = if (is.null(y_col)) 1 else !!ggplot2::sym(y_col), 
                   label = topic_number),
      size = popout_size - 3, 
      color = "black", 
      hjust = 0.5, 
      vjust = 0.5
    )
  }
  
  # Determine maximum absolute x-value
  x_values <- c(popout[[x_col]], background[[x_col]])
  max_abs_x <- max(abs(x_values))
  if (!is.null(y_col)){
    y_values <- c(popout[[y_col]], background[[y_col]])
    max_abs_y <- max(abs(y_values))
  }
  
  # Create symmetrical breaks
  # Find a suitable interval for the breaks. We'll try to get around 5 breaks.
  n_breaks <- 5
  interval <- max_abs_x/(n_breaks/2)
  breaks_x <- seq(-ceiling(max_abs_x/interval)*interval, ceiling(max_abs_x/interval)*interval, by = interval)
  if (!is.null(y_col)){
    interval <- max_abs_y/(n_breaks/2)
    breaks_y <- seq(-ceiling(max_abs_y/interval)*interval, ceiling(max_abs_y/interval)*interval, by = interval)
  }
  
  # 3. Set symmetrical x/y-axis limits AND explicit breaks
  plot <- plot + ggplot2::scale_x_continuous(limits = c(-max_abs_x, max_abs_x), breaks = breaks_x,
                                             labels = function(x) sprintf("%.2f", x))
  if (!is.null(y_col)){
    plot <- plot + ggplot2::scale_y_continuous(limits = c(-max_abs_y, max_abs_y), breaks = breaks_y,
                                               labels = function(x) sprintf("%.2f", x))
  }
  
  if (is.null(y_col)){
    plot <- plot + 
      ggplot2::theme(
        axis.title.x = ggplot2::element_text(hjust = 0.5,
                                             margin = ggplot2::margin(t = 21.3, unit = "pt")
        ),
        axis.text.x = ggplot2::element_text(margin = ggplot2::margin(t = 21, unit = "pt"), size = 12),
        legend.position = "none",
        axis.title.y = ggplot2::element_blank(),
        axis.text.y = ggplot2::element_blank(),
        axis.ticks.y = ggplot2::element_blank(),
        axis.line.y = ggplot2::element_blank(),
        axis.ticks.length.y = ggplot2::unit(0, "pt"),
        panel.spacing.y= ggplot2::unit(0, "lines"),
        panel.border = ggplot2::element_blank(),
        panel.grid.major.y = ggplot2::element_blank(),
        panel.grid.minor.y = ggplot2::element_blank(),
        aspect.ratio = 1/20,
        plot.margin = ggplot2::margin(0.5, 0.5, 1, 0.5, "cm") 
      ) 
  }else{
    plot <- plot +
      ggplot2::theme(
        axis.title.x = ggplot2::element_text(hjust = 0.5,
                                             margin = ggplot2::margin(t = 10.6, unit = "pt")
        ),
        axis.text.x = ggplot2::element_text(margin = ggplot2::margin(t = 10.3, unit = "pt"), size = 12),
        legend.position = "none",
        axis.ticks.x = ggplot2::element_line(),
        axis.text.y = ggplot2::element_text(size = 12),
        plot.margin = ggplot2::margin(0.5, 0.5, 0.5, 0.5, "cm")
      )
  }
  
  #Add concentric circles 2 dimensional scatter legend plots
  if(!is.null(y_col) && scatter_legend_circles){
    
    if(scatter_legend_circles_radius == 0) {
      radius <- interval/2 
    } 
    else {
      radius <- scatter_legend_circles_radius
    }
    
    x0 <- y0 <- r <- NULL
    circles <- data.frame(
      x0 = 0,
      y0 = 0,
      r = seq(radius,  by=radius, length.out = scatter_legend_circles_num)
    )
    plot <- plot  + ggplot2::geom_hline(yintercept = 0, size = 0.2, color = "#787373") + ggplot2::geom_vline(xintercept = 0, size = 0.2, color = "#787373")
    plot <- plot + ggforce::geom_circle(aes(x0 = x0, y0 = y0, r = r), linetype = 2, data = circles, linewidth = 0.2, color = "#787373")
  }
  
  plot <- plot + ggplot2::coord_cartesian(clip = "off")
  
  return(plot)
}



#### topicsGridLegend ####
#'  Plot a grid (matrix) legend (available for the text-package)
#' @param bivariate_color_codes A vector of color codes specifying the colors for the 3x3 grid legend.
#'                              Default: c("#398CF9", "#60A1F7", "#5dc688", "#e07f6a", "#EAEAEA", "#40DD52", "#FF0000", "#EA7467", "#85DB8E").
#' @param filtered_test A data frame containing the filtered topic data. Must include a `color_categories` column.
#' @param cor_var A string used to name the correlation variable, included in the file name of the saved plot. Default: "".
#' @param save_dir Directory where the grid legend plot will be saved. Default: "./results".
#' @param figure_format File format for the saved grid legend plot. Examples: "svg", "png", "pdf". Default: "svg".
#' @param seed Seed for reproducibility, ensuring consistent plot generation. Default: 42.
#' @param width Width of the saved grid legend in inches. Default: 10.
#' @param height Height of the saved grid legend in inches. Default: 8.
#' @param y_axes_1 Specifies axis alignment for the grid legend. Options: 2 (2D grid with x and y axes) or 1 (1D legend for x-axis only). Default: 2.
#' @param legend_title Title text for the grid legend. Must be provided by the user.
#' @param legend_title_size Font size of the legend title text. Must be provided by the user.
#' @param titles_color Color of the title and axis labels in the legend. Must be provided by the user.
#' @param legend_x_axes_label Label for the x-axis of the grid legend. Must be provided by the user.
#' @param legend_y_axes_label Label for the y-axis of the grid legend. Must be provided by the user.
#' @param topic_data_all A data frame containing all topic data, including the `color_categories` column used for counting topics.
#' @param legend_number_color Color of the numeric annotations in the grid legend. Must be provided by the user.
#' @param legend_number_size Font size of the numeric annotations in the grid legend. Must be provided by the user.' @return A legend plot saved that can be combined with the plot object.
#' @param title_font Font family used for all non-word text elements in the plots (e.g., titles, axis labels, tick labels,
#' legend text, annotations). Default: "sans". Examples: "serif", "mono", or a system-installed font family name.
#' @importFrom tidyr gather separate
#' @importFrom dplyr mutate
#' @importFrom ggplot2 geom_tile ggtitle scale_fill_identity labs theme_void annotate theme element_text coord_fixed ggsave
#' @export
topicsGridLegend <- function(
    bivariate_color_codes = c(
      "#398CF9", "#60A1F7", "#5dc688",
      "#e07f6a", "#EAEAEA", "#40DD52",
      "#FF0000", "#EA7467", "#85DB8E"),
    filtered_test,
    cor_var = "",
    save_dir,
    figure_format = 'svg',
    seed = 42,
    width = 10, 
    height = 8, 
    y_axes_1 = 2,
    legend_title,
    legend_title_size,
    titles_color,
    legend_x_axes_label,
    legend_y_axes_label,
    topic_data_all,
    legend_number_color,
    legend_number_size,
    title_font = "sans"
) {
  if (y_axes_1 == 2){y_axes_1 <- ""}else{y_axes_1 <- "only_x_dimension"}
  legCor <- bivariate_color_codes
  if(y_axes_1 == "only_x_dimension"){
    bivariate_color_data <- tibble::tibble(
      "1 - 3" = '', "2 - 3" = '', "3 - 3" = '',
      "1 - 2" = legCor[1], "2 - 2" = legCor[2], "3 - 2" = legCor[3],
      "1 - 1" = '', "2 - 1" = '', "3 - 1" = '')
  }else{bivariate_color_data <- tibble::tibble(
    "1 - 3" = legCor[1], "2 - 3" = legCor[2], "3 - 3" = legCor[3],
    "1 - 2" = legCor[4], "2 - 2" = legCor[5], "3 - 2" = legCor[6],
    "1 - 1" = legCor[7], "2 - 1" = legCor[8], "3 - 1" = legCor[9]
  )}
  bivariate_color_data <- rbind(bivariate_color_data, bivariate_color_codes)
  bivariate_color_data <- bivariate_color_data[-1, ]
  
  if (y_axes_1 == "only_x_dimension") {
    # Only select 3 colors
    bivariate_color_data <- bivariate_color_data[, c(4, 5, 6)]
    colnames(bivariate_color_data) <- c("1 - 2", "2 - 2", "3 - 2")
    legend_y_axes_label <- " "}
  
  # To output the number of categories for dim 1 and dim 2 (plot 1dim or 2dim)
  if (y_axes_1 == "only_x_dimension") {
    categoryTotal_x_axes = c(
      sum(topic_data_all$color_categories == 1,
          na.rm = TRUE),
      sum(topic_data_all$color_categories == 2,
          na.rm = TRUE),
      sum(topic_data_all$color_categories == 3,
          na.rm = TRUE))
  }else{ categoryTotal_x_axes = c(
    sum(topic_data_all$color_categories == 4,
        na.rm = TRUE),
    sum(topic_data_all$color_categories == 5,
        na.rm = TRUE),
    sum(topic_data_all$color_categories == 6,
        na.rm = TRUE))}
  
  legend <- bivariate_color_data %>%
    tidyr::gather("group", "fill") %>%
    tidyr::separate(group, into = c("x", "y"), sep = " - ") %>%
    dplyr::mutate(
      x = as.integer(x),
      y = as.integer(y)
    ) %>%
    ggplot2::ggplot(ggplot2::aes(x, y)) +
    ggplot2::geom_tile(ggplot2::aes(fill = fill)) +
    ggplot2::ggtitle(paste0(legend_title)) +
    ggplot2::scale_fill_identity() +
    ggplot2::labs(
      x = legend_x_axes_label,
      y = legend_y_axes_label
    ) +
    ggplot2::theme_void() +
    {
      if (y_axes_1 != "only_x_dimension") {
        ggplot2::annotate(
          geom = "text", x = 1, y = 3, label = sum(topic_data_all$color_categories == 1,
                                                   na.rm = TRUE
          ),
          color = legend_number_color, size = legend_number_size
        )
      }
    } +
    {
      if (y_axes_1 != "only_x_dimension") {
        ggplot2::annotate(
          geom = "text", x = 2, y = 3, label = sum(topic_data_all$color_categories == 2,
                                                   na.rm = TRUE
          ),
          color = legend_number_color, size = legend_number_size
        )
      }
    } +
    {
      if (y_axes_1 != "only_x_dimension") {
        ggplot2::annotate(
          geom = "text", x = 3, y = 3, label = sum(topic_data_all$color_categories == 3,
                                                   na.rm = TRUE
          ),
          color = legend_number_color, size = legend_number_size
        )
      }
    } +
    ggplot2::annotate(
      geom = "text", x = 1, y = 2, label = categoryTotal_x_axes[1],
      color = legend_number_color, size = legend_number_size
    ) +
    ggplot2::annotate(
      geom = "text", x = 2, y = 2, label = categoryTotal_x_axes[2],
      color = legend_number_color, size = legend_number_size
    ) +
    ggplot2::annotate(
      geom = "text", x = 3, y = 2, label = categoryTotal_x_axes[3],
      color = legend_number_color, size = legend_number_size
    ) +
    {
      if (y_axes_1 != "only_x_dimension") {
        ggplot2::annotate(
          geom = "text", x = 1, y = 1, label = sum(topic_data_all$color_categories == 7,
                                                   na.rm = TRUE
          ),
          color = legend_number_color, size = legend_number_size
        )
      }
    } +
    {
      if (y_axes_1 != "only_x_dimension") {
        ggplot2::annotate(
          geom = "text", x = 2, y = 1, label = sum(topic_data_all$color_categories == 8,
                                                   na.rm = TRUE
          ),
          color = legend_number_color, size = legend_number_size
        )
      }
    } +
    {
      if (y_axes_1 != "only_x_dimension") {
        ggplot2::annotate(
          geom = "text", x = 3, y = 1, label = sum(topic_data_all$color_categories == 9,
                                                   na.rm = TRUE
          ),
          color = legend_number_color, size = legend_number_size
        )
      }
    } +
    ggplot2::theme(
      text = ggplot2::element_text(family = title_font),
      plot.title = ggplot2::element_text(hjust = 0.5, size = legend_title_size + 1, family = title_font),
      title = ggplot2::element_text(color = titles_color, family = title_font),
      axis.title.x = ggplot2::element_text(color = titles_color, family = title_font),
      axis.title = ggplot2::element_text(size = legend_title_size, family = title_font),
      axis.title.y = ggplot2::element_text(angle = 90, color = titles_color, family = title_font)
    ) +
    ggplot2::coord_fixed()
  
  if (!is.null(save_dir)){
    ggplot2::ggsave(paste0(save_dir,"/seed_", seed, 
                           "/wordclouds/",
                           "grid_legend_",
                           "corvar_", cor_var,
                           ".",
                           figure_format),
                    plot = legend, 
                    width = width, 
                    height = height, 
                    units = "in", 
                    create.dir = TRUE)
  }
  
  return(legend)
}


#' The function to create lda wordclouds
#' @param model (list) The trained model
#' @param test (list) The test results
#' @param popout (tibble) The tibble containing the topic idx to popout
#' @param color_negative_cor (R_obj) The color gradient for negative correlations
#' @param color_positive_cor (R_obj) The color gradient for positive correlations
#' @param grid_pos (numeric) The position for grid topics
#' @param scale_size (logical) Whether to scale the size of the words
#' @param scatter_legend_dots_alpha The transparency of the dots
#' @param scatter_legend_bg_dots_alpha The transparency of the dots
#' @param plot_topics_idx (vector) The topics to plot determined by index
#' @param p_alpha (integer) The p-value threshold to use for significance
#' @param highlight_topic_words (str vector) The dictionary to popout negative words to an individual plot for easier reading. 
#'  Default words are "not", "never". Individual words are elements in the vector. 
#' @param save_dir (string) The directory to save the wordclouds
#' @param figure_format (string) Set the figure format, e.g., svg, or png.
#' @param width (integer) The width of the topic (units = "in"). 
#' @param height (integer) The width of the topic (units = "in").
#' @param max_size (integer) The max size of the words.
#' @param seed (integer) The seed to set for reproducibility
#' @param word_font Font family used for the word text in the wordclouds (i.e., the plotted words only).
#' Default: "sans". Examples: "serif", "mono", or a system-installed font family name (e.g., "Arial", "Times New Roman").
#' @param title_font Font family used for all non-word text elements in the plots (e.g., titles, axis labels, tick labels,
#' legend text, annotations). Default: "sans". Examples: "serif", "mono", or a system-installed font family name.
#' @return nothing is returned, the wordclouds are saved in the save_dir
#' @noRd
topicsPlot1 <- function(
    model = NULL,
    ngrams = NULL,
    test = NULL,
    popout = NULL,
    color_negative_cor = NULL,
    color_positive_cor = NULL,
    grid_pos = "",
    scale_size = FALSE,
    scatter_legend_dots_alpha, 
    scatter_legend_bg_dots_alpha, 
    plot_topics_idx = NULL,
    p_alpha = 0.05,
    highlight_topic_words = c("not", "never"),    
    save_dir = NULL,
    figure_format = "svg",
    width = 10, 
    height = 8,
    max_size = 10, 
    seed = 42,
    word_font  = "sans",
    title_font = "sans"
){
  
  df_list = NULL
  
  if (!is.null(model)){
    if (model$model_type == "bert_topic") {
      
      if(!is.null(test)){
        num_topics <- nrow(test$test)
      } else {
        num_topics <- length(model$model$summary$topic)
        model$summary <- model$model$summary
      }
      
      if(num_topics == 0){
        stop("There are no significant topics to plot.")
      }
      
      if(model$model_type == "bert_topic"){
        df_list <- create_df_list_bert_topics(df = model$model)
        summary = model$model$summary
        cor_var = test$x_y_axis
        test_type = test$test_method
        test = test$test
      }
      
    } else {
      # if from mallet: 
      model <- name_cols_with_vocab(model, "phi", model$vocabulary)
      df_list <- create_topic_words_dfs(model$summary)
      df_list <- assign_phi_to_words(df_list, model$phi, "mallet")
    }
  }
  
  if (!is.null(test) && !is.null(model) && !model$model_type == "bert_topic"){
    summary = model$summary
    cor_var = test$x_y_axis
    test_type = test$test_method
    test = test$test
  }
  
  if (!is.null(model) && is.null(test)){
    summary = model$summary
  }
  
  if (is.null(model) && !is.null(ngrams) && !is.null(test)){
    test = test$test
  }
  
  plot <- create_plots(
    df_list = df_list, 
    summary = summary,
    ngrams = ngrams$ngrams,
    test = test, 
    test_type = test_type,
    cor_var = cor_var,
    popout = popout,
    color_negative_cor = color_negative_cor,
    color_positive_cor = color_positive_cor,
    grid_pos = grid_pos,
    scale_size = scale_size,
    plot_topics_idx = plot_topics_idx,
    p_alpha = p_alpha,
    highlight_topic_words = highlight_topic_words,  
    save_dir = save_dir,
    figure_format = figure_format,
    width = width, 
    height = height,
    max_size = max_size,
    seed = seed,
    word_font = word_font,
    title_font = title_font
  )
  
  return(plot)
}

#' The function sets default colors or arranges user specified colors
#' @param color_scheme (string or vector of strings) 
#' @return default colors or specified user colours in the right order and structure.
#' @noRd
colour_settings <- function(
    color_scheme, 
    test, 
    ngrams, 
    model, 
    dim){
  
  bivariate_color_codes <- NULL
  bivariate_color_codes_b <- NULL
  bivariate_color_codes_f <- NULL  
  #### Checking and arranging colors ####
  if(!color_scheme[[1]] == "default"){
    
    # Dim 0 (i.e., no test)
    if(is.null(test)){
      
      if(length(color_scheme) == 2) {
        bivariate_color_codes <- rep(color_scheme, 2)
      } else {
        stop("Please provide 2 colours in the color_scheme parameter or set it to 'default'.")
      }
    }
    
    # Dim 1: N-gram
    if (dim == 1 && !is.null(ngrams)){
      
      if(length(color_scheme) == 4) {
        bivariate_color_codes <- color_scheme
      } else {
        stop("Please provide 4 colours  in the color_scheme parameter or set it to 'default'.")
      }
    }
    
    # Dim 1: topics
    if (dim == 1 && !is.null(model)){
      
      if(length(color_scheme) == 6) {
        # Select every second color for "back" colour in the gradient 
        bivariate_color_codes_b <- color_scheme[seq(1, length(color_scheme), by = 2)]
        # Select every second color for "front" colour in the gradient 
        bivariate_color_codes_f <- color_scheme[seq(2, length(color_scheme), by = 2)]
        bivariate_color_codes_f <- setNames(bivariate_color_codes_f, seq_along(bivariate_color_codes_f))
        
      } else {
        stop("Please provide 6 colours for the gradient.")
      }
    }
    
    # Dim 2: topics
    if (dim == 2 && !is.null(model)){
      
      if(length(color_scheme) == 18) {
        # Select every second color for "back" colour in the gradient 
        bivariate_color_codes_b <- color_scheme[seq(1, length(color_scheme), by = 2)]
        # Select every second color for "front" colour in the gradient 
        bivariate_color_codes_f <- color_scheme[seq(2, length(color_scheme), by = 2)]
        bivariate_color_codes_f <- setNames(bivariate_color_codes_f, seq_along(bivariate_color_codes_f))
        
      } else {
        stop("Please provide 18 colours or use color_scheme = 'default'.")
      }
    }
  }
  
  #### Setting the (default) colours ####
  if (color_scheme[[1]] == "default"){
    
    if (is.null(test)){ # && is.null(ngrams)
      bivariate_color_codes <- c(
        # gradient colours 1 and 2
        "#EAEAEA", "darkblue", 
        "#EAEAEA", "darkblue")
    }
    
    if (dim == 1 && !is.null(ngrams)){
      # gradient pairs
      bivariate_color_codes <- c(
        "#d0d0d0", "#ff7c00", # negative ngrams colours (Orange)
        "#d0d0d0", "#00508c"  # positve ngrams colours (Blue)
      )
    }
    
    # Below colours are used in for loop iterations; so easiest to have back and front colors separate
    
    if (dim == 1 && is.null(ngrams)){
      # gradient pairs
      
      # Colors for the "background" words (The "Low" value)
      bivariate_color_codes_b <- c(
        "#d0d0d0",  "#d0d0d0","#d0d0d0"
      )
      # Colors for the "front" words (The "High" value)
      bivariate_color_codes_f <- c(
        "#ff7c00", "#a4a4a4", "#00508c" # Orange, Gray, Blue
      )
    }
    
    if (dim == 2){
      # Colors for the "background" words
      bivariate_color_codes_b <- rep("#d0d0d0", 9)
      
      # Colors for the "front" words 
      bivariate_color_codes_f <-  c(
        "#00508c",  # quadrant 1 (upper left corner)
        "#635089",  # quadrant 2 
        "#A64E85",  # quadrant 3 (upper right corner)
        "#00807d",  # quadrant 4
        "#a4a4a4",  # quadrant 5 (middle square)
        "#bf6f51",  # quadrant 6 
        "#009E73",  # quadrant 7 (bottom left corner)
        "#8a9f2e",  # quadrant 8 
        "#E69F00")  # quadrant 9 (bottom right corner)
    }
  }
  
  codes <- list(
    bivariate_color_codes,
    bivariate_color_codes_b,
    bivariate_color_codes_f)
  
  return(codes)
}


#' General function to clean characters in a specified column
#'
#' @param data  
#' @param column  
#' @return default colors or specified user colours in the right order and structure.
#' @noRd
clean_characters <- function(
    data, 
    column) {
  
  # Replace "<" and ">" with "_"
  data[[column]] <- gsub("[<>]", "_", data[[column]])
  
  # Replace "-" with "_-_"
  data[[column]] <- gsub("-", "_-_", data[[column]])
  
  # Replace digits 0-9 with "_digit_"
  data[[column]] <- gsub("([0-9])", "_\\1_", data[[column]])
  
  # Special case: Replace "0" with "_10_"
  data[[column]] <- gsub("_0_", "_10_", data[[column]])
  
  return(data)
}

# Wrapper function for cleaning 'ngrams$ngrams$ngrams'
clean_characters_for_plotting_grams <- function(ngrams) {
  
  ngrams$ngrams <- clean_characters(ngrams$ngrams, "ngrams")
  
  return(ngrams)
}

# Wrapper function for cleaning 'test$test$top_terms'
clean_characters_for_plotting_test <- function(test) {
  
  test$test <- clean_characters(test$test, "top_terms")
  
  return(test)
}


#' Combine topics and distribution legend into an overview figure (experimental)
#'
#' Assembles a single composite figure from the individual topic word clouds
#' returned by `topicsPlot()`. The shape of the composition depends on
#' `overview_plot_type`:
#' \itemize{
#'   \item `"topics_grid"` / `"textTopics"`: a grid of the most prevalent topics
#'     (no test), controlled by `overview_n_topics`.
#'   \item `"one_dimension_topics"`: three columns (negative / non-significant /
#'     positive), controlled by `scatter_legend_n` (length 3).
#'   \item `"two_dimension_topics"`: a 3x3 quadrant layout with the distribution
#'     legend in the centre, controlled by `scatter_legend_n` (length 9).
#' }
#'
#' This function is intended as a convenience for quickly inspecting results.
#' For publication figures, users typically save the individual topic plots
#' (returned separately by `topicsPlot()`) and assemble them in external
#' software such as PowerPoint, Google Drawings, or Illustrator.
#'
#' @param plot_list (list) Output from the `topicsPlot()` function.
#' @param overview_plot_type (character) One of `"topics_grid"`, `"textTopics"`,
#'   `"one_dimension_topics"`, or `"two_dimension_topics"`.
#' @param overview_n_topics (integer) For the `"topics_grid"` / `"textTopics"`
#'   overview only: the number of topic plots to include. Plots are arranged
#'   in 3 columns (the default of 9 produces a 3x3 grid). Ignored for the
#'   test-based 1D and 2D overviews, which instead use `scatter_legend_n`.
#'   Default: 9.
#' @param scatter_legend_n (numeric vector) For the test-based 1D and 2D
#'   overviews: the per-category counts that determine how many topic plots
#'   appear in the overview. Length 3 for 1D (negative, non-significant,
#'   positive) and length 9 for 2D (one value per quadrant). A value of 0
#'   omits that category. Ignored for the prevalent-topics overview.
#' @param title_font Font family used for all non-word text elements in the plots
#'   (e.g., titles, axis labels, tick labels, legend text, annotations).
#'   Default: `"sans"`.
#' @return A patchwork composition that can be printed, saved with `ggsave()`,
#'   or further modified with patchwork operators.
#' @importFrom  ggplot2 labs element_blank theme margin element_text
#' @importFrom patchwork wrap_plots plot_layout plot_spacer
#' @export
topicsPlotOverview <- function(
    plot_list,
    overview_plot_type,
    overview_n_topics = 9,
    scatter_legend_n = NULL,
    title_font = "sans"
) {

  # ---- Sanity check: warn if the composition is going to be very dense ----
  # The overview is a convenience layout. Users who need many topics per panel
  # generally get better results by saving the individual plots and assembling
  # them in external software (PowerPoint, Google Drawings, Illustrator, etc.).
  overview_total <- NULL
  if (overview_plot_type %in% c("topics_grid", "textTopics")) {
    overview_total <- overview_n_topics
  } else if (overview_plot_type == "one_dimension_topics" &&
             !is.null(scatter_legend_n)) {
    overview_total <- sum(scatter_legend_n[seq_len(min(3, length(scatter_legend_n)))],
                          na.rm = TRUE)
  } else if (overview_plot_type == "two_dimension_topics" &&
             !is.null(scatter_legend_n)) {
    overview_total <- sum(scatter_legend_n[seq_len(min(9, length(scatter_legend_n)))],
                          na.rm = TRUE)
  }

  if (!is.null(overview_total) && overview_total > 12) {
    message(
      "Note: the overview is being built from ", overview_total, " topic plots, ",
      "which may produce a cramped figure with overlapping words. ",
      "The individual topic plots are also returned by `topicsPlot()` and can be ",
      "saved separately (e.g., via `save_dir` / `figure_format`) and assembled ",
      "manually in PowerPoint, Google Drawings, Illustrator, or a similar tool ",
      "for publication-quality layouts."
    )
  }
  
  # Helper: Handles NULL, Lists of plots, and Single plots
  get_plot_or_spacer <- function(plot_obj) {
    if (is.null(plot_obj) || length(plot_obj) == 0) {
      return(patchwork::plot_spacer())
    }
    
    # If it's a list (like square1), take first. If it's a ggplot, take it.
    if (is.list(plot_obj) && !ggplot2::is.ggplot(plot_obj)) {
      p <- plot_obj[[1]]
    } else {
      p <- plot_obj
    }
    
    return(
      p +
        ggplot2::theme_void() + 
        ggplot2::labs(x = NULL, y = NULL) + 
        ggplot2::theme(
          text = ggplot2::element_text(family = title_font),
          plot.margin = ggplot2::margin(2, 2, 2, 2)
        )
    )
  } 
  
  # N-grams
  if (overview_plot_type == "ngrams_grid") {
    p_pos <- get_plot_or_spacer(plot_list$positive_association)
    p_neg <- get_plot_or_spacer(plot_list$negative_association)

    # Place each plot's legend on its own side: left for negative, right for positive.
    # Force prevalence (size) on top and estimate (colour) below on both sides.
    legend_order <- ggplot2::guides(
      size   = ggplot2::guide_legend(order = 1),
      colour = ggplot2::guide_colourbar(order = 2)
    )

    p_neg <- p_neg + ggplot2::theme(legend.position = "left")  + legend_order
    p_pos <- p_pos + ggplot2::theme(legend.position = "right") + legend_order

    combined <- (p_neg | p_pos)
  }
  
  # Selected topics Grid Version (3 columns) ---
  if (overview_plot_type == "topics_grid" | overview_plot_type == "textTopics") {
    # Keep only the first `overview_n_topics` plots in the grid
    plots_for_grid <- plot_list
    if (!is.null(overview_n_topics) &&
        length(plots_for_grid) > overview_n_topics) {
      plots_for_grid <- plots_for_grid[seq_len(overview_n_topics)]
    }

    cleaned_plots <- lapply(plots_for_grid, get_plot_or_spacer)

    combined <- patchwork::wrap_plots(cleaned_plots, ncol = 3) +
      patchwork::plot_layout(guides = 'collect') &
      ggplot2::theme(
        text = ggplot2::element_text(family = title_font),
        plot.margin = ggplot2::margin(5, 5, 5, 5)
      )
  }
  
  # One-dimensional topic plot
  if (overview_plot_type == "one_dimension_topics") {
    # Default to a single plot per category if scatter_legend_n is missing
    n_per_cat <- if (is.null(scatter_legend_n)) c(1, 1, 1) else scatter_legend_n
    if (length(n_per_cat) < 3) n_per_cat <- rep(n_per_cat, length.out = 3)

    # Helper: take the first `n` plots from a square list, padding with spacers
    take_n_plots <- function(square_obj, n) {
      if (n <= 0) return(list(patchwork::plot_spacer()))
      if (is.null(square_obj) || length(square_obj) == 0) {
        return(replicate(n, patchwork::plot_spacer(), simplify = FALSE))
      }
      # If it's a single ggplot, wrap it
      if (ggplot2::is.ggplot(square_obj)) square_obj <- list(square_obj)
      available <- length(square_obj)
      take <- min(n, available)
      plots <- lapply(square_obj[seq_len(take)], function(p) {
        get_plot_or_spacer(p)
      })
      if (take < n) {
        plots <- c(plots, replicate(n - take, patchwork::plot_spacer(), simplify = FALSE))
      }
      plots
    }

    col_neg <- take_n_plots(plot_list$square1, n_per_cat[1])
    col_mid <- take_n_plots(plot_list$square2, n_per_cat[2])
    col_pos <- take_n_plots(plot_list$square3, n_per_cat[3])

    # Stack each category vertically into a column
    stack_col <- function(plots) {
      Reduce(function(a, b) a / b, plots)
    }

    col1 <- stack_col(col_neg)
    col2 <- stack_col(col_mid)
    col3 <- stack_col(col_pos)

    top_row <- (col1 | col2 | col3)

    combined <- patchwork::wrap_plots(top_row, plot_list$distribution, ncol = 1) +
      patchwork::plot_layout(heights = c(max(n_per_cat), 1)) &
      ggplot2::theme(text = ggplot2::element_text(family = title_font))
  }
  
  # Two-dimensional topic plot
  if (overview_plot_type == "two_dimension_topics") {
    # Default to a single plot per quadrant if scatter_legend_n is missing
    n_per_q <- if (is.null(scatter_legend_n)) rep(1, 9) else scatter_legend_n
    if (length(n_per_q) < 9) n_per_q <- rep(n_per_q, length.out = 9)

    # Helper: combine the first `n` plots from a square list horizontally
    # into a single cell. Returns a patchwork composition or a spacer.
    build_quadrant_cell <- function(square_obj, n) {
      if (is.null(n) || n <= 0) return(patchwork::plot_spacer())
      if (is.null(square_obj) || length(square_obj) == 0) return(patchwork::plot_spacer())
      if (ggplot2::is.ggplot(square_obj)) square_obj <- list(square_obj)

      take <- min(n, length(square_obj))
      plots <- lapply(square_obj[seq_len(take)], function(p) get_plot_or_spacer(p))

      if (length(plots) == 1) return(plots[[1]])
      Reduce(function(a, b) a | b, plots)
    }

    q1 <- build_quadrant_cell(plot_list$square1, n_per_q[1])
    q2 <- build_quadrant_cell(plot_list$square2, n_per_q[2])
    q3 <- build_quadrant_cell(plot_list$square3, n_per_q[3])
    q4 <- build_quadrant_cell(plot_list$square4, n_per_q[4])
    q6 <- build_quadrant_cell(plot_list$square6, n_per_q[6])
    q7 <- build_quadrant_cell(plot_list$square7, n_per_q[7])
    q8 <- build_quadrant_cell(plot_list$square8, n_per_q[8])
    q9 <- build_quadrant_cell(plot_list$square9, n_per_q[9])

    dist_plot <- plot_list$distribution +
      ggplot2::theme(
        text = ggplot2::element_text(family = title_font),
        plot.margin = ggplot2::margin(5, 5, 5, 5)
      )

    combined <- (q1 | q2 | q3) /
      (q4 | dist_plot | q6) /
      (q7 | q8 | q9) +
      patchwork::plot_layout(guides = 'collect') &
      ggplot2::theme(
        text = ggplot2::element_text(family = title_font),
        plot.margin = ggplot2::margin(0, 0, 0, 0),
        axis.title = ggplot2::element_blank()
      )
  }
  
  return(combined)
}

#plotout <- create_overview_plot(plot_list)

#' Plot word clouds
#' 
#' This function create word clouds and topic figures
#' @param model (list) A trained topics model, e.g., from topicsModel(). Should be NULL if plotting ngrams.
#' @param ngrams (list) The output from the the topicsGram() function. Should be NULL if plotting topics.
#' Note 1: it is not possible to plot tags like <place>; so the < are replaced with underscore. 
#' Note 2: it is not possible to plot dash - alone, it is replaced with `_-_`.
#' @param test (list) The test results; if plotting according to dimension(s) include the object from topicsTest() function. 
#' @param p_alpha (integer) The p-value threshold to use for significance testing.
#' @param p_adjust_method (character) Method to adjust/correct p-values for multiple comparisons (default = "none"; 
#' see also "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr").
#' @param ngrams_max (integer) The maximum number of n-grams to plot.
#' @param ngram_select (character) Method to select ngrams_max, when using both ngram and test use "prevalence" or "estimate"; 
#' if you only use ngrams use "pmi", "frequency", or "prevalence". 
#' @param color_scheme (string 'default' or vector) The color scheme.
#'  
#' For plots not including a test, the color_scheme should in clude 2 colours (1 gradient pair), such as:
#'
#' c("lightgray", "darkblue)
#' 
#' For 1 dimensional plots of n-grams it should contain 4 colours (2 gradient pairs), such as: 
#'
#' c(
#' "#EAEAEA", "darkred", # negative ngrams colors
#' 
#' "#EAEAEA", "darkgreen" # positve ngrams colors)
#' 
#' 
#' 
#' For 1-dimension plots of topics, it should contain 6 colours (3 gradient pairs), such as 
#'
#'  c(
#' "#EAEAEA", "darkred",     # negative topics colors
#' 
#' "#EAEAEA", "darkgray",     # colours of topics not significantly associated
#' 
#' "#EAEAEA", "darkgreen"     # positve topics colors)
#' 
#' 
#'
#'  For 2-dimensional plots of topics, the color scheme should contain 18 colours (9 gradient pairs), such as:
#'  
#'  c(
#'   "lightgray", "#398CF9",     # quadrant 1 (upper left corner)
#'   
#'   "lightgray", "#60A1F7",     # quadrant 2 
#'   
#'   "lightgray", "#5dc688",     # quadrant 3 (upper right corner)
#'   
#'   "lightgray", "#e07f6a",     # quadrant 4
#'   
#'   "lightgray", "darkgray",     # quadrant 5 (middle square)
#'   
#'   "lightgray", "#40DD52",     # quadrant 6 
#'   
#'   "lightgray", "#FF0000",     # quadrant 7 (bottom left corner)
#'   
#'   "lightgray", "#EA7467",     # quadrant 8 
#'   
#'   "lightgray", "#85DB8E")     # quadrant 9 (bottom right corner).
#'
#' 
#' @param word_font Font family used for the word text in the wordclouds (i.e., the plotted words only).
#' Default: "sans". Examples: "serif", "mono", or a system-installed font family name (e.g., "Arial", "Times New Roman").
#' @param title_font Font family used for all non-word text elements in the plots (e.g., titles, axis labels, tick labels,
#' legend text, annotations). Default: "sans". Examples: "serif", "mono", or a system-installed font family name.
#' @param overview_plot (boolean) Whether to produce a combined overview plot (`$overview_plot`)
#' that arranges selected topic word clouds and (where applicable) the distribution
#' legend into a single figure (experimental). The individual topic plots are also
#' returned separately in the result list, so users who prefer to assemble their own
#' figure in PowerPoint, Google Drawings, Illustrator, etc. can ignore the overview
#' and combine the individual plots manually. Default: TRUE.
#' @param overview_n_topics (integer) Used only by the **prevalent-topics** overview
#' (i.e., when `plot_n_most_prevalent_topics` is set and no `test` is provided).
#' Number of topic plots to include in the grid; plots are arranged in 3 columns
#' (e.g., the default of 9 produces a 3x3 grid). For test-based overviews
#' (1D and 2D differential plots), the number of topics shown is instead
#' controlled by `scatter_legend_n`. Default: 9.
#' @param highlight_topic_words (str vector) Words to highlight in topics (e.g., negative words). Format: highlight_topic_words = c("not", "never"). The default value is NULL.
#' @param allowed_word_overlap (numeric) A filter function determining the maximum number of identical words in the topics to be plotted. 
#' This filter removes topics within each "color group" and also include removing topics from the distribution and grid legends; 
#' (Note that the adjustment for multiple comparison is taking place before these are removed; i.e., the adjusted p-values are not affected by this filter).   
#' @param scale_size (logical) Whether to scale the size of the words.
#' @param plot_topics_idx (vector)  The index or indices of the topics to plot 
#' (look in the model-object for the indices). They can, for example, be c(1, 3:5) to plot topic t_1, t_3, t_4 and t_5) (optional).
#' @param plot_n_most_prevalent_topics (numeric) Plots the n most prevalent topics in a given model. 
#' @param save_dir (string) The directory to save the plots.
#' @param figure_format (string) Set the figure format, e.g., ".svg", or ".png".
#' @param width (integer) The width of the topic (units = "in"). 
#' @param height (integer) The width of the topic (units = "in").
#' @param max_size (integer) The maximum size of the words.
#' @param seed (integer) The seed to set for reproducibility.
#' @param scatter_legend_dot_size (integer) The size of dots in the scatter legend. If set to "prevalence", the size will change accordingly.
#' @param scatter_legend_bg_dot_size (integer) The size of background dots in the scatter legend.
#' @param scatter_legend_n (numeric or vector) A vector determining the number of dots to emphasize
#' in each quadrant of the scatter legend. For 1-dimensional plots this is a length-3 vector
#' `c(negative, non-significant, positive)`; for 2-dimensional plots it is a length-9 vector
#' (one value per quadrant). Example: `c(1,1,1,1,0,1,1,1,1)` highlights one dot in each
#' quadrant except the middle one.
#'
#' These same counts also determine how many topic word clouds appear in the test-based
#' `$overview_plot` (1D or 2D), so the dots emphasized in the scatter legend and the
#' word clouds in the overview stay in sync. Setting larger values produces a denser
#' overview; setting a value to 0 omits that category entirely. If you want to build
#' your own figure layout in external software (PowerPoint, Google Drawings, etc.),
#' the individual topic plots are always returned separately in the result list — the
#' overview is only one possible composition.
#' @param scatter_legend_dots_alpha (numeric) The transparency alphe level of the dots.
#' @param scatter_legend_bg_dots_alpha (numeric) The transparency alphe level of the background dots.
#' @param scatter_legend_method (string) The method to filter topics to be emphasized in the scatter legend; either "mean", "max_x", or "max_y".
#' @param scatter_legend_specified_topics (vector) Specify which topic(s) to emphasize in the scatter legend. 
#' @param scatter_legend_circles Plot concentric circles for the scatter legend
#' @param scatter_legend_circles_radius Radius of first concentric circle
#' @param scatter_legend_circles_num Number of Concentric circles
#' For example, c("t_1", "t_2"). If set, scatter_legend_method will have no effect.
#' @param scatter_legend_topic_n (boolean) If TRUE, the topic numbers are shown in the scatter legend.
#' @param scatter_show_axis_values (boolean) If TRUE, the estimate values are shown on the distribution plot axes.
#' @param grid_legend_title Title of the grid topic plot.
#' @param grid_legend_title_size Title size of the grid topic plot.
#' @param grid_legend_title_color Legend title color of the grid topic plot.
#' @param grid_legend_x_axes_label x-axis label of the grid topic plot.
#' @param grid_legend_y_axes_label y-axis label of the grid topic plot.
#' @param grid_legend_number_color Text color in the legend boxes of the grid topic plot.
#' @param grid_legend_number_size Text size in the legend boxes.
#' @return The function provides a list of topic plots (if there are any significant topics), a legend plot, and a plot showing the topic distribution.
#' If save_dir is specified, it saves all plots in this directory. 
#' If you want to show all plots irrespective of the topics' significance, set p_alpha = 1. 
#' @importFrom dplyr filter arrange desc top_n select
#' @importFrom ggplot2 scale_color_gradient
#' @importFrom tibble as_tibble
#' @importFrom stats p.adjust
#' @export
topicsPlot <- function(
    model = NULL,
    ngrams = NULL,
    test = NULL,
    p_alpha = 0.05,
    p_adjust_method = "none",
    ngrams_max = 30,
    ngram_select = "estimate",
    color_scheme = "default",
    word_font  = "sans",
    title_font = "sans",
    overview_plot = TRUE,
    overview_n_topics = 9,
    highlight_topic_words = NULL,
    scale_size = FALSE,
    plot_topics_idx = NULL,
    allowed_word_overlap = NULL,
    plot_n_most_prevalent_topics = NULL,
    save_dir = NULL,
    figure_format = "svg",
    width = 6,
    height = 5,
    max_size = 10,
    seed = 42,
    scatter_legend_dot_size = c(3,8),
    scatter_legend_bg_dot_size = c(1,3),
    scatter_legend_dots_alpha = 0.8,
    scatter_legend_bg_dots_alpha = 0.2,
    scatter_legend_n = c(1,1,1,1,0,1,1,1,1),
    scatter_legend_method = c("mean"),
    scatter_legend_specified_topics = NULL,
    scatter_legend_topic_n = FALSE,
    scatter_show_axis_values = TRUE,
    scatter_legend_circles = FALSE,
    scatter_legend_circles_radius = 0,
    scatter_legend_circles_num = 4,
    grid_legend_title = "legend_title",
    grid_legend_title_size = 5,
    grid_legend_title_color = 'black',
    grid_legend_x_axes_label = "legend_x_axes_label",
    grid_legend_y_axes_label = "legend_y_axes_label",
    grid_legend_number_color = 'white',
    grid_legend_number_size = 15
){
  
  set.seed(seed)
  
  #### Setting the number of dimensions to plot ####
  if(is.null(test)){
    dim = 0
  }
  
  if(!is.null(test)){
    dim = 1
    if(ncol(test$test) == 12) {
      dim = 2
    }
  }
  
  #### Setting type of overview plot ####
  if(overview_plot) {
    if(!is.null(test)) {
      overview_type = "one_dimension_topics"
      if(ncol(test$test) == 12) {
        overview_type = "two_dimension_topics"
      }
    }
    
    if(!is.null(plot_topics_idx) ){
      if(plot_topics_idx[[1]] == "textTopics"){
        overview_type = "textTopics"
        plot_topics_idx = NULL
      }
    }
    
    if(!is.null(plot_topics_idx) | !is.null(plot_n_most_prevalent_topics)) {
      overview_type = "topics_grid"
    }
    
    if(!is.null(ngrams)) {
      overview_type = "ngrams_grid"
    }
  }
  
  #### Setting colors ####
  codes <- colour_settings(
    color_scheme = color_scheme, 
    model = model,
    test = test, 
    ngrams = ngrams, 
    dim = dim)
  
  bivariate_color_codes   <- codes[[1]]
  bivariate_color_codes_b <- codes[[2]]
  bivariate_color_codes_f <- codes[[3]]
  
  if (!is.null(bivariate_color_codes_f)){
    names(bivariate_color_codes_f) <- as.character(seq(1:length(bivariate_color_codes_f)))
  }
  
  #### Controlling parameter settings and giving instructions #####
  if (!is.vector(scatter_legend_n) || !is.numeric(scatter_legend_n)){
    msg <- "The parameter 'scatter_legend_n' should be either a numeric vector or a number.\n"
    message(colourise(msg, "brown"))
    return (NULL)
  }
  
  #### Add adjustment of p-values for multiple comparisons ####
  if (p_adjust_method != "none"){
    test$test[[8]]<- stats::p.adjust(p = test$test[[7]],
                                     method = p_adjust_method)
    
    if(dim == 2) test$test[[12]]<- stats::p.adjust(p = test$test[[11]],
                                                   method = p_adjust_method)
  } 
  
  if(p_adjust_method == "none"){
    test$test[[8]] <- test$test[[7]]
    if(dim == 2) test$test[[12]] <- test$test[[11]]
  }
  
  #### Setting colour-categories: Selecting elements to plot according to the p_alpha ####
  if (dim == 1) {
    bak1 <- colnames(test$test)[c(5,8)]
    colnames(test$test)[c(5,8)] <- c('x_plotted', 'adjusted_p_values.x')
    test$test <- topicsNumAssign_dim2(test$test, p_alpha, 1)
    colnames(test$test)[c(5,8)] <- bak1
  }
  if (dim == 2){
    bak1 <- colnames(test$test)[c(5,8,9,12)]
    colnames(test$test)[c(5,8,9,12)] <- c('x_plotted', 'adjusted_p_values.x',
                                          'y_plotted', 'adjusted_p_values.y')
    test$test <- topicsNumAssign_dim2(test$test, p_alpha, 2)
    colnames(test$test)[c(5,8,9,12)] <- bak1
  }
  
  #### Filtering duplicate topics #### 
  if (!is.null(allowed_word_overlap) & is.null(plot_n_most_prevalent_topics)){
    arranged_topics <- test$test 
    max_n_texts <- nrow(test$test )
    original_col_order <- names(arranged_topics)
    
    test$test <- arranged_topics %>%
      dplyr::mutate(color_categories1 = color_categories) %>%
      dplyr::group_by(color_categories) %>%
      dplyr::group_modify(~ select_non_overlapping_texts(
        df = .x, 
        text_column = "top_terms", 
        n_texts = max_n_texts, 
        allowed_word_overlap)) %>%
      dplyr::ungroup() %>%
      dplyr::select(-color_categories1) %>%
      dplyr::select(dplyr::all_of(original_col_order))
  }
  
  #### Selecting the most prevalence topics ####
  if(!is.null(plot_n_most_prevalent_topics) & !is.null(plot_topics_idx)){
    stop("Please note that you cannot set both the plot_n_most_prevalent_topics and the plot_topics_idx parameters.")
  }
  
  if (!is.null(plot_n_most_prevalent_topics)) {
    arranged_topics <- model$summary %>% 
      dplyr::arrange(dplyr::desc(prevalence))
    
    arranged_topics <- select_non_overlapping_texts(
      arranged_topics, 
      "top_terms", 
      n_texts = plot_n_most_prevalent_topics, 
      allowed_word_overlap = allowed_word_overlap 
    )
    
    plot_topics_idx <- arranged_topics$topic
  }
  
  #### NGRAM filtering and fixing tags (e.g., <place>) #### 
  if(!is.null(ngrams) & !is.null(ngrams_max)){
    
    if(is.null(test)){
      if (!ngram_select %in% c("pmi", "frequency", "prevalence")){
        stop("ngram_select incorrect -- can only select pmi, frequency, or prevalence when not including a test.")
      }
      
      ngrams$ngrams <- ngrams$ngrams %>% 
        dplyr::arrange(
          if (ngram_select == "pmi") {
            dplyr::desc(pmi)
          } else if (ngram_select == "frequency") {
            dplyr::desc(freq)
          } else if (ngram_select == "prevalence") {
            dplyr::desc(prevalence)
          } else {
            stop("Invalid value for ngram_select")
          }
        ) %>%
        dplyr::slice_head(n = ngrams_max)
      
      ngrams <- clean_characters_for_plotting_grams(ngrams)
    }
    
    if(!is.null(test)){
      if (!ngram_select %in% c("prevalence", "estimate")){
        stop("ngram_select incorrect -- when using ngram and test use one of prevalence or estimate")
      }
      
      beta_column <- names(test$test)[grepl("_beta$", names(test$test))][1]
      
      positive_ngrams <- test$test %>%
        dplyr::filter(.data[[beta_column]] > 0) %>%
        {
          if (ngram_select == "estimate") {
            dplyr::arrange(., dplyr::desc(.data[[beta_column]]))
          } else if (ngram_select == "prevalence") {
            dplyr::arrange(., dplyr::desc(prevalence))
          } else {
            stop("Invalid value for ngram_select")
          }
        } %>%
        dplyr::slice_head(n = ngrams_max)
      
      negative_ngrams <- test$test %>%
        dplyr::filter(.data[[beta_column]] < 0) %>%
        {
          if (ngram_select == "estimate") {
            #dplyr::arrange(., dplyr::desc(.data[[beta_column]]))
            dplyr::arrange(., .data[[beta_column]])
          } else if (ngram_select == "prevalence") {
            dplyr::arrange(., dplyr::desc(prevalence))
          } else {
            stop("Invalid value for ngram_select")
          }
        } %>%
        dplyr::slice_head(n = ngrams_max)
      
      
      test$test <- dplyr::bind_rows(positive_ngrams, negative_ngrams)
      test <- clean_characters_for_plotting_test(test)
    }
  }
  
  #### Making the plots ####
  if(!is.null(model) & is.null(test) | 
     !is.null(ngrams) && !is.null(test)|
     !is.null(ngrams) && is.null(test)){
    
    plot_list <- topicsPlot1(
      model = model,
      ngrams = ngrams,
      test = test,
      p_alpha = p_alpha,
      highlight_topic_words = NULL,
      scale_size = scale_size,
      plot_topics_idx = plot_topics_idx,
      popout = NULL,
      color_negative_cor = ggplot2::scale_color_gradient(
        low = bivariate_color_codes[1], high = bivariate_color_codes[2]),
      color_positive_cor = ggplot2::scale_color_gradient(
        low = bivariate_color_codes[3], high = bivariate_color_codes[4]),
      save_dir = save_dir,
      figure_format = figure_format,
      width = width, 
      height = height,
      max_size = max_size, 
      seed = seed,
      word_font = word_font,
      title_font = title_font
    )
  }
  
  #### 1- or 2 dimensional topic-plots ####
  if(is.null(ngrams) & !is.null(test$test)){
    popout1 <- topicsScatterLegend(
      bivariate_color_codes = bivariate_color_codes_f,
      filtered_test = test$test,
      num_popout = scatter_legend_n,
      y_axes_1 = dim,
      cor_var = test$x_y_axis,
      label_x_name = grid_legend_x_axes_label,
      label_y_name = grid_legend_y_axes_label,
      way_popout_topics = scatter_legend_method,
      user_spec_topics = scatter_legend_specified_topics,
      allow_topic_num_legend = scatter_legend_topic_n,
      scatter_popout_dot_size = scatter_legend_dot_size,
      scatter_bg_dot_size = scatter_legend_bg_dot_size,
      scatter_legend_dots_alpha = scatter_legend_dots_alpha, 
      scatter_legend_bg_dots_alpha = scatter_legend_bg_dots_alpha,
      scatter_show_axis_values = scatter_show_axis_values,
      scatter_legend_circles = scatter_legend_circles,
      scatter_legend_circles_radius = scatter_legend_circles_radius,
      scatter_legend_circles_num = scatter_legend_circles_num,
      save_dir = save_dir,
      figure_format = figure_format,
      width = 15,
      height = 15*9/16,
      seed = seed,
      title_font = title_font
    )
    popout <- popout1$popout
  }
  
  if (!is.null(model) & !is.null(test)){
    
    if (dim == 1){
      plot_list <- list()
      plot_list <- vector("list", length = 3)
      names(plot_list) <- paste0("square", 1:3)
      
      for (i in 1:3){
        if (! (i %in% test$test$color_categories)){next}
        
        filtered_test <- test
        filtered_test$test <- dplyr::filter(
          tibble::as_tibble(filtered_test$test,.name_repair="minimal"),
          color_categories == i)
        color_b <- bivariate_color_codes_b[i]
        color_f <- bivariate_color_codes_f[i]
        
        plot_topics_idx <- as.numeric(sub(".*_", "", filtered_test[["test"]]$topic))
        
        plot <- topicsPlot1(
          model = model,
          test = filtered_test,
          popout = popout,
          color_negative_cor = ggplot2::scale_color_gradient(
            low = color_b, high = color_f),
          color_positive_cor = ggplot2::scale_color_gradient(
            low = color_b, high = color_f),
          grid_pos = i,
          scale_size = scale_size,
          plot_topics_idx = plot_topics_idx,
          p_alpha = p_alpha,
          highlight_topic_words = highlight_topic_words,
          save_dir = save_dir,
          figure_format = figure_format,
          width = width, 
          height = height,
          max_size = max_size, 
          seed = seed,
          word_font = word_font,
          title_font = title_font
        )
        plot_list[[i]] <- plot
      }
    }
    
    if (dim == 2){
      plot_list <- list()
      plot_list <- vector("list", length = 9)
      names(plot_list) <- paste0("square", 1:9)
      for (k in 1:9){
        if (! (k %in% test$test$color_categories)){next}
        filtered_test <- test
        filtered_test$test <- dplyr::filter(
          tibble::as_tibble(filtered_test$test,.name_repair="minimal"),
          color_categories == k)
        color_b <- bivariate_color_codes_b[k]
        color_f <- bivariate_color_codes_f[k]
        
        plot_topics_idx <- as.numeric(sub(".*_", "", filtered_test[["test"]]$topic))
        
        plot <- topicsPlot1(
          model = model,
          test = filtered_test,
          popout = popout,
          color_negative_cor = ggplot2::scale_color_gradient(low = color_b, high = color_f),
          color_positive_cor = ggplot2::scale_color_gradient(low = color_b, high = color_f),
          grid_pos = k,
          scale_size = scale_size,
          plot_topics_idx = plot_topics_idx,
          p_alpha = p_alpha,
          highlight_topic_words = highlight_topic_words,
          save_dir = save_dir,
          figure_format = figure_format,
          width = width, 
          height = height,
          max_size = max_size, 
          seed = seed,
          word_font = word_font,
          title_font = title_font
        )
        plot_list[[k]] <- plot
      }
    }
    
    legend <- topicsGridLegend(
      bivariate_color_codes = bivariate_color_codes_f,
      filtered_test = test$test,
      cor_var = test$x_y_axis,
      save_dir = save_dir,
      figure_format = figure_format,
      seed = seed,
      y_axes_1 = dim,
      legend_title = grid_legend_title,
      legend_title_size = grid_legend_title_size,
      titles_color = grid_legend_title_color,
      legend_x_axes_label = grid_legend_x_axes_label,
      legend_y_axes_label = grid_legend_y_axes_label,
      topic_data_all = test[["test"]],
      legend_number_color = grid_legend_number_color,
      legend_number_size = grid_legend_number_size,
      title_font = title_font
    )
    
    plot_list[["legend"]] <- legend
    plot_list[["distribution"]] <- popout1$legend
  }
  
  # Create overview image
  if(overview_plot){
    overview_p <- topicsPlotOverview(
      plot_list = plot_list,
      overview_plot_type = overview_type,
      overview_n_topics = overview_n_topics,
      scatter_legend_n = scatter_legend_n,
      title_font = title_font
    )
    plot_list[["overview_plot"]] <- overview_p
  }
  
  return(plot_list)
}

# Function to add missing rows to a stats tibble in case one ngram is removed completely
#' @param input_tibble (tibble) A stats tibble with n_grams type column and the number of removed ngrams
#' @param value_list (character) A list of all ngram types
#' @importFrom dplyr bind_rows
#' @importFrom data.table :=
#' @noRd
add_missing_rows <- function(vec, full_range) {
  # 1. Handle NULLs immediately
  if (is.null(vec) || length(vec) == 0) {
    return(rep(0, length(full_range)))
  }
  
  # 2. Ensure we are working with a plain numeric vector
  # unname() strips the "1", "2", "3" labels that cause the dimnames error
  clean_vec <- as.numeric(unname(vec))
  
  # 3. If the vector is already the right length, just return it
  if (length(clean_vec) == length(full_range)) {
    return(clean_vec)
  }
  
  # 4. If it's shorter (e.g., missing 3-grams), pad it with 0s
  # (This assumes your input vec is always in order of ngram_type)
  length_diff <- length(full_range) - length(clean_vec)
  return(c(clean_vec, rep(0, length_diff)))
}


# Function to filter ngrams based on different modes
#' @param ngrams (tibble) A tibble with n_grams and freq columns representing n-grams and their frequencies.
#' @param removal_mode (character) The mode of filtering. Choose from 'term', 'frequency', or 'prevalence'.
#' @param removal_rate_most (numeric) The number of terms to remove from the top or the maximum frequency.
#' @param removal_rate_least (numeric) The number of terms to remove from the bottom or the minimum frequency.
#' @importFrom dplyr arrange filter
#' @importFrom utils tail
#' @noRd
filter_ngrams <- function(
    ngrams,
    removal_mode = c("frequency", "percentage", "term"),
    removal_rate_most = NULL,
    removal_rate_least = NULL
) {
  removal_mode <- match.arg(removal_mode)
  
  # Work per n-gram type (so 1-grams and 3-grams are filtered comparably)
  split_list <- split(ngrams, ngrams$n_gram_type)
  before <- vapply(split_list, nrow, integer(1))
  
  filtered_list <- lapply(split_list, function(df) {
    if (nrow(df) == 0) return(df)
    
    # sort by frequency (high -> low)
    df <- dplyr::arrange(df, dplyr::desc(freq))
    n <- nrow(df)
    
    if (removal_mode == "percentage") {
      k_most  <- if (!is.null(removal_rate_most)  && removal_rate_most  > 0) ceiling(n * removal_rate_most)  else 0L
      k_least <- if (!is.null(removal_rate_least) && removal_rate_least > 0) ceiling(n * removal_rate_least) else 0L
      
      keep <- rep(TRUE, n)
      
      # drop most frequent (top rows)
      if (k_most > 0) {
        k_most <- min(k_most, n)
        keep[seq_len(k_most)] <- FALSE
      }
      
      # drop least frequent (bottom rows)
      if (k_least > 0) {
        k_least <- min(k_least, sum(keep))
        idx <- which(keep)
        drop_idx <- utils::tail(idx, k_least)
        keep[drop_idx] <- FALSE
      }
      
      df <- df[keep, , drop = FALSE]
      
    } else if (removal_mode == "term") {
      k_most  <- if (!is.null(removal_rate_most)  && removal_rate_most  > 0) as.integer(removal_rate_most)  else 0L
      k_least <- if (!is.null(removal_rate_least) && removal_rate_least > 0) as.integer(removal_rate_least) else 0L
      
      keep <- rep(TRUE, n)
      
      if (k_most > 0) {
        k_most <- min(k_most, n)
        keep[seq_len(k_most)] <- FALSE
      }
      
      if (k_least > 0) {
        k_least <- min(k_least, sum(keep))
        idx <- which(keep)
        drop_idx <- utils::tail(idx, k_least)
        keep[drop_idx] <- FALSE
      }
      
      df <- df[keep, , drop = FALSE]
      
    } else if (removal_mode == "frequency") {
      # Here rates are thresholds, not percentages
      if (!is.null(removal_rate_least)) {
        df <- df[df$freq > removal_rate_least, , drop = FALSE]
      }
      if (!is.null(removal_rate_most)) {
        df <- df[df$freq < removal_rate_most, , drop = FALSE]
      }
    }
    
    df
  })
  
  ngrams_filtered <- dplyr::bind_rows(filtered_list)
  after <- table(ngrams_filtered$n_gram_type)
  
  # build stats
  ngram_types <- sort(unique(ngrams$n_gram_type))
  after_vec <- as.integer(after[as.character(ngram_types)])
  after_vec[is.na(after_vec)] <- 0L
  before_vec <- as.integer(before[as.character(ngram_types)])
  before_vec[is.na(before_vec)] <- 0L
  
  stats <- tibble::tibble(
    ngram_type = as.integer(ngram_types),
    removal_mode_removed = as.integer(before_vec - after_vec)
  )
  
  list(
    ngrams_filtered = ngrams_filtered,
    stats = stats
  )
}
# #filter_ngrams <- function(
# #    ngrams, 
# #    removal_mode, 
# #    removal_rate_most = NULL, 
# #    removal_rate_least = NULL) {
# #  if (removal_mode == "term") {
# #    
# #    ngrams_filtered <- ngrams %>%
# #      dplyr::arrange(desc(freq)) 
# #    # 1. Remove top n and bottom m terms based on frequency
# #    if (!is.null(removal_rate_most)){
# #      ngrams_filtered <- ngrams_filtered[-(1:removal_rate_most), ]
# #    }
# #    if (!is.null(removal_rate_least)){
# #      ngrams_filtered <- ngrams_filtered[-((nrow(ngrams_filtered) - removal_rate_least + 1):nrow(ngrams_filtered)), ]
# #    }
# #  } else if (removal_mode == "frequency") {
# #    # 2. Remove words with frequency over n and under m
# #    if (is.null(removal_rate_most)){
# #      removal_rate_most <- max(ngrams$freq)
# #    }
# #    if (is.null(removal_rate_least)){
# #      removal_rate_least <- 0
# #    }
# #    ngrams_filtered <- ngrams %>%
# #        dplyr::filter(freq <= removal_rate_most & freq >= removal_rate_least)
# #    
# #    
# #  } else if (removal_mode == "percentage") {
# #    # 3. Remove terms that occur in more than n% and less than m% of the time
# #    if (is.null(removal_rate_most)){
# #      removal_rate_most <- 0
# #    } 
# #    if (is.null(removal_rate_least)){
# #      removal_rate_least <- 1
# #    } 
# #    removal_rate_most <- nrow(ngrams) * removal_rate_most
# #    removal_rate_least <- nrow(ngrams) - nrow(ngrams) * removal_rate_least
# #    # take all rows from removal_rate_least to removal_rate_most
# #    ngrams_filtered <- ngrams[removal_rate_least:removal_rate_most, ]
# #
# #    #ngrams_filtered <- ngrams %>%
# #    #    dplyr::filter(prevalence <= removal_rate_most & prevalence >= removal_rate_least)
# #
# #    
# #  } else {
# #    stop("Invalid mode. Choose 'term', 'frequency', or 'prevalence'.")
# #  }
# #  # Compute the stats
# #  initial_count = table(ngrams$n_gram_type)
# #  final_count = table(ngrams_filtered$n_gram_type)
# #  
# #  stats <-  tibble::tibble(
# #    ngram_type = unique(ngrams$n_gram_type),
# #    removal_mode_removed = initial_count - final_count
# #  )
# #  
# #  return(list(
# #    ngrams_filtered = ngrams_filtered, 
# #    stats = stats))
# #}



# Function to remove stopwords from a single row of text
#' @param text The data to be be cleaned from stopwords
#' @param stopwords (character) Words to be removed
#' @importFrom dplyr pull select filter mutate anti_join summarise pull group_by group_modify ungroup
#' @noRd
remove_stopwords <- function(
    text, 
    stopwords) {
  
  words <- unlist(strsplit(text, "\\s+"))                 # Split the text into words
  cleaned_words <- words[!tolower(words) %in% stopwords] # Remove stopwords (case insensitive)
  cleaned_text <- paste(cleaned_words, collapse = " ")   # Reconstruct the cleaned text
  cleaned_text <- stringr::str_squish(cleaned_text) # Removes leading and trailing whitespace; ensures one space between words
  
  return(cleaned_text)
}

#' @param ngram_tibble: A tibble with n_grams and freq columns representing n-grams and their frequencies.
#' @param unigram_tibble: A tibble with n_grams and freq columns representing unigrams and their frequencies.
#' @param pmi_threshold: The PMI threshold for filtering n-grams (default is 0).
#' @importFrom dplyr mutate if_else filter 
#' @importFrom tibble as_tibble
#' @importFrom stringr str_detect
#' @importFrom purrr map_dbl
#' @noRd
filter_ngrams_by_pmi <- function(ngram_tibble, unigram_tibble, pmi_threshold) {
  if(nrow(ngram_tibble) == 0) return(list(filtered_ngrams=ngram_tibble, stats=NULL))
  
  # 1. Use the same "Total N" for both to keep probabilities on the same scale
  # Usually, this is the total number of tokens (unigrams) in the corpus
  total_tokens <- sum(unigram_tibble$freq)
  
  # 2. Probability lookup for individual words
  uni_lookup <- setNames(unigram_tibble$freq / total_tokens, unigram_tibble$ngrams)
  
  # 3. Probability of the n-gram occurring
  # NOTE: Use total_tokens here as well to normalize
  ngram_tibble$joint_prob <- ngram_tibble$freq / total_tokens
  
  # 4. Calculate the product of individual probabilities
  ngram_tibble$comp_prob <- sapply(strsplit(as.character(ngram_tibble$ngrams), "\\s+"), function(w) {
    p <- uni_lookup[w]
    p[is.na(p)] <- 1e-10 # Floor for missing words
    prod(p)
  })
  
  # 5. The Formula: log2( P(xy) / (P(x)P(y)) )
  ngram_tibble$pmi <- log2(ngram_tibble$joint_prob / pmax(ngram_tibble$comp_prob, 1e-10))
  
  # 6. FIX: Handle Unigrams
  # We should probably set them to NA or a value that indicates "Not Applicable"
  # but ensures they survive the filter.
  is_uni <- ngram_tibble$n_gram_type == 1
  ngram_tibble$pmi[is_uni] <- Inf # Unigrams always pass
  
  # 7. Apply the filter
  filtered <- if(!is.null(pmi_threshold)) {
    ngram_tibble[ngram_tibble$pmi >= pmi_threshold, ]
  } else {
    ngram_tibble
  }
  
  # Clean up Inf for the final display
  filtered$pmi[is.infinite(filtered$pmi)] <- NA
  
  # 8. Stats
  types <- sort(unique(ngram_tibble$n_gram_type))
  stats <- tibble::tibble(
    ngram_type = types,
    pmi_removed = as.numeric(unname(table(factor(ngram_tibble$n_gram_type, levels=types)) - 
                                      table(factor(filtered$n_gram_type, levels=types))))
  )
  
  return(list(filtered_ngrams = filtered[, !names(filtered) %in% c("joint_prob", "comp_prob")], stats = stats))
}


#' N-grams
#'
#' Compute n-grams and per-document relative frequencies from text.
#'
#' This function extracts n-grams using  and returns:
#' (1) a tibble of n-grams with frequency/prevalence and document frequency,
#' (2) per-document relative frequencies for the retained n-grams (wide or long format),
#' (3) filtering statistics,
#' (4) a named list of settings (for reproducibility).
#'
#' @param data A character vector of texts or a data.frame/tibble (first column used as text).
#' @param ngram_window Integer vector specifying n-gram sizes.
#'   If length 1, only that n is used (e.g., \code{2}).
#'   If length 2 and increasing, it is interpreted as an inclusive range (e.g., \code{c(1, 3)} -> 1,2,3).
#'   Otherwise it is treated as an explicit set (e.g., \code{c(1, 3)} means only 1 and 3).
#' @param stopwords Character vector of stopwords to remove (e.g.,
#'   \code{stopwords::stopwords("en", source = "snowball")}).
#' @param removalword Character vector of words to remove from the raw text prior to tokenization.
#' @param pmi_threshold Numeric PMI threshold used to filter multi-word n-grams.
#'   If \code{NULL}, PMI filtering is skipped.
#' @param occurance_rate Numeric in [0,1]. Terms are removed if they occur in fewer than
#'   \code{round(N_docs * occurance_rate) - 1} documents, where \code{N_docs} is the number of texts.
#'   Example: with 1000 documents and \code{occurance_rate = 0.05}, terms occurring in fewer than ~50 documents are removed.
#' @param removal_mode Character. Removal mode passed to \code{filter_ngrams()}.
#'   Common options include \code{"frequency"} and \code{"percentage"} (depending on your helper implementation).
#' @param removal_rate_most Numeric. Rate/threshold for removing the most frequent n-grams
#'   (interpreted by \code{filter_ngrams()} according to \code{removal_mode}).
#' @param removal_rate_least Numeric. Rate/threshold for removing the least frequent n-grams
#'   (interpreted by \code{filter_ngrams()} according to \code{removal_mode}).
#' @param shuffle Logical. If \code{TRUE}, texts are shuffled (seed-controlled) before processing.
#'   The original ids are preserved in output.
#' @param lower Logical. If \code{TRUE}, text is lowercased prior to tokenization.
#' @param remove_punctuation Logical. Passed to \code{quanteda::tokens(remove_punct = ...)}.
#' @param remove_numbers Logical. Passed to \code{quanteda::tokens(remove_numbers = ...)}.
#' @param stem_lemma_function Optional function applied to tokens (via \code{quanteda::tokens_apply()}).
#'   Should accept and return a character vector of tokens.
#' @param verbose Logical. If \code{TRUE}, prints simple progress messages (if implemented).
#' @param seed Integer. Random seed used when \code{shuffle = TRUE}.
#' @param threads Integer. Number of threads used by \pkg{quanteda} via \code{quanteda_options(threads = ...)}.
#' @param top_frequent Integer or \code{NULL}. If set, keeps only the \code{top_frequent} most frequent n-grams
#'   after filtering (recommended for large corpora). If \code{NULL}, keeps all retained n-grams.
#' @param freq_per_user_format Output format for \code{freq_per_user}:
#'   \describe{
#'     \item{\code{"wide"}}{Wide document-by-ngram table (may be large).}
#'     \item{\code{"long"}}{Long sparse table with columns \code{id}, \code{ngram}, \code{rel_freq}.}
#'     \item{\code{"auto"}}{Uses wide format only if \code{N_docs * N_ngrams <= max_wide_cells}; otherwise returns long.}
#'   }
#' @param max_wide_cells Numeric. Safety limit for \code{"auto"}: if \code{N_docs * N_ngrams} exceeds this,
#'   \code{freq_per_user} is returned in long format to avoid dense allocation and excessive memory use.
#'
#' @return A named list with:
#' \describe{
#'   \item{settings}{A named list of settings used to generate the results (for reproducibility).}
#'   \item{n_grams_pmi}{PMI information if available from your PMI helper, otherwise a message.}
#'   \item{ngrams}{A tibble of retained n-grams and statistics (e.g., \code{freq}, \code{prevalence}, \code{num_docs}).}
#'   \item{freq_per_user}{Per-document relative frequencies, either wide or long depending on \code{freq_per_user_format}.}
#'   \item{stats}{A tibble summarizing how many n-grams were removed by each filtering step, by n-gram size.}
#' }
#' @importFrom quanteda tokens tokens_remove tokens_ngrams dfm featnames dfm_select quanteda_options
#' @importFrom Matrix colSums
#' @importFrom tibble tibble as_tibble
#' @importFrom dplyr arrange bind_cols filter select slice_head
#' @importFrom stats complete.cases
#' @export
topicsGrams <- function(
    data,
    ngram_window = c(1, 3),
    stopwords = stopwords::stopwords("en", source = "snowball"),
    removalword = "",
    pmi_threshold = NULL,
    occurance_rate = 0,
    removal_mode = "frequency",
    removal_rate_most = NULL,
    removal_rate_least = NULL,
    shuffle = TRUE,
    lower = TRUE,
    remove_punctuation = TRUE,
    remove_numbers = TRUE,
    stem_lemma_function = NULL,
    verbose = FALSE,
    seed = 42L,
    threads = 1,
    top_frequent = NULL,
    freq_per_user_format = c("auto", "wide", "long"),
    max_wide_cells = 5e7
    ) {
      
      freq_per_user_format <- match.arg(freq_per_user_format)
      
      # ---- harmonize ngram_window behavior (like topicsDtm) ----
      # - if length 1 -> same min/max
      # - if length 2 and increasing -> inclusive range
      # - else treat as explicit set
      if (length(ngram_window) == 1) {
        ngram_sizes <- ngram_window
      } else if (length(ngram_window) == 2 && isTRUE(ngram_window[2] >= ngram_window[1])) {
        ngram_sizes <- seq(ngram_window[1], ngram_window[2])
      } else {
        ngram_sizes <- sort(unique(ngram_window))
      }
      ngram_sizes <- as.integer(ngram_sizes)
      if (any(ngram_sizes < 1)) stop("ngram_window must contain values >= 1")
      
      # ---- settings ----
      settings <- list(
        ngram_window = ngram_window,
        ngram_sizes = ngram_sizes,
        stopwords = stopwords,
        removalword = removalword,
        pmi_threshold = pmi_threshold,
        occurance_rate = occurance_rate,
        removal_mode = removal_mode,
        removal_rate_most = removal_rate_most,
        removal_rate_least = removal_rate_least,
        shuffle = shuffle,
        lower = lower,
        remove_punctuation = remove_punctuation,
        remove_numbers = remove_numbers,
        stem_lemma_function = stem_lemma_function,
        verbose = verbose,
        seed = seed,
        threads = threads,
        top_frequent = top_frequent,
        freq_per_user_format = freq_per_user_format,
        max_wide_cells = max_wide_cells
      )
      
      # ---- input handling ----
      if (length(data) == 0) {
        if (verbose) message("The data provided is empty.")
        return(NULL)
      }
      if (is.data.frame(data)) data <- data[, 1]
      
      set.seed(seed)
      
      text_cols <- tibble::tibble(
        id = seq_along(data),
        text = as.character(data)
      )
      text_cols <- text_cols[stats::complete.cases(text_cols), , drop = FALSE]
      text_cols <- text_cols[text_cols$text != "", , drop = FALSE]
      
      if (nrow(text_cols) == 0) {
        return(list(
          settings = settings,
          ngrams = tibble::tibble(),
          freq_per_user = tibble::tibble(),
          stats = tibble::tibble(),
          n_grams_pmi = "No texts after cleaning."
        ))
      }
      
      # shuffle like topicsDtm (but keep id so you can map back)
      if (shuffle) {
        text_cols <- text_cols[sample.int(nrow(text_cols)), , drop = FALSE]
      }
      
      # removalword (same idea as topicsDtm)
      if (length(removalword) > 0 && any(removalword != "")) {
        rw <- removalword
        if (lower) rw <- stringi::stri_trans_tolower(rw)
        pattern <- paste0("\\b(", paste(rw, collapse = "|"), ")\\b")
        text_cols$text <- gsub(pattern, "", text_cols$text, perl = TRUE)
      }
      
      #  lowercase (optional)
      if (lower) {
        text_cols$text <- stringi::stri_trans_tolower(text_cols$text)
      }
      
      # quanteda threads (restore on exit) 
      old_threads <- tryCatch(quanteda::quanteda_options("threads"), error = function(e) NULL)
      quanteda::quanteda_options(threads = threads)
      on.exit({
        if (!is.null(old_threads)) {
          quanteda::quanteda_options(threads = old_threads)
        }
      }, add = TRUE)
      
      # tokenize once 
      toks <- quanteda::tokens(
        text_cols$text,
        what = "word",
        remove_punct = remove_punctuation,
        remove_numbers = remove_numbers,
        remove_symbols = TRUE,
        remove_separators = TRUE
      )
      
      if (length(stopwords) > 0) {
        toks <- quanteda::tokens_remove(toks, pattern = stopwords)
      }
      
      # optional stemming/lemmatization hook (user-provided function)
#      # expected: function(character_vector) -> character_vector
#      if (!is.null(stem_lemma_function)) {
#        toks <- quanteda::tokens_apply(toks, FUN = stem_lemma_function)
#      }
#      
      # build document–feature matrix (dfm) for each requested n and bind columns (base::cbind works) ----
      dfm_list <- vector("list", length(ngram_sizes))
      ngrams_list <- vector("list", length(ngram_sizes))
      
      for (i in seq_along(ngram_sizes)) {
        n <- ngram_sizes[i]
        toks_n <- quanteda::tokens_ngrams(toks, n = n, concatenator = "_")
        dfm_n <- quanteda::dfm(toks_n)
        
        # feature frequencies (sparse-safe)
        freq <- Matrix::colSums(dfm_n)
        
        ngrams_list[[i]] <- tibble::tibble(
          ngrams = names(freq),                # underscore form (matches your end format)
          freq = as.numeric(freq),
          n_gram_type = n
        )
        dfm_list[[i]] <- dfm_n
      }
      
      ngrams <- dplyr::bind_rows(ngrams_list)
      if (nrow(ngrams) == 0) {
        return(list(
          settings = settings,
          ngrams = tibble::tibble(),
          freq_per_user = tibble::tibble(),
          stats = tibble::tibble(),
          n_grams_pmi = if (!is.null(pmi_threshold)) tibble::tibble() else "No pmi_threshold was used"
        ))
      }
      
      # combined dfm across n
      dfm_all <- do.call(base::cbind, dfm_list)
      
      # prevalence + placeholder coherence
      total <- sum(ngrams$freq)
      ngrams$prevalence <- ngrams$freq / total
      ngrams$coherence <- NA_real_
      
      ngram_types_seen <- ngrams$n_gram_type
      initial_count <- table(ngrams$n_gram_type)
      
      # filter_ngrams 
      tmp <- filter_ngrams(
        ngrams = ngrams,
        removal_mode = removal_mode,
        removal_rate_most = removal_rate_most,
        removal_rate_least = removal_rate_least
      )
      filter_stats <- tmp$stats
      ngrams_filtered <- tmp$ngrams_filtered
      
      # PMI filtering (your helper uses spaces; convert just for PMI step) ----
      pmi_tibble <- NULL
      
      if (!is.null(pmi_threshold)) {
        
        # unigrams for PMI
        toks_1 <- quanteda::tokens_ngrams(toks, n = 1, concatenator = "_")
        dfm_1 <- quanteda::dfm(toks_1)
        freq_1 <- Matrix::colSums(dfm_1)
        
        unigram_tibble <- tibble::tibble(
          ngrams = gsub("_", " ", names(freq_1), fixed = TRUE),
          freq = as.numeric(freq_1),
          prevalence = as.numeric(freq_1) / sum(freq_1)
        )
        
        # convert candidate ngrams to spaces
        ngrams_for_pmi <- ngrams_filtered
        ngrams_for_pmi$ngrams <- gsub("_", " ", ngrams_for_pmi$ngrams, fixed = TRUE)
        
        tmp2 <- filter_ngrams_by_pmi(
          ngram_tibble = ngrams_for_pmi,
          unigram_tibble = unigram_tibble,
          pmi_threshold = pmi_threshold
        )
        
        pmi_stats <- tmp2$stats
        ngrams_filtered <- tmp2$filtered_ngrams
        
        # back to underscore form
        if (nrow(ngrams_filtered) > 0) {
          ngrams_filtered$ngrams <- gsub(" ", "_", ngrams_filtered$ngrams, fixed = TRUE)
        }
        
        # if your helper returns extra PMI info, keep it if present
        if (!is.null(tmp2$pmi_tibble)) pmi_tibble <- tmp2$pmi_tibble
        
      } else {
        # create a compatible stats skeleton if PMI not used
        pmi_stats <- tibble::tibble(
          ngram_type = sort(unique(ngram_types_seen)),
          pmi_removed = 0L
        )
      }
      
      # nothing left after PMI
      if (nrow(ngrams_filtered) == 0) {
        stats <- merge(pmi_stats, filter_stats, by = "ngram_type", all = TRUE)
        stats$initial_count <- as.integer(initial_count[match(stats$ngram_type, names(initial_count))])
        stats$removal_mode_removed[is.na(stats$removal_mode_removed)] <- 0L
        stats$pmi_removed[is.na(stats$pmi_removed)] <- 0L
        stats$occurance_removed <- 0L
        stats$final_count <- 0L
        stats <- stats[, c("ngram_type","initial_count","pmi_removed","removal_mode_removed","occurance_removed","final_count")]
        
        return(list(
          settings = settings,
          ngrams = tibble::tibble(),
          freq_per_user = tibble::tibble(),
          stats = stats,
          n_grams_pmi = if (!is.null(pmi_threshold)) pmi_tibble else "No pmi_threshold was used"
        ))
      }
      
      # Fast doc frequency + occurrence filter
      present <- ngrams_filtered$ngrams %in% quanteda::featnames(dfm_all)
      ngrams_filtered <- ngrams_filtered[present, , drop = FALSE]
      
      dfm_filtered <- quanteda::dfm_select(dfm_all, pattern = ngrams_filtered$ngrams)
      M <- methods::as(dfm_filtered, "dgCMatrix")
      
      # document frequency
      num_docs <- Matrix::colSums(M > 0)
      ngrams_filtered$num_docs <- as.integer(num_docs[match(ngrams_filtered$ngrams, names(num_docs))])
      
      occurance_before <- table(ngrams_filtered$n_gram_type)
      occurance_before <- add_missing_rows(occurance_before, ngram_sizes)
      
      if (occurance_rate > 0) {
        removal_frequency <- round(nrow(text_cols) * occurance_rate) - 1
        ngrams_filtered <- ngrams_filtered %>% dplyr::filter(num_docs >= removal_frequency)
      }
      
      occurance_after <- table(ngrams_filtered$n_gram_type)
      occurance_after <- add_missing_rows(occurance_after, ngram_sizes)
      
      occurance_stats <- tibble::tibble(
        ngram_type = sort(unique(ngram_types_seen)),
        occurance_removed = as.integer(occurance_before - occurance_after)
      )
      
      # final stats table
      final_count <- initial_count -
        occurance_stats$occurance_removed -
        filter_stats$removal_mode_removed -
        pmi_stats$pmi_removed
      
      stats <- merge(pmi_stats, filter_stats, by = "ngram_type", all = TRUE)
      stats <- merge(stats, occurance_stats, by = "ngram_type", all = TRUE)
      
      stats$initial_count <- as.integer(initial_count[match(stats$ngram_type, names(initial_count))])
      stats$pmi_removed[is.na(stats$pmi_removed)] <- 0L
      stats$removal_mode_removed[is.na(stats$removal_mode_removed)] <- 0L
      stats$occurance_removed[is.na(stats$occurance_removed)] <- 0L
      stats$final_count <- as.integer(final_count)
      
      stats <- stats[, c("ngram_type","initial_count","pmi_removed","removal_mode_removed","occurance_removed","final_count")]
      
      # sort, optional top_frequent BEFORE freq_per_user to control size
      ngrams_filtered <- ngrams_filtered %>% dplyr::arrange(dplyr::desc(freq))
      if (!is.null(top_frequent)) {
        ngrams_filtered <- ngrams_filtered %>% dplyr::slice_head(n = top_frequent)
      }
      
      # recompute prevalence after filtering
      total2 <- sum(ngrams_filtered$freq)
      ngrams_filtered$prevalence <- ngrams_filtered$freq / total2
      
      # freq_per_user (sparse-safe) 
      keep_terms <- ngrams_filtered$ngrams
      keep_idx <- match(keep_terms, colnames(M))
      keep_idx <- keep_idx[!is.na(keep_idx)]
      M2 <- M[, keep_idx, drop = FALSE]
      
      row_tot <- Matrix::rowSums(M2)
      row_tot[row_tot == 0] <- 1
      relM <- M2 / row_tot
      
      n_docs <- nrow(relM)
      n_terms <- ncol(relM)
      would_be_cells <- as.double(n_docs) * as.double(n_terms)
      
      # long sparse (always safe) with original ids
      trip <- Matrix::summary(relM)
      freq_long <- tibble::tibble(
        id = text_cols$id[trip$i],
        ngram = colnames(relM)[trip$j],
        rel_freq = trip$x
      )
      
      if (freq_per_user_format == "long") {
        freq_out <- freq_long
      } else {
        use_wide <- (freq_per_user_format == "wide") ||
          (freq_per_user_format == "auto" && would_be_cells <= max_wide_cells)
        
        if (!use_wide) {
          freq_out <- freq_long
          attr(freq_out, "note") <- paste0(
            "Returned long format to avoid dense allocation: docs=", n_docs,
            ", ngrams=", n_terms, ", docs*ngrams=", format(would_be_cells, scientific = FALSE),
            " > max_wide_cells=", format(max_wide_cells, scientific = FALSE),
            ". Set top_frequent or freq_per_user_format='wide' if you really want wide."
          )
        } else {
          wide_mat <- as.matrix(relM)
          
          ord <- order(text_cols$id)
          wide_mat <- wide_mat[ord, , drop = FALSE]
          
          freq_out <- tibble::as_tibble(wide_mat, .name_repair = "minimal")
          colnames(freq_out) <- colnames(relM)
          
        }
      }
      
      # match your original end behavior: drop n_gram_type in the ngrams output
      ngrams_filtered <- ngrams_filtered %>% dplyr::select(-n_gram_type)
      
      list(
        settings = settings,
        ngrams = tibble::as_tibble(ngrams_filtered),
        n_grams_pmi = if (!is.null(pmi_threshold)) pmi_tibble else "No pmi_threshold was used",
        freq_per_user = freq_out,
        stats = stats
      )
}

library(testthat)
library(topics)
library(tibble)

#test_that("topicsGrams with default parameters", {
#
#  testthat::skip_on_cran()
#  
#  ngrams <- topics::topicsGrams(
#    data = dep_wor_data$Worphrase, 
#    ngram_window = c(1,3),
#    stopwords = stopwords::stopwords("en", source = "snowball"),
#    pmi_threshold = 0, 
#    top_frequent = NULL)
#  
#  testthat::expect_true(is.list(ngrams))
#  testthat::expect_true(is_tibble(ngrams$ngrams))
#  testthat::expect_true(is_tibble(ngrams$freq_per_user))
#  
#  testthat::expect_equal(unname(ngrams$ngrams$ngrams[1:5]),
#    unname(c("anxious", "worried", "scared", "nervous", "concerned")))
#  
#  testthat::expect_equal(ngrams$ngrams$freq[1:5], c(174, 125, 63, 57, 50))
#  testthat::expect_equal(ngrams$ngrams$prop[1:5], c(0.04763208, 0.03421845, 0.01724610, 0.01560361, 0.01368738), tolerance = 0.0001)
#  testthat::expect_equal(ngrams$ngrams$prevalence[1:5], c(0.027588394, 0.019819248, 0.009988901, 0.009037577, 0.007927699), tolerance = 0.0001)
#  testthat::expect_equal(ngrams$ngrams$pmi[3000:3005], c(12.916076, 17.390007, 10.709120, 20.596543,  7.938796, 17.048970), tolerance = 0.0001)
#  testthat::expect_equal(ngrams$freq_per_user$worried[1:5], c(0.000, 0.000, 0.000, 0.000, 0.008), tolerance = 0.0001)
#  testthat::expect_equal(ngrams$freq_per_user$concerned[1:5], c(0.00, 0.00, 0.00, 0.02, 0.00), tolerance = 0.0001)
#  
#  testthat::expect_equal(ngrams$stats$initial_count[[1]], 1225)
#  testthat::expect_equal(ngrams$stats$initial_count[[2]], 2636)
#  testthat::expect_equal(ngrams$stats$pmi_removed[[1]], 0)
#  testthat::expect_equal(ngrams$stats$final_count[[1]], 1225)
#  
#})

test_that("topicsGrams with default parameters", {
  
  testthat::skip_on_cran()
  
  ngrams <- topics::topicsGrams(
    data = dep_wor_data$Worphrase, 
    ngram_window = c(1,3),
    stopwords = NULL,
    pmi_threshold = NULL)
  
  testthat::expect_true(is.list(ngrams))
  testthat::expect_true(is_tibble(ngrams$ngrams))
  testthat::expect_true(is_tibble(ngrams$freq_per_user))
})



test_that("topicsGrams with selection parameters", {
  
  testthat::skip_on_cran()
  
  ngrams <- topics::topicsGrams(
    data = dep_wor_data$Worphrase[1:100], 
    ngram_window = c(1,2),
    stopwords::stopwords("en", source = "snowball"),
    pmi_threshold = 1, 
    top_frequent = 100
    )
  
  testthat::expect_true(is.list(ngrams))
  testthat::expect_true(is_tibble(ngrams$ngrams))
  testthat::expect_true(is_tibble(ngrams$freq_per_user))
})

test_that("topicsGrams with occurance rate", {
  
  testthat::skip_on_cran()
  
  ngrams <- topics::topicsGrams(
    data = dep_wor_data$Worphrase[1:100], 
    ngram_window = c(1,3),
    stopwords = NULL,
    pmi_threshold = NULL,
    occurance_rate = 0.05)
  
  testthat::expect_true(is.list(ngrams))
  testthat::expect_true(is_tibble(ngrams$ngrams))
  testthat::expect_true(is_tibble(ngrams$freq_per_user))
})

test_that("topicsGrams with removal mode: term", {
  
  testthat::skip_on_cran()
  
  ngrams <- topics::topicsGrams(
    data = dep_wor_data$Worphrase[1:100], 
    ngram_window = c(1,3),
    stopwords = NULL,
    pmi_threshold = NULL,
    removal_mode = "term",
    removal_rate_most = 1,
    removal_rate_least = 1)
  
  testthat::expect_true(is.list(ngrams))
  testthat::expect_true(is_tibble(ngrams$ngrams))
  testthat::expect_true(is_tibble(ngrams$freq_per_user))
})

test_that("topicsGrams with removal mode: frequency", {
  
  testthat::skip_on_cran()
  
  ngrams <- topics::topicsGrams(
    data = dep_wor_data$Worphrase[1:100], 
    ngram_window = c(1,3),
    stopwords = NULL,
    pmi_threshold = NULL,
    removal_mode = "frequency",
    removal_rate_most = 30,
    removal_rate_least = 1)
  
  testthat::expect_true(is.list(ngrams))
  testthat::expect_true(is_tibble(ngrams$ngrams))
  testthat::expect_true(is_tibble(ngrams$freq_per_user))
})

test_that("topicsGrams with removal mode: percentage", {
  
  testthat::skip_on_cran()
  
  ngrams <- topics::topicsGrams(
    data = dep_wor_data$Worphrase[1:100], 
    ngram_window = c(1,3),
    stopwords = NULL,
    pmi_threshold = NULL,
    removal_mode = "percentage",
    removal_rate_most = 0.01,
    removal_rate_least = 0.01)
  
  testthat::expect_true(is.list(ngrams))
  testthat::expect_true(is_tibble(ngrams$ngrams))
  testthat::expect_true(is_tibble(ngrams$freq_per_user))
})

#' check_java_available
#'
#' Checks whether the specified package (typically 'rJava') is installed and, 
#' if applicable, whether Java is correctly configured. 
#' Used to safely guard functions that rely on Java dependencies.
#'
#' @param pkg (string) The name of the package to check, default is "rJava".
#' @param func_name (string) Optional. The name of the calling function, 
#'        used to improve error messaging.
#' @return (invisible) TRUE if the package and Java (if applicable) are available.
#' @noRd
check_java_available <- function(pkg = "rJava", func_name = NULL) {
  
  if (!requireNamespace(pkg, quietly = TRUE)) {
    msg <- paste0("This function requires the '", pkg, "' package, which is not installed.",
                  "\nPlease install it using: install.packages('", pkg, "')")
    if (!is.null(func_name)) {
      msg <- paste0(msg, "\n\nWhile running: ", func_name)
    }
    message(colourise(msg, "brown"))
    return(FALSE)
  }
  
  # Dynamically call .jinit() only if rJava is available
  if (pkg == "rJava") {
    ok <- tryCatch(
      {
        getNamespace("rJava")[[".jinit"]]()  # avoids hard dependency
        TRUE
      },
      error = function(e) {
        msg <- paste0("Java is not properly configured. Please ensure Java is installed and working.",
                      "\nError from rJava:::.jinit(): ", conditionMessage(e))
        message(colourise(msg, "brown"))
        FALSE
      }
    )
    if (!ok) return(FALSE)
  }
  
  TRUE
}


#' Internal helper to safely access rJava functions dynamically
#' @param name The name of the rJava function (e.g. ".jnew", "J")
#' @return The corresponding function from rJava, or NULL if unavailable
#' @noRd
.j <- function(name) {
  getNamespace("rJava")[[name]]
}

#' inferencer
#' @param model (tibble) A model
#' @return inferencer
#' @noRd
inferencer <- function(model) {
  model$model$getInferencer()
}


#' write_inferencer
#' @param inf  a reference to a topic inferencer, from inferencer()
#' @param out_file (string)  the name of a file to save to (will overwrite an existing file)
#' @return save an inferencer object to a file
#' @noRd
write_inferencer <- function(inf, out_file) {
  if (!check_java_available(pkg = "rJava", func_name = "write_inferencer")) return(NULL)
  
  .jnew <- .j(".jnew")
  .jcast <- .j(".jcast")
  
  fos <- .jnew("java/io/FileOutputStream", out_file)
  oos <- .jnew("java/io/ObjectOutputStream", .jcast(fos, "java/io/OutputStream"))
  
  oos$writeObject(inf)
  oos$close()
}


#' retrieve an inferencer object from a file
#' @param in_file (string) path to inferencer file
#' @return returns a reference to a topic inferencer object
#' @noRd
read_inferencer <- function(in_file) {
  if (!check_java_available(pkg = "rJava", func_name = "read_inferencer")) return(NULL)
  
  J <- .j("J")
  new_obj <- methods::new(J("java.io.File"), in_file)
  J("cc.mallet.topics.TopicInferencer")$read(new_obj)
}


#' Infer document topics using an inferencer and compatible instances
#' @param inferencer a topic inferencer object
#' @param instances an instances list object from compatible_instances()
#' @param n_iterations number of Gibbs sampling iterations
#' @param sampling_interval thinning interval
#' @param burn_in number of burn-in iterations
#' @param random_seed integer random seed; set for reproducibility
#' @return matrix of estimated document-topic proportions
#' @noRd
infer_topics <- function(
    inferencer,
    instances,
    n_iterations = 100,
    sampling_interval = 10,
    burn_in = 10,
    random_seed = NULL) {
  
  if (!check_java_available(pkg = "rJava", func_name = "infer_topics")) return(NULL)
  
  .jcall <- .j(".jcall")
  
  iter <- instances$iterator()
  n_iterations <- as.integer(n_iterations)
  sampling_interval <- as.integer(sampling_interval)
  burn_in <- as.integer(burn_in)
  
  if (!is.null(random_seed)) {
    inferencer$setRandomSeed(as.integer(random_seed))
  }
  
  doc_topics <- vector("list", instances$size())
  
  for (j in 1:instances$size()) {
    inst <- .jcall(iter, "Ljava/lang/Object;", "next")
    doc_topics[[j]] <- inferencer$getSampledDistribution(
      inst,
      n_iterations,
      sampling_interval,
      burn_in
    )
  }
  
  do.call(rbind, doc_topics)
}


#' Create a compatible instance list for inference
#' @param ids character vector of item ids
#' @param texts character vector of texts
#' @param instances reference to a training instance list
#' @return a new compatible instance list
#' @noRd
compatible_instances <- function(ids, texts, instances) {
  if (!check_java_available(pkg = "rJava", func_name = "compatible_instances")) return(NULL)
  
  .jnew <- .j(".jnew")
  .jcast <- .j(".jcast")
  .jarray <- .j(".jarray")
  J <- .j("J")
  
  mallet_pipe <- instances$getPipe()
  
  new_insts <- .jnew("cc/mallet/types/InstanceList",
                     .jcast(mallet_pipe, "cc/mallet/pipe/Pipe"))
  
  java_ids <- .jarray(ids, "java/lang/String")
  java_texts <- .jarray(texts, "java/lang/String")
  
  J("cc.mallet.topics.RTopicModel")$addInstances(new_insts, ids, texts)
  
  new_insts
}


#' Number of tokens in each document
#' @param instances reference to an instance list
#' @return vector of token counts
#' @noRd
instances_lengths <- function(instances) {
  if (!check_java_available(pkg = "rJava", func_name = "instances_lengths")) return(NULL)
  
  .jcall <- .j(".jcall")
  iter <- instances$iterator()
  
  replicate(instances$size(),
            .jcall(iter, "Ljava/lang/Object;", "next")$getData()$size())
}

#' Download and Prepare Tutorial Data
#' 
#' Downloads the Big5 Essays dataset from Hugging Face, filters by word count,
#' and returns a sample of a specified size.
#' @param sample_size (integer) The number of rows to return. Default is 1000.
#' @param min_word_count (integer) Minimum words per essay. Default is 250.
#' @param max_word_count (integer) Maximum words per essay. Default is 750.
#' @param seed (integer) Seed for reproducibility. Default is 42.
#' @param verbose (boolean) Whether to print status messages. Default is TRUE.
#' @return A processed tibble ready for topic modeling.
# @importFrom arrow read_parquet
# @importFrom httr GET write_disk status_code
#' @importFrom dplyr mutate filter
#' @importFrom stringr str_count
#' @export
topicsTutorialData <- function(
    sample_size = 1000,
    min_word_count = 250,
    max_word_count = 750,
    seed = 42,
    verbose = TRUE) {
  
  if (verbose) message(colourise("Downloading tutorial data from Hugging Face...", "blue"))
  
  url <- "https://huggingface.co/datasets/jingjietan/essays-big5/resolve/main/data/train-00000-of-00001.parquet"
  temp_file <- tempfile(fileext = ".parquet")
  
  # Download
  response <- httr::GET(url, httr::write_disk(temp_file, overwrite = TRUE))
  
  if (httr::status_code(response) != 200) {
    stop("Failed to download data. Check connection or URL.")
  }
  
  # Read
  df <- arrow::read_parquet(temp_file)
  if (file.exists(temp_file)) unlink(temp_file)
  
  if (verbose) message(colourise("Processing and filtering data...", "blue"))
  
  # Set seed for reproducibility
  set.seed(seed)
  
  # Process
  df_processed <- df %>%
    dplyr::mutate(
      WordCount = stringr::str_count(text, "\\S+") 
    ) %>%
    dplyr::filter(WordCount >= min_word_count, 
                  WordCount <= max_word_count)
  
  # Safety check for sample size
  actual_sample_size <- min(nrow(df_processed), sample_size)
  
  if (actual_sample_size < sample_size && verbose) {
    warning(paste("Only", actual_sample_size, "rows met the criteria. Returning all."))
  }
  
  df_processed <- df_processed[sample(nrow(df_processed), actual_sample_size), ]
  
  msg <- paste("Successfully loaded", nrow(df_processed), "rows.")
  if (verbose) message(colourise(msg, "blue"))
  
  return(df_processed)
}

library(testthat)

library(topics)


test_that("topicsTutorialData ", {
  

  testthat::skip_on_os(c("linux", "windows"))  # Skip on Ubuntu (Linux) and Windows
  testthat::skip_on_cran()
  
  df <- topics::topicsTutorialData()
  testthat::expect_true(tibble::is_tibble(df))
  
  testthat::expect_equal(
    colnames(df), 
    c("O", "C", "E", "A", "N", "ptype", "text", "__index_level_0__", "WordCount")
    )
  
   
})

---
title: "Installation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Installation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Installation

The `topics` package relies on **Java** for high-performance processing. To ensure a smooth installation, please follow these two steps.

### Step 1: Install Java (Required)
Before installing the R package, you must have the Java Runtime Environment (JRE) installed on your computer.
1. Download Java from [here](https://www.oracle.com/java/technologies/downloads/) or www.java.com/en/download/.
2. Restart your computer after installation to ensure R can detect the new system paths.

### Step 2: Install topics in R
Open R or RStudio and run the following commands.

```r
# Install and verify the Java interface
install.packages("topics")
```
You can also install the development version from GitHub.

```r

# Install devtools if not already present
if (!requireNamespace("devtools", quietly = TRUE)) install.packages("devtools")

# Install topics from GitHub
devtools::install_github("theharmonylab/topics")
```

### Step 3: Memory Configuration
Topic modeling is memory-intensive. You must increase the Java memory limit before loading the library. 
If you skip this, you may encounter OutOfMemoryError during model training.

```
# Set memory limit to 5GB (Run this BEFORE loading the library)
options(java.parameters = "-Xmx5000m")
library(topics)
```

---
title: "Getting started"
description: "Introduction to topics in R."
author: ""
opengraph:
  image: 
    src: "http://r-text.org/articles/text_files/figure-html/unnamed-chunk-5-1.png"
  twitter:
    card: summary_large_image
    creator: "@oscarkjell"
output: github_document #rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{text}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
evaluate = FALSE
```
The *topics*-package enables Differential Language Analysis using words, phrases and topics

Please reference our tutorial article when using the package: **Language visualisation methods for psychological assessments** and Ackermann L., Zhuojun G. & Kjell O.N.E. (2024). An R-package for visualizing text in topics. https://github.com/theharmonylab/topics. `DOI:zenodo.org/records/11165378`..

This Getting Started tutorial is going through the most central *topics* functions.  

## Usage
In an example where the topics are used to predict the PHQ-9 score, the pipeline can be run as follows:

**1. Data Preprocessing**<br>
To preprocess the data, run the following command:

```{r dtm, eval = FALSE, warning=TRUE, message=TRUE}

library(topics)

dtm <- topicsDtm(
  data = dep_wor_data$Depword)

# Check the results from the dtm and refine stopwords and removal rates if necessary
dtm_evaluation <- topicsDtmEval(
  dtm)
dtm_evaluation$frequency_plot
```

**2. Model Training**<br>
To train the LDA model, run the following command:

```{r model, eval = FALSE, warning=FALSE, message=FALSE}

model <- topicsModel(
  dtm = dtm,
  num_topics = 20,
  num_iterations = 1000)

```

**3. Model Inference**<br>
To infer the topic term distribution of the documents, run the following command:
```{r preds, eval = FALSE, warning=FALSE, message=FALSE}

preds <- topicsPreds(
  model = model,
  data = dep_wor_data$Depword)

```


**4. Statistical Analysis**<br>
To analyze the relationship between the topics and the prediction variable, run the following command:
```{r test, eval = FALSE, warning=FALSE, message=FALSE}

test <- topicsTest(
  data = dep_wor_data,
  model = model,
  preds = preds,
  x_variable = "PHQ9tot",
  controls = c("Age"),
  test_method = "linear_regression")

```

**5. Visualization**<br>
To visualize the significant topics as wordclouds, run the following command:
```{r plot_list, eval = FALSE, warning=FALSE, message=FALSE}

plot_list <- topicsPlot(
  model = model,
  test = test,
  figure_format = "png")

# showing some of the plots
plot_list$square1
```



### Articles using the topics-package
Differentiating balance and harmony through natural language analysis: A cross-national exploration of two understudied wellbeing-related concepts


### Other relevant references
The below list consists of papers analyzing human language in a similar fashion that is possible in *topics*.

***Methods Articles***

[Gaining insights from social media language: Methodologies and challenges.](http://www.peggykern.org/uploads/5/6/6/7/56678211/kern_2016_-_gaining_insights_from_social_media_language-_methodologies_and_challenges.pdf).  
*Kern et al., (2016). Psychological Methods.*


***Computer Science: Python Software***  

[DLATK: Differential language analysis toolkit.](https://aclanthology.org/D17-2010/)
*Schwartz, H. A., Giorgi, et al., (2017).  In Proceedings of the 2017 Conference on Empirical Methods in Natural Language Processing: System Demonstrations* 


[DLATK](https://github.com/dlatk/dlatk)

---
title: "Resources"
output: rmarkdown::html_document
vignette: >
  %\VignetteIndexEntry{resources}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Resources

## Article tutorials
- **Multiple Methods for Visualizing Human Language: A Tutorial for Social and Behavioural Scientists** 

  [OSF preprint](https://osf.io/preprints/psyarxiv/nxfvr_v1)
  

- **The *text* package tutorial**

  [Full text on PsycNET](https://doi.org/10.1037/met0000542).
  
  [OSF preprint](https://osf.io/preprints/psyarxiv/293kt_v1).
  
  [Tutorial code](https://r-text.org/articles/psychological_methods.html)

- **The Language-Based Assessment Model (L-BAM) Library** 

  [OSF preprint](https://osf.io/preprints/psyarxiv/n8rza_v1)
  
  [Tutorial code](https://r-text.org/articles/lbam_tutorial.html)


## Selected published articles advancing the *text* package

- Nilsson, A. H., Runge, J. M., Ganesan, A. V., Lövenstierne, C. V. N., Soni, N., & Kjell, O. N. (2025).  
  *Automatic implicit motive codings are at least as accurate as humans’ and 99% faster.*  
  *Journal of Personality and Social Psychology.*  
  [Article link](https://doi.org/10.1037/pspp0000544)

- Gu, Z., Kjell, K., Schwartz, H. A., & Kjell, O. (2025).  
  *Natural language response formats for assessing depression and worry with large language models: A sequential evaluation with model pre-registration.*  
  *Assessment, 10731911251364022.*  
  [Article link](https://journals.sagepub.com/doi/full/10.1177/10731911251364022)

- Nilsson, A. H., Eichstaedt, J. C., Lomas, T., Schwartz, A., & Kjell, O. (2024).  
  *The Cantril Ladder elicits thoughts about power and wealth.*  
  *Scientific Reports, 14(1), 2642.*  
  [Article link](https://www.nature.com/articles/s41598-024-52939-y)

## Blog posts and tutorials

- [**Implicit Motives Tutorial**](https://r-text.org/articles/implicit_motives_tutorial.html)  

- [**Installing and Managing Python Environments with `reticulate`**](https://r-text.org/articles/reticulate.html)

- [**Creating a Singularity Container to Run HuggingFace Transformers Models in R**](https://r-text.org/articles/singularity_transformers_container.html) 

- [**How to Best Manage Computationally Heavy Analyses**](https://r-text.org/articles/)  

- [**HuggingFace Language Models Are Downloaded in `.cache`**](https://r-text.org/articles/removing_huggingface_transformers_cache_files.html)  

- [**HuggingFace Transformers in R: Word Embeddings Defaults and Specifications**](https://r-text.org/articles/text.html)  

- [**Pre-registration and Researcher Degrees of Freedom**](https://r-text.org/articles/pre_registration_and_transformers.html)  

---
title: "Installation"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Installation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Installation

The `topics` package relies on **Java** for high-performance processing. To ensure a smooth installation, please follow these two steps.

### Step 1: Install Java (Required)
Before installing the R package, you must have the Java Runtime Environment (JRE) installed on your computer.
1. Download Java from [here](https://www.oracle.com/java/technologies/downloads/) or www.java.com/en/download/.
2. Restart your computer after installation to ensure R can detect the new system paths.

On Linux run: 
`sudo apt install libtirpc-dev libfreetype6-dev libpng-dev libtiff5-dev libjpeg-dev libharfbuzz-dev libfribidi-dev  libgit2-dev libcairo2-dev`

### Step 2: Install topics in R
Open R or RStudio and run the following commands.

```r
# Install and verify the Java interface
install.packages("topics")
```
You can also install the development version from GitHub.

```r

# Install devtools if not already present
if (!requireNamespace("devtools", quietly = TRUE)) install.packages("devtools")

# Install topics from GitHub
devtools::install_github("theharmonylab/topics")
```

### Step 3: Memory Configuration
Topic modeling is memory-intensive. You must increase the Java memory limit before loading the library. 
If you skip this, you may encounter OutOfMemoryError during model training.

```
# Set memory limit to 5GB (Run this BEFORE loading the library)
options(java.parameters = "-Xmx5000m")
library(topics)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/4_1_topicsPlot.R
\name{topicsGridLegend}
\alias{topicsGridLegend}
\title{Plot a grid (matrix) legend (available for the text-package)}
\usage{
topicsGridLegend(
  bivariate_color_codes = c("#398CF9", "#60A1F7", "#5dc688", "#e07f6a", "#EAEAEA",
    "#40DD52", "#FF0000", "#EA7467", "#85DB8E"),
  filtered_test,
  cor_var = "",
  save_dir,
  figure_format = "svg",
  seed = 42,
  width = 10,
  height = 8,
  y_axes_1 = 2,
  legend_title,
  legend_title_size,
  titles_color,
  legend_x_axes_label,
  legend_y_axes_label,
  topic_data_all,
  legend_number_color,
  legend_number_size,
  title_font = "sans"
)
}
\arguments{
\item{bivariate_color_codes}{A vector of color codes specifying the colors for the 3x3 grid legend.
Default: c("#398CF9", "#60A1F7", "#5dc688", "#e07f6a", "#EAEAEA", "#40DD52", "#FF0000", "#EA7467", "#85DB8E").}

\item{filtered_test}{A data frame containing the filtered topic data. Must include a `color_categories` column.}

\item{cor_var}{A string used to name the correlation variable, included in the file name of the saved plot. Default: "".}

\item{save_dir}{Directory where the grid legend plot will be saved. Default: "./results".}

\item{figure_format}{File format for the saved grid legend plot. Examples: "svg", "png", "pdf". Default: "svg".}

\item{seed}{Seed for reproducibility, ensuring consistent plot generation. Default: 42.}

\item{width}{Width of the saved grid legend in inches. Default: 10.}

\item{height}{Height of the saved grid legend in inches. Default: 8.}

\item{y_axes_1}{Specifies axis alignment for the grid legend. Options: 2 (2D grid with x and y axes) or 1 (1D legend for x-axis only). Default: 2.}

\item{legend_title}{Title text for the grid legend. Must be provided by the user.}

\item{legend_title_size}{Font size of the legend title text. Must be provided by the user.}

\item{titles_color}{Color of the title and axis labels in the legend. Must be provided by the user.}

\item{legend_x_axes_label}{Label for the x-axis of the grid legend. Must be provided by the user.}

\item{legend_y_axes_label}{Label for the y-axis of the grid legend. Must be provided by the user.}

\item{topic_data_all}{A data frame containing all topic data, including the `color_categories` column used for counting topics.}

\item{legend_number_color}{Color of the numeric annotations in the grid legend. Must be provided by the user.}

\item{legend_number_size}{Font size of the numeric annotations in the grid legend. Must be provided by the user.' @return A legend plot saved that can be combined with the plot object.}

\item{title_font}{Font family used for all non-word text elements in the plots (e.g., titles, axis labels, tick labels,
legend text, annotations). Default: "sans". Examples: "serif", "mono", or a system-installed font family name.}
}
\description{
Plot a grid (matrix) legend (available for the text-package)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_description.R
\docType{data}
\name{dep_wor_data}
\alias{dep_wor_data}
\title{Example data about mental health descirptions
.}
\format{
A data frame with 500 participants and 13 variables:
\describe{
  \item{Depselect}{Words that respondents have selected from a pre-defined list}
  \item{Worselect}{Words that respondents have selected from a pre-defined list}
  \item{Depword}{Wrods where respondents describe their experience with depression in life}
  \item{Worword}{Words where respondents describe their experience with depression in life}
  \item{Depphrase}{phrases where respondents describe their experience with depression in life}
  \item{Worphrase}{Phrases where respondents describe their experience with anxiety in life}
  \item{Deptext}{Text where respondents describe their experience with depression in life}
  \item{Wortext}{Text where respondents describe their experience with anxiety in life}
  \item{Gender}{Respondents gender 0=male, 1=female}
  \item{Age}{respondents age in years}
  \item{PHQ9tot}{total score of the respondents PHQ-9}
  \item{GAD7tot}{total score of the respondents GAD-7}
  \item{MentalSickDays}{self-reported sick days due to mental health issues}
  \item{MentalSickDaysYear}{self-reported sick days due to mental health issues in the last year}
  \item{HealthCareMental}{self-reported health care visits due to mental health}
  \item{SickDaysMonth}{self-reported sick days last month}
  \item{SickDaysYear}{self-reported sick days last year}
  \item{HealthCareVisits}{self-reported health care visits}
}
}
\source{
<https://osf.io/preprints/psyarxiv/p67db>
}
\usage{
dep_wor_data
}
\description{
Example data about mental health descirptions
.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/4_1_topicsPlot.R
\name{topicsPlot}
\alias{topicsPlot}
\title{Plot word clouds}
\usage{
topicsPlot(
  model = NULL,
  ngrams = NULL,
  test = NULL,
  p_alpha = 0.05,
  p_adjust_method = "none",
  ngrams_max = 30,
  ngram_select = "estimate",
  color_scheme = "default",
  word_font = "sans",
  title_font = "sans",
  overview_plot = TRUE,
  overview_n_topics = 9,
  highlight_topic_words = NULL,
  scale_size = FALSE,
  plot_topics_idx = NULL,
  allowed_word_overlap = NULL,
  plot_n_most_prevalent_topics = NULL,
  save_dir = NULL,
  figure_format = "svg",
  width = 6,
  height = 5,
  max_size = 10,
  seed = 42,
  scatter_legend_dot_size = c(3, 8),
  scatter_legend_bg_dot_size = c(1, 3),
  scatter_legend_dots_alpha = 0.8,
  scatter_legend_bg_dots_alpha = 0.2,
  scatter_legend_n = c(1, 1, 1, 1, 0, 1, 1, 1, 1),
  scatter_legend_method = c("mean"),
  scatter_legend_specified_topics = NULL,
  scatter_legend_topic_n = FALSE,
  scatter_show_axis_values = TRUE,
  scatter_legend_circles = FALSE,
  scatter_legend_circles_radius = 0,
  scatter_legend_circles_num = 4,
  grid_legend_title = "legend_title",
  grid_legend_title_size = 5,
  grid_legend_title_color = "black",
  grid_legend_x_axes_label = "legend_x_axes_label",
  grid_legend_y_axes_label = "legend_y_axes_label",
  grid_legend_number_color = "white",
  grid_legend_number_size = 15
)
}
\arguments{
\item{model}{(list) A trained topics model, e.g., from topicsModel(). Should be NULL if plotting ngrams.}

\item{ngrams}{(list) The output from the the topicsGram() function. Should be NULL if plotting topics.
Note 1: it is not possible to plot tags like <place>; so the < are replaced with underscore. 
Note 2: it is not possible to plot dash - alone, it is replaced with `_-_`.}

\item{test}{(list) The test results; if plotting according to dimension(s) include the object from topicsTest() function.}

\item{p_alpha}{(integer) The p-value threshold to use for significance testing.}

\item{p_adjust_method}{(character) Method to adjust/correct p-values for multiple comparisons (default = "none"; 
see also "holm", "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr").}

\item{ngrams_max}{(integer) The maximum number of n-grams to plot.}

\item{ngram_select}{(character) Method to select ngrams_max, when using both ngram and test use "prevalence" or "estimate"; 
if you only use ngrams use "pmi", "frequency", or "prevalence".}

\item{color_scheme}{(string 'default' or vector) The color scheme.
 
For plots not including a test, the color_scheme should in clude 2 colours (1 gradient pair), such as:

c("lightgray", "darkblue)

For 1 dimensional plots of n-grams it should contain 4 colours (2 gradient pairs), such as: 

c(
"#EAEAEA", "darkred", # negative ngrams colors

"#EAEAEA", "darkgreen" # positve ngrams colors)



For 1-dimension plots of topics, it should contain 6 colours (3 gradient pairs), such as 

 c(
"#EAEAEA", "darkred",     # negative topics colors

"#EAEAEA", "darkgray",     # colours of topics not significantly associated

"#EAEAEA", "darkgreen"     # positve topics colors)



 For 2-dimensional plots of topics, the color scheme should contain 18 colours (9 gradient pairs), such as:
 
 c(
  "lightgray", "#398CF9",     # quadrant 1 (upper left corner)
  
  "lightgray", "#60A1F7",     # quadrant 2 
  
  "lightgray", "#5dc688",     # quadrant 3 (upper right corner)
  
  "lightgray", "#e07f6a",     # quadrant 4
  
  "lightgray", "darkgray",     # quadrant 5 (middle square)
  
  "lightgray", "#40DD52",     # quadrant 6 
  
  "lightgray", "#FF0000",     # quadrant 7 (bottom left corner)
  
  "lightgray", "#EA7467",     # quadrant 8 
  
  "lightgray", "#85DB8E")     # quadrant 9 (bottom right corner).}

\item{word_font}{Font family used for the word text in the wordclouds (i.e., the plotted words only).
Default: "sans". Examples: "serif", "mono", or a system-installed font family name (e.g., "Arial", "Times New Roman").}

\item{title_font}{Font family used for all non-word text elements in the plots (e.g., titles, axis labels, tick labels,
legend text, annotations). Default: "sans". Examples: "serif", "mono", or a system-installed font family name.}

\item{overview_plot}{(boolean) Whether to produce a combined overview plot (`$overview_plot`)
that arranges selected topic word clouds and (where applicable) the distribution
legend into a single figure (experimental). The individual topic plots are also
returned separately in the result list, so users who prefer to assemble their own
figure in PowerPoint, Google Drawings, Illustrator, etc. can ignore the overview
and combine the individual plots manually. Default: TRUE.}

\item{overview_n_topics}{(integer) Used only by the **prevalent-topics** overview
(i.e., when `plot_n_most_prevalent_topics` is set and no `test` is provided).
Number of topic plots to include in the grid; plots are arranged in 3 columns
(e.g., the default of 9 produces a 3x3 grid). For test-based overviews
(1D and 2D differential plots), the number of topics shown is instead
controlled by `scatter_legend_n`. Default: 9.}

\item{highlight_topic_words}{(str vector) Words to highlight in topics (e.g., negative words). Format: highlight_topic_words = c("not", "never"). The default value is NULL.}

\item{scale_size}{(logical) Whether to scale the size of the words.}

\item{plot_topics_idx}{(vector)  The index or indices of the topics to plot 
(look in the model-object for the indices). They can, for example, be c(1, 3:5) to plot topic t_1, t_3, t_4 and t_5) (optional).}

\item{allowed_word_overlap}{(numeric) A filter function determining the maximum number of identical words in the topics to be plotted. 
This filter removes topics within each "color group" and also include removing topics from the distribution and grid legends; 
(Note that the adjustment for multiple comparison is taking place before these are removed; i.e., the adjusted p-values are not affected by this filter).}

\item{plot_n_most_prevalent_topics}{(numeric) Plots the n most prevalent topics in a given model.}

\item{save_dir}{(string) The directory to save the plots.}

\item{figure_format}{(string) Set the figure format, e.g., ".svg", or ".png".}

\item{width}{(integer) The width of the topic (units = "in").}

\item{height}{(integer) The width of the topic (units = "in").}

\item{max_size}{(integer) The maximum size of the words.}

\item{seed}{(integer) The seed to set for reproducibility.}

\item{scatter_legend_dot_size}{(integer) The size of dots in the scatter legend. If set to "prevalence", the size will change accordingly.}

\item{scatter_legend_bg_dot_size}{(integer) The size of background dots in the scatter legend.}

\item{scatter_legend_dots_alpha}{(numeric) The transparency alphe level of the dots.}

\item{scatter_legend_bg_dots_alpha}{(numeric) The transparency alphe level of the background dots.}

\item{scatter_legend_n}{(numeric or vector) A vector determining the number of dots to emphasize
in each quadrant of the scatter legend. For 1-dimensional plots this is a length-3 vector
`c(negative, non-significant, positive)`; for 2-dimensional plots it is a length-9 vector
(one value per quadrant). Example: `c(1,1,1,1,0,1,1,1,1)` highlights one dot in each
quadrant except the middle one.

These same counts also determine how many topic word clouds appear in the test-based
`$overview_plot` (1D or 2D), so the dots emphasized in the scatter legend and the
word clouds in the overview stay in sync. Setting larger values produces a denser
overview; setting a value to 0 omits that category entirely. If you want to build
your own figure layout in external software (PowerPoint, Google Drawings, etc.),
the individual topic plots are always returned separately in the result list — the
overview is only one possible composition.}

\item{scatter_legend_method}{(string) The method to filter topics to be emphasized in the scatter legend; either "mean", "max_x", or "max_y".}

\item{scatter_legend_specified_topics}{(vector) Specify which topic(s) to emphasize in the scatter legend.}

\item{scatter_legend_topic_n}{(boolean) If TRUE, the topic numbers are shown in the scatter legend.}

\item{scatter_show_axis_values}{(boolean) If TRUE, the estimate values are shown on the distribution plot axes.}

\item{scatter_legend_circles}{Plot concentric circles for the scatter legend}

\item{scatter_legend_circles_radius}{Radius of first concentric circle}

\item{scatter_legend_circles_num}{Number of Concentric circles
For example, c("t_1", "t_2"). If set, scatter_legend_method will have no effect.}

\item{grid_legend_title}{Title of the grid topic plot.}

\item{grid_legend_title_size}{Title size of the grid topic plot.}

\item{grid_legend_title_color}{Legend title color of the grid topic plot.}

\item{grid_legend_x_axes_label}{x-axis label of the grid topic plot.}

\item{grid_legend_y_axes_label}{y-axis label of the grid topic plot.}

\item{grid_legend_number_color}{Text color in the legend boxes of the grid topic plot.}

\item{grid_legend_number_size}{Text size in the legend boxes.}
}
\value{
The function provides a list of topic plots (if there are any significant topics), a legend plot, and a plot showing the topic distribution.
If save_dir is specified, it saves all plots in this directory. 
If you want to show all plots irrespective of the topics' significance, set p_alpha = 1.
}
\description{
This function create word clouds and topic figures
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_topicsDtm.R
\name{topicsDtmEval}
\alias{topicsDtmEval}
\title{Summarize and Visualize your Document Term Matrix}
\usage{
topicsDtmEval(dtm)
}
\arguments{
\item{dtm}{(R_obj) The document term matrix -> output of topicsDtm function}
}
\value{
A named list containing:
\describe{
  \item{dtm_summary}{A dataframe of terms and their frequencies.}
  \item{frequency_plot}{A bar plot of all term frequencies with example terms.}
  \item{frequency_plot_30_least}{A bar plot of the 30 least frequent terms (if numer of terms > 30).}
  \item{frequency_plot_30_most}{A bar plot of the 30 most frequent terms (if numer of terms > 30).}
  \item{histogram_of_frequencies}{A histogram of term frequencies (this is the same information as
  in the frequency_plot but presented differently).}
}
}
\description{
This function creates a frequency table of your DTM and generates up to four plots for visualization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_2_topicsGrams.R
\name{topicsGrams}
\alias{topicsGrams}
\title{N-grams}
\usage{
topicsGrams(
  data,
  ngram_window = c(1, 3),
  stopwords = stopwords::stopwords("en", source = "snowball"),
  removalword = "",
  pmi_threshold = NULL,
  occurance_rate = 0,
  removal_mode = "frequency",
  removal_rate_most = NULL,
  removal_rate_least = NULL,
  shuffle = TRUE,
  lower = TRUE,
  remove_punctuation = TRUE,
  remove_numbers = TRUE,
  stem_lemma_function = NULL,
  verbose = FALSE,
  seed = 42L,
  threads = 1,
  top_frequent = NULL,
  freq_per_user_format = c("auto", "wide", "long"),
  max_wide_cells = 5e+07
)
}
\arguments{
\item{data}{A character vector of texts or a data.frame/tibble (first column used as text).}

\item{ngram_window}{Integer vector specifying n-gram sizes.
If length 1, only that n is used (e.g., \code{2}).
If length 2 and increasing, it is interpreted as an inclusive range (e.g., \code{c(1, 3)} -> 1,2,3).
Otherwise it is treated as an explicit set (e.g., \code{c(1, 3)} means only 1 and 3).}

\item{stopwords}{Character vector of stopwords to remove (e.g.,
\code{stopwords::stopwords("en", source = "snowball")}).}

\item{removalword}{Character vector of words to remove from the raw text prior to tokenization.}

\item{pmi_threshold}{Numeric PMI threshold used to filter multi-word n-grams.
If \code{NULL}, PMI filtering is skipped.}

\item{occurance_rate}{Numeric in [0,1]. Terms are removed if they occur in fewer than
\code{round(N_docs * occurance_rate) - 1} documents, where \code{N_docs} is the number of texts.
Example: with 1000 documents and \code{occurance_rate = 0.05}, terms occurring in fewer than ~50 documents are removed.}

\item{removal_mode}{Character. Removal mode passed to \code{filter_ngrams()}.
Common options include \code{"frequency"} and \code{"percentage"} (depending on your helper implementation).}

\item{removal_rate_most}{Numeric. Rate/threshold for removing the most frequent n-grams
(interpreted by \code{filter_ngrams()} according to \code{removal_mode}).}

\item{removal_rate_least}{Numeric. Rate/threshold for removing the least frequent n-grams
(interpreted by \code{filter_ngrams()} according to \code{removal_mode}).}

\item{shuffle}{Logical. If \code{TRUE}, texts are shuffled (seed-controlled) before processing.
The original ids are preserved in output.}

\item{lower}{Logical. If \code{TRUE}, text is lowercased prior to tokenization.}

\item{remove_punctuation}{Logical. Passed to \code{quanteda::tokens(remove_punct = ...)}.}

\item{remove_numbers}{Logical. Passed to \code{quanteda::tokens(remove_numbers = ...)}.}

\item{stem_lemma_function}{Optional function applied to tokens (via \code{quanteda::tokens_apply()}).
Should accept and return a character vector of tokens.}

\item{verbose}{Logical. If \code{TRUE}, prints simple progress messages (if implemented).}

\item{seed}{Integer. Random seed used when \code{shuffle = TRUE}.}

\item{threads}{Integer. Number of threads used by \pkg{quanteda} via \code{quanteda_options(threads = ...)}.}

\item{top_frequent}{Integer or \code{NULL}. If set, keeps only the \code{top_frequent} most frequent n-grams
after filtering (recommended for large corpora). If \code{NULL}, keeps all retained n-grams.}

\item{freq_per_user_format}{Output format for \code{freq_per_user}:
\describe{
  \item{\code{"wide"}}{Wide document-by-ngram table (may be large).}
  \item{\code{"long"}}{Long sparse table with columns \code{id}, \code{ngram}, \code{rel_freq}.}
  \item{\code{"auto"}}{Uses wide format only if \code{N_docs * N_ngrams <= max_wide_cells}; otherwise returns long.}
}}

\item{max_wide_cells}{Numeric. Safety limit for \code{"auto"}: if \code{N_docs * N_ngrams} exceeds this,
\code{freq_per_user} is returned in long format to avoid dense allocation and excessive memory use.}
}
\value{
A named list with:
\describe{
  \item{settings}{A named list of settings used to generate the results (for reproducibility).}
  \item{n_grams_pmi}{PMI information if available from your PMI helper, otherwise a message.}
  \item{ngrams}{A tibble of retained n-grams and statistics (e.g., \code{freq}, \code{prevalence}, \code{num_docs}).}
  \item{freq_per_user}{Per-document relative frequencies, either wide or long depending on \code{freq_per_user_format}.}
  \item{stats}{A tibble summarizing how many n-grams were removed by each filtering step, by n-gram size.}
}
}
\description{
Compute n-grams and per-document relative frequencies from text.
}
\details{
This function extracts n-grams using  and returns:
(1) a tibble of n-grams with frequency/prevalence and document frequency,
(2) per-document relative frequencies for the retained n-grams (wide or long format),
(3) filtering statistics,
(4) a named list of settings (for reproducibility).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3_topicTest.R
\name{topicsTest}
\alias{topicsTest}
\title{Test topics or n-grams}
\usage{
topicsTest(
  data,
  model = NULL,
  preds = NULL,
  ngrams = NULL,
  x_variable = NULL,
  y_variable = NULL,
  controls = c(),
  test_method = "default",
  p_adjust_method = "fdr",
  complete_cases = FALSE,
  seed = 42
)
}
\arguments{
\item{data}{(tibble) The tibble containing the variables to be tested.}

\item{model}{(list) A trained model LDA-model from the topicsModel() function.}

\item{preds}{(tibble) The predictions from the topicsPred() function.}

\item{ngrams}{(list) Output of the n-gram function.}

\item{x_variable}{(string) The x variable name to be predicted, and to be plotted (only needed for regression or correlation).}

\item{y_variable}{(string) The y variable name to be predicted, and to be plotted (only needed for regression or correlation).}

\item{controls}{(vector) The control variables (not supported yet).}

\item{test_method}{(string) The test method to use. "default" checks if x_variable and y_variable
only contain 0s and 1s, for which it applies logistic regression; otherwise it applies linear regression. 
Alternatively, if only specifying x_variable, the user may manually specify either "linear_regression" or "logistic_regression".}

\item{p_adjust_method}{(character) Method to adjust/correct p-values for multiple comparisons
(default = "fdr"; see also "holm", "hochberg", "hommel", "bonferroni", "BH", "BY",  "none").}

\item{complete_cases}{If TRUE, it will only use complete cases for x_variable, y_variable, controls, and preds.}

\item{seed}{(integer) The seed to set for reproducibility}
}
\value{
A list of the test results, test method, and prediction variable.
}
\description{
Statistically test topics or n-grams in relation to one or two other variables using 
regression or t-test.
}
\examples{
\donttest{
# Test the topic document distribution in respect to a variable

dtm <- topicsDtm(
  data = dep_wor_data$Depphrase)

model <- topicsModel(
  dtm = dtm, # output of topicsDtm()
  num_topics = 20,
  num_top_words = 10,
  num_iterations = 1000,
  seed = 42)
                     
preds <- topicsPreds(
 model = model, # output of topicsModel()
 data = dep_wor_data$Depphrase)
                     
test <- topicsTest(
  model = model, # output of topicsModel()
  data=dep_wor_data,
  preds = preds, # output of topicsPreds()
  test_method = "linear_regression",
  x_variable = "Age")
}                 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_download.R
\name{topicsTutorialData}
\alias{topicsTutorialData}
\title{Download and Prepare Tutorial Data}
\usage{
topicsTutorialData(
  sample_size = 1000,
  min_word_count = 250,
  max_word_count = 750,
  seed = 42,
  verbose = TRUE
)
}
\arguments{
\item{sample_size}{(integer) The number of rows to return. Default is 1000.}

\item{min_word_count}{(integer) Minimum words per essay. Default is 250.}

\item{max_word_count}{(integer) Maximum words per essay. Default is 750.}

\item{seed}{(integer) Seed for reproducibility. Default is 42.}

\item{verbose}{(boolean) Whether to print status messages. Default is TRUE.}
}
\value{
A processed tibble ready for topic modeling.
}
\description{
Downloads the Big5 Essays dataset from Hugging Face, filters by word count,
and returns a sample of a specified size.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/4_1_topicsPlot.R
\name{topicsScatterLegend}
\alias{topicsScatterLegend}
\title{Plot a distribution plot (available for the text-package)}
\usage{
topicsScatterLegend(
  bivariate_color_codes,
  filtered_test,
  num_popout = 1,
  way_popout_topics = "mean",
  user_spec_topics = NULL,
  allow_topic_num_legend = FALSE,
  scatter_show_axis_values = TRUE,
  y_axes_1 = 2,
  cor_var = "",
  label_x_name = "x",
  label_y_name = "y",
  save_dir,
  figure_format = "svg",
  scatter_popout_dot_size = c(1, 5),
  scatter_bg_dot_size = c(1, 5),
  scatter_legend_dots_alpha = 0.8,
  scatter_legend_bg_dots_alpha = 0.2,
  scatter_legend_circles = FALSE,
  scatter_legend_circles_radius = 0,
  scatter_legend_circles_num = 4,
  width = 10,
  height = 8,
  seed = 42,
  title_font = "sans"
)
}
\arguments{
\item{bivariate_color_codes}{A vector of color codes specifying colors for 
different categories in the scatter plot. 
Default: c("#398CF9", "#60A1F7", "#5dc688", "#e07f6a", "#EAEAEA", "#40DD52", "#FF0000", "#EA7467", "#85DB8E").}

\item{filtered_test}{A data frame containing the input data for the scatter plot. 
Must include columns like `color_categories` and other variables used in the function.}

\item{num_popout}{The number of topics to "pop out" in each category. Default: 1.
Can be a single integer (applies to all categories) or a vector for specific categories.}

\item{way_popout_topics}{The method for selecting pop-out topics. Options: "mean", "max_y", or "max_x". Default: "mean".}

\item{user_spec_topics}{A vector of user-specified topics to highlight in the scatter plot. Default: NULL.}

\item{allow_topic_num_legend}{Logical; if TRUE, displays topic numbers in the legend. Default: FALSE.}

\item{scatter_show_axis_values}{Show values on the axises.}

\item{y_axes_1}{Specifies axis alignment for the scatter legend. Options: 1 (x-axis) or 2 (y-axis). Default: 2.}

\item{cor_var}{A string used for naming the correlation variable in labels or file names. Default: "".}

\item{label_x_name}{Label for the x-axis in the scatter plot. Default: "x".}

\item{label_y_name}{Label for the y-axis in the scatter plot. Default: "y".}

\item{save_dir}{Directory where the scatter legend plot will be saved. Default: "./results".}

\item{figure_format}{File format for the saved scatter plot. Examples: "svg", "png", "pdf". Default: "svg".}

\item{scatter_popout_dot_size}{Size of the dots for pop-out topics in the scatter legend. Set to "prevalence" for dot size changing based on topic prevalence. Default: 15.}

\item{scatter_bg_dot_size}{Size of the dots for background topics in the scatter legend. Default: 9.}

\item{scatter_legend_dots_alpha}{The transparency of the dots}

\item{scatter_legend_bg_dots_alpha}{The transparency of the dots}

\item{scatter_legend_circles}{Plot concentric circles for the scatter legend}

\item{scatter_legend_circles_radius}{Radius of first concentric circle}

\item{scatter_legend_circles_num}{Number of Concentric circles}

\item{width}{Width of the saved scatter plot in inches. Default: 10.}

\item{height}{Height of the saved scatter plot in inches. Default: 8.}

\item{seed}{Seed for reproducibility, ensuring consistent plot generation. Default: 42.}

\item{title_font}{Font family used for all non-word text elements in the plots (e.g., titles, axis labels, tick labels,
legend text, annotations). Default: "sans". Examples: "serif", "mono", or a system-installed font family name.}
}
\description{
Plot a distribution plot (available for the text-package)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_1_1_topicsModel_Pred.R
\name{topicsModel}
\alias{topicsModel}
\title{Topic modelling}
\usage{
topicsModel(
  dtm,
  num_topics = 20,
  num_top_words = 10,
  num_iterations = 1000,
  seed = 42,
  matrix_size_check = TRUE
)
}
\arguments{
\item{dtm}{(R_obj) The document term matrix -> output of topicsDtm function}

\item{num_topics}{(integer) The number of topics to be created}

\item{num_top_words}{(integer) The number of top words to be displayed}

\item{num_iterations}{(integer) The number of iterations to run the model}

\item{seed}{(integer) A seed to set for reproducibility}

\item{matrix_size_check}{(boolean) If TRUE, checks whether the vocabulary 
size and Java heap memory are sufficient before fitting the model. If the 
vocabulary is too large or the heap too small, the function stops early with 
advice on how to fix the issue - preventing R from crashing. Set to FALSE to 
skip this check and attempt to fit the model regardless, though this may crash 
your R session if memory is insufficient.}
}
\value{
A named list containing the following elements:
\describe{
  \item{name}{Description}
  \item{instances}{Java object reference: A list of all documents used for topic modeling, 
  in which each document is preprocessed (e.g., tokenized and vectorized). This object is part of 
  the Mallet package's internal structure.}
  \item{inferencer}{Java object reference: This is the topic inferencer, which allows the inference 
  of topic distributions for new, unseen documents based on the trained model.}
  \item{top_terms_mallet}{A data frame containing the top terms of each topic, showing which concepts
   each topic likely represents. The number of top terms shown here can be adjusted with the argument
    num_top_words.}
  \item{top_terms}{A data frame containing the top terms of each topic, showing which concepts each 
  topic likely represents. The number of top terms shown here can be adjusted with the argument 
  num_top_words.}
  \item{phi}{A matrix of the topic-word distribution: Each row represents a topic, and each column 
  represents a word from the document term matrix. The values show the probability of a word given a 
  topic P(word|topic).}
  \item{topic_docs}{A matrix of document-topic distribution: Each row represents a document, and each 
  column represents a topic. The values show the probability of a topic given a document, P(topic|document).}
  \item{frequencies}{A data frame of term frequencies. word = every word in the document term matrix, 
  word.freq = the frequency of each word across all documents, doc.freq = the number of documents in which each word appears.}
  \item{vocabulary}{A character vector of all unique terms in the document term matrix.}
  \item{labels}{A list of topic labels. These short labels are the most representative term for each topic, 
  making it easy to identify and understand them.}
  \item{theta}{A data frame of document-topic probabilities: each row represents a document, and each column
   represents a topic. Similar to topic_docs, this shows the contribution of each topic to each document. 
   Each row sums to 1, representing the document's composition of topics.}
  \item{prevalence}{A numeric vector showing the overall prevalence (prominence) of each topic in the corpus.
   The prevalences are expressed as percentages relative to the other topics  and add up to 100%. 
   Higher values indicate topics that are present in more documents.}
  \item{coherence}{A numeric vector showing the coherence of each topic. Coherence scores indicate how
   semantically consistent and interpretable the topics are. Higher coherence generally indicates 
   better-quality topics.}
  \item{pred_model}{A list containing components of the predictive model, including phi
   (word-topic probability matrix), theta (document-topic probabilities matrix), alpha (Dirichlet prior of topics), 
   gamma (hyperparameters of word-topic assignments), and data (sparse matrix representing the document term matrix.)}
  \item{dtm_settings}{A list of settings used for preprocessing and building the document term matrix (dtm), 
  including n-gram ranges, stopword removal, frequency thresholds, and random seed settings.}
  \item{summary}{A summary data frame comprising of the topic numbers, labels, coherence scores, prevalence scores, and top terms.}
 }
}
\description{
The function to create and train and an LDA model.
}
\examples{
\donttest{
# Create LDA Topic Model 
save_dir_temp <- tempfile()
dtm <- topicsDtm(data = dep_wor_data$Depphrase)

model <- topicsModel(
dtm = dtm, # output of topicsDtm()
num_topics = 20,
num_top_words = 10,
num_iterations = 1000,
seed = 42)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/4_1_topicsPlot.R
\name{topicsPlotOverview}
\alias{topicsPlotOverview}
\title{Combine topics and distribution legend into an overview figure (experimental)}
\usage{
topicsPlotOverview(
  plot_list,
  overview_plot_type,
  overview_n_topics = 9,
  scatter_legend_n = NULL,
  title_font = "sans"
)
}
\arguments{
\item{plot_list}{(list) Output from the `topicsPlot()` function.}

\item{overview_plot_type}{(character) One of `"topics_grid"`, `"textTopics"`,
`"one_dimension_topics"`, or `"two_dimension_topics"`.}

\item{overview_n_topics}{(integer) For the `"topics_grid"` / `"textTopics"`
overview only: the number of topic plots to include. Plots are arranged
in 3 columns (the default of 9 produces a 3x3 grid). Ignored for the
test-based 1D and 2D overviews, which instead use `scatter_legend_n`.
Default: 9.}

\item{scatter_legend_n}{(numeric vector) For the test-based 1D and 2D
overviews: the per-category counts that determine how many topic plots
appear in the overview. Length 3 for 1D (negative, non-significant,
positive) and length 9 for 2D (one value per quadrant). A value of 0
omits that category. Ignored for the prevalent-topics overview.}

\item{title_font}{Font family used for all non-word text elements in the plots
(e.g., titles, axis labels, tick labels, legend text, annotations).
Default: `"sans"`.}
}
\value{
A patchwork composition that can be printed, saved with `ggsave()`,
  or further modified with patchwork operators.
}
\description{
Assembles a single composite figure from the individual topic word clouds
returned by `topicsPlot()`. The shape of the composition depends on
`overview_plot_type`:
\itemize{
  \item `"topics_grid"` / `"textTopics"`: a grid of the most prevalent topics
    (no test), controlled by `overview_n_topics`.
  \item `"one_dimension_topics"`: three columns (negative / non-significant /
    positive), controlled by `scatter_legend_n` (length 3).
  \item `"two_dimension_topics"`: a 3x3 quadrant layout with the distribution
    legend in the centre, controlled by `scatter_legend_n` (length 9).
}
}
\details{
This function is intended as a convenience for quickly inspecting results.
For publication figures, users typically save the individual topic plots
(returned separately by `topicsPlot()`) and assemble them in external
software such as PowerPoint, Google Drawings, or Illustrator.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1_topicsDtm.R
\name{topicsDtm}
\alias{topicsDtm}
\title{Document Term Matrix}
\usage{
topicsDtm(
  data,
  ngram_window = c(1, 3),
  stopwords = stopwords::stopwords("en", source = "snowball"),
  removalword = "",
  pmi_threshold = NULL,
  occurance_rate = 0,
  removal_mode = "percentage",
  removal_rate_most = 0,
  removal_rate_least = 0,
  shuffle = TRUE,
  lower = TRUE,
  remove_punctuation = TRUE,
  remove_numbers = TRUE,
  stem_lemma_function = NULL,
  verbose = FALSE,
  seed = 42L,
  threads = 1
)
}
\arguments{
\item{data}{(list) A list containing the text data with each entry belonging to a unique id}

\item{ngram_window}{(list) The minimum and maximum n-gram length, e.g., c(1,3)}

\item{stopwords}{(stopwords) The stopwords to remove, e.g., stopwords::stopwords("en", source = "snowball")}

\item{removalword}{(string) Character vector of words to remove; e.g., "word1" or c("word1", "word2", ...)}

\item{pmi_threshold}{(integer; experimental) Pointwise Mutual Information (PMI) measures the association 
between terms by comparing their co-occurrence probability to their individual probabilities, 
highlighting term pairs that occur together more often than expected by chance; in this implementation,
terms with average PMI below the specified threshold (pmi_threshold) are removed from the document-term matrix.}

\item{occurance_rate}{(numerical) The occurance rate (0-1) removes words that occur less then in (occurance_rate)*(number of documents). Example: If the training dataset has 1000 documents and the occurrence rate is set to 0.05, the code will remove terms that appear in less than 49 documents.}

\item{removal_mode}{(string) Mode of removal -> one of c("none", "frequency", "term", "percentage"). frequency removes all words under a certain frequency or over a certain frequency, as indicated by removal_rate_least and removal_rate_most. term removes an absolute number of terms that are most frequent and least frequent. percentage removes the number of terms indicated by removal_rate_least and removal_rate_most relative to the number of terms in the matrix}

\item{removal_rate_most}{(integer) The rate of most frequent words to be removed, functionality depends on removal_mode}

\item{removal_rate_least}{(integer) The rate of least frequent words to be removed, functionality depends on removal_mode}

\item{shuffle}{(boolean) Shuffle the data before analyses}

\item{lower}{(boolean) If TRUE, converts all text to lowercase before embedding.}

\item{remove_punctuation}{(boolean) If TRUE, replaces non-alphanumeric characters with spaces.}

\item{remove_numbers}{(boolean) If TRUE, replaces all numerical digits with spaces.}

\item{stem_lemma_function}{(function). A custom function for stemming or lemmatization.}

\item{verbose}{Logical. If TRUE, displays progress bars and status updates during 
text vectorization and the embedding process. Defaults to FALSE.}

\item{seed}{(integer) A seed to set for reproducibility}

\item{threads}{(integer) The number of threads to use; also called cpu in (CreateDtm).}
}
\value{
The document term matrix
}
\description{
This function creates a document term matrix
}
\examples{
\donttest{

# Create a Dtm and remove the terms that occur less than 4 times and more than 500 times.

dtm <- topicsDtm(data = dep_wor_data$Depphrase,
                 removal_mode = "frequency",
                 removal_rate_least = 4,
                 removal_rate_most = 500)

# Create Dtm and remove the 1 least and 1 most frequent terms.
dtm <- topicsDtm(data = dep_wor_data$Depphrase,
                 removal_mode = "term",
                 removal_rate_least = 1,
                 removal_rate_most = 1)

# Create Dtm and remove the 1\% least frequent and 1\% most frequent terms. 
# The percentage values are scaled to values between 0 and 1.
dtm <- topicsDtm(data = dep_wor_data$Depphrase,
                 removal_mode = "percentage",
                 removal_rate_least = 0.01,
                 removal_rate_most = 0.01)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2_1_1_topicsModel_Pred.R
\name{topicsPredict}
\alias{topicsPredict}
\alias{topicsAssess}
\alias{topicsClassify}
\alias{topicsPreds}
\title{topicsPredict, topicsPreds, topicsAssess and topicsClassify}
\usage{
topicsPredict(
  model,
  data,
  num_iterations = 200,
  sampling_interval = 10,
  burn_in = 10,
  seed = 42,
  create_new_dtm = FALSE
)

topicsAssess(
  model,
  data,
  num_iterations = 200,
  sampling_interval = 10,
  burn_in = 10,
  seed = 42,
  create_new_dtm = FALSE
)

topicsClassify(
  model,
  data,
  num_iterations = 200,
  sampling_interval = 10,
  burn_in = 10,
  seed = 42,
  create_new_dtm = FALSE
)

topicsPreds(
  model,
  data,
  num_iterations = 200,
  sampling_interval = 10,
  burn_in = 10,
  seed = 42,
  create_new_dtm = FALSE
)
}
\arguments{
\item{model}{(list) The trained model.}

\item{data}{(tibble) The text variable for which you want to infer the topic distribution. This can be the same data as used to create the dtm or new data.}

\item{num_iterations}{(integer) The number of iterations to run the model.}

\item{sampling_interval}{The number of iterations between consecutive samples collected.
during the Gibbs Sampling process. This technique, known as thinning, helps reduce the 
correlation between consecutive samples and improves the quality of the final estimates 
by ensuring they are more independent.
Purpose: By specifying a sampling_interval, you avoid collecting highly correlated samples, which can 
lead to more robust and accurate topic distributions.
Example: If sampling_interval = 10, the algorithm collects a 
sample every 10 iterations (e.g., at iteration 10, 20, 30, etc.).
Typical Values: Default: 10; Range: 5 to 50 (depending on the complexity and size of the data).}

\item{burn_in}{The number of initial iterations discarded during the Gibbs Sampling process. 
These early iterations may not be representative of the final sampling distribution because the model is still stabilizing.
Purpose: The burn_in period allows the model to converge to a more stable state before collecting samples, 
improving the quality of the inferred topic distributions.
Example: If burn_in = 50, the first 50 iterations of the Gibbs Sampling process are discarded,
and sampling begins afterward. Typical Values: Default: 50 to 100 
Range: 10 to 1000 (larger datasets or more complex models may require a longer burn-in period).}

\item{seed}{(integer) A seed to set for reproducibility.}

\item{create_new_dtm}{(boolean) If applying the model on new data (not used in training), it can help to make a new dtm.
Currently this is experimental, and using the textmineR::CreateDtm() function rather than the topicsDtm() function, which has more functions.}
}
\value{
A tibble of the predictions: The rows represent the documents, and the columns represent the topics. The values in the cells indicate the proportion of each topic within the corresponding document.
}
\description{
The function to predict the topics of a new document with the trained model.
}
\examples{
\donttest{
# Predict topics for new data with the trained model

dtm <- topicsDtm(
data = dep_wor_data$Depphrase)

model <- topicsModel(dtm = dtm, # output of topicsDtm()
                     num_topics = 20,
                     num_top_words = 10,
                     num_iterations = 1000,
                     seed = 42)
                     
preds <- topicsPredict(model = model, # output of topicsModel()
                     data = dep_wor_data$Depphrase)
}
}
